@@ -90,7 +90,7 @@ class CreatePageAliasAction:
             and not parent.permissions_for_user(self.user).can_publish_subpage()
         ):
             raise CreatePageAliasPermissionError(
-                "You do not have permission to publish a page at the destination"
+                "You do not have permission to publish a page at the destination."
             )
 
     def _create_alias(
@@ -106,7 +106,6 @@ class CreatePageAliasAction:
         reset_translation_key,
         _mpnode_attrs,
     ):
-
         specific_page = page.specific
 
         # FIXME: Switch to the same fields that are excluded from copy
@@ -121,6 +120,7 @@ class CreatePageAliasAction:
             "path",
             "index_entries",
             "postgres_index_entries",
+            "latest_revision",  # for page aliases do not have revisions
         ]
 
         update_attrs = {
@@ -149,7 +149,7 @@ class CreatePageAliasAction:
         )
 
         # Update any translatable child objects
-        for (child_relation, old_pk), child_object in child_object_map.items():
+        for child_object in child_object_map.values():
             if isinstance(child_object, TranslatableMixin):
                 if update_locale:
                     child_object.locale = update_locale
@@ -199,25 +199,18 @@ class CreatePageAliasAction:
                     else None,
                 },
             )
-            if alias.live:
-                # Log the publish
-                log(
-                    instance=alias,
-                    action="wagtail.publish",
-                    user=user,
-                )
 
         logger.info(
             'Page alias created: "%s" id=%d from=%d', alias.title, alias.id, page.id
         )
 
+        from wagtail.models import Page, PageViewRestriction
+
         # Copy child pages
         if recursive:
-            from wagtail.models import Page
-
             numchild = 0
 
-            for child_page in page.get_children().specific():
+            for child_page in page.get_children().specific().iterator():
                 newdepth = _mpnode_attrs[1] + 1
                 child_mpnode_attrs = (
                     Page._get_path(_mpnode_attrs[0], newdepth, numchild),
@@ -240,6 +233,23 @@ class CreatePageAliasAction:
                 alias.numchild = numchild
                 alias.save(clean=False, update_fields=["numchild"])
 
+        # Copy across any view restrictions defined directly on the page,
+        # unless the destination page already has view restrictions defined
+        if parent:
+            parent_page_restriction = parent.get_view_restrictions()
+        else:
+            parent_page_restriction = page.get_parent().get_view_restrictions()
+
+        if not parent_page_restriction.exists():
+            for view_restriction in page.view_restrictions.all():
+                view_restriction_copy = PageViewRestriction(
+                    restriction_type=view_restriction.restriction_type,
+                    password=view_restriction.password,
+                    page=alias,
+                )
+                view_restriction_copy.save(user=self.user)
+                view_restriction_copy.groups.set(view_restriction.groups.all())
+
         return alias
 
     def execute(self, skip_permission_checks=False):
