@@ -1,7 +1,7 @@
 from django.apps import AppConfig
 from django.utils.translation import gettext_lazy as _
 
-from . import checks  # NOQA
+from . import checks  # NOQA: F401
 
 
 class WagtailAdminAppConfig(AppConfig):
@@ -14,3 +14,5 @@ class WagtailAdminAppConfig(AppConfig):
         from wagtail.admin.signal_handlers import register_signal_handlers
 
         register_signal_handlers()
+
+        from wagtail.admin.telepath import widgets  # noqa: F401
