@@ -8,7 +8,7 @@ def css_install_check(app_configs, **kwargs):
     errors = []
 
     css_path = os.path.join(
-        os.path.dirname(__file__), "static", "wagtailadmin", "css", "normalize.css"
+        os.path.dirname(__file__), "static", "wagtailadmin", "css", "core.css"
     )
 
     if not os.path.isfile(css_path):
@@ -152,7 +152,7 @@ There are no tabs on non-Page model editing within InlinePanels.""".format(
                 class_name, panel_name
             )
         else:
-            error_hint = """Ensure that {} uses `panels` instead of `{}`\
+            error_hint = """Ensure that {} uses `panels` instead of `{}` \
 or set up an `edit_handler` if you want a tabbed editing interface.
 There are no default tabs on non-Page models so there will be no \
 {} tab for the {} to render in.""".format(
@@ -177,9 +177,103 @@ def wagtail_admin_base_url_check(app_configs, **kwargs):
             Warning(
                 "The WAGTAILADMIN_BASE_URL setting is not defined",
                 hint="This should be the base URL used to access the Wagtail admin site. "
-                "Without this, URLs in notification emails will not display correctly.",
+                "Without this, admin URLs outside of the admin (e.g. notification "
+                "emails and the user bar) will not display correctly.",
                 id="wagtailadmin.W003",
             )
         )
 
     return errors
+
+
+@register("file_overwrite")
+def file_overwrite_check(app_configs, **kwargs):
+    from django import VERSION as DJANGO_VERSION
+    from django.conf import settings
+
+    if DJANGO_VERSION >= (5, 1):
+        file_storage = settings.STORAGES["default"]["BACKEND"]
+    else:
+        try:
+            file_storage = settings.STORAGES["default"]["BACKEND"]
+        except AttributeError:
+            file_storage = getattr(settings, "DEFAULT_FILE_STORAGE", None)
+
+    errors = []
+
+    if file_storage == "storages.backends.s3boto3.S3Boto3Storage" and getattr(
+        settings, "AWS_S3_FILE_OVERWRITE", True
+    ):
+        errors.append(
+            Warning(
+                "The AWS_S3_FILE_OVERWRITE setting is set to True",
+                hint="This should be set to False. The incorrect setting can cause documents and "
+                "other user-uploaded files to be silently overwritten or deleted.",
+                id="wagtailadmin.W004",
+            )
+        )
+    if file_storage == "storages.backends.azure_storage.AzureStorage" and getattr(
+        settings, "AZURE_OVERWRITE_FILES", False
+    ):
+        errors.append(
+            Warning(
+                "The AZURE_OVERWRITE_FILES setting is set to True",
+                hint="This should be set to False. The incorrect setting can cause documents and "
+                "other user-uploaded files to be silently overwritten or deleted.",
+                id="wagtailadmin.W004",
+            )
+        )
+    if file_storage == "storages.backends.gcloud.GoogleCloudStorage" and getattr(
+        settings, "GS_FILE_OVERWRITE", True
+    ):
+        errors.append(
+            Warning(
+                "The GS_FILE_OVERWRITE setting is set to True",
+                hint="This should be set to False. The incorrect setting can cause documents and "
+                "other user-uploaded files to be silently overwritten or deleted.",
+                id="wagtailadmin.W004",
+            )
+        )
+
+    return errors
+
+
+@register("datetime_format")
+def datetime_format_check(app_configs, **kwargs):
+    """
+    If L10N is enabled, check if WAGTAIL_* formats are compatible with Django input formats.
+    See https://docs.djangoproject.com/en/stable/topics/i18n/formatting/#creating-custom-format-files
+    See https://docs.wagtail.org/en/stable/reference/settings.html#wagtail-date-format-wagtail-datetime-format-wagtail-time-format
+    """
+
+    from django.conf import settings
+    from django.utils import formats
+
+    errors = []
+
+    if not getattr(settings, "USE_L10N", False):
+        return errors
+
+    for code, label in settings.LANGUAGES:
+        for wagtail_setting, django_setting in [
+            ("WAGTAIL_DATE_FORMAT", "DATE_INPUT_FORMATS"),
+            ("WAGTAIL_DATETIME_FORMAT", "DATETIME_INPUT_FORMATS"),
+            ("WAGTAIL_TIME_FORMAT", "TIME_INPUT_FORMATS"),
+        ]:
+            wagtail_format_value = getattr(settings, wagtail_setting, None)
+            if wagtail_format_value is None:
+                # Skip the iteration if wagtail_format is not present
+                continue
+
+            input_formats = formats.get_format(django_setting, lang=code)
+            if wagtail_format_value not in input_formats:
+                errors.append(
+                    Error(
+                        "Configuration error",
+                        hint=f"'{wagtail_format_value}' must be in {django_setting} for language {label} ({code}).",
+                        obj=wagtail_setting,
+                        id="wagtailadmin.E003",
+                    )
+                )
+
+    return errors
