@@ -18,7 +18,7 @@ class DateRangePickerWidget(SuffixedMultiWidget):
     A widget allowing a start and end date to be picked.
     """
 
-    template_name = "wagtailadmin/widgets/daterange_input.html"
+    template_name = "wagtailadmin/widgets/range_input.html"
     suffixes = ["from", "to"]
 
     def __init__(self, attrs=None):
@@ -34,6 +34,27 @@ class DateRangePickerWidget(SuffixedMultiWidget):
         return [None, None]
 
 
+class NumberRangeWidget(SuffixedMultiWidget):
+    """
+    A widget allowing a minimum and maximum usage count to be picked.
+    """
+
+    template_name = "wagtailadmin/widgets/range_input.html"
+    suffixes = ["min", "max"]
+
+    def __init__(self, attrs=None):
+        widgets = (
+            forms.NumberInput(attrs={"placeholder": _("Minimum"), "min": "0"}),
+            forms.NumberInput(attrs={"placeholder": _("Maximum"), "min": "0"}),
+        )
+        super().__init__(widgets, attrs)
+
+    def decompress(self, value):
+        if value:
+            return [value.start, value.stop]
+        return [None, None]
+
+
 class FilteredModelChoiceIterator(django_filters.fields.ModelChoiceIterator):
     """
     A variant of Django's ModelChoiceIterator that, instead of yielding (value, label) tuples,
@@ -221,10 +242,10 @@ class CollectionFilter(django_filters.ModelChoiceFilter):
     field_class = CollectionChoiceField
 
 
-class PopularTagsFilter(django_filters.MultipleChoiceFilter):
-    # This uses a MultipleChoiceFilter instead of a ModelMultipleChoiceFilter
-    # because the queryset has been sliced, which means ModelMultipleChoiceFilter
-    # cannot do further queries to validate the selected tags.
+class RelatedFilterMixin:
+    # Workaround for https://github.com/wagtail/wagtail/issues/6616 by changing
+    # a filter on a related field into a filter on the primary key of instances
+    # that match the filter value.
 
     def __init__(self, *args, use_subquery=False, **kwargs):
         super().__init__(*args, **kwargs)
@@ -234,15 +255,26 @@ class PopularTagsFilter(django_filters.MultipleChoiceFilter):
         filtered = super().filter(qs, value)
         if not self.use_subquery or not value:
             return filtered
-
-        # Workaround for https://github.com/wagtail/wagtail/issues/6616
         pks = list(filtered.values_list("pk", flat=True))
         return qs.filter(pk__in=pks)
 
 
+class PopularTagsFilter(RelatedFilterMixin, django_filters.MultipleChoiceFilter):
+    # This uses a MultipleChoiceFilter instead of a ModelMultipleChoiceFilter
+    # because the queryset has been sliced, which means ModelMultipleChoiceFilter
+    # cannot do further queries to validate the selected tags.
+    pass
+
+
 class BaseMediaFilterSet(WagtailFilterSet):
     permission_policy = None
 
+    usage_count = django_filters.RangeFilter(
+        field_name="usage_count",
+        label=_("Usage count"),
+        widget=NumberRangeWidget(),
+    )
+
     def __init__(
         self, data=None, queryset=None, *, request=None, prefix=None, is_searching=None
     ):
