@@ -1,7 +1,11 @@
 import copy
 
+from django import VERSION as DJANGO_VERSION
+from django import forms
 from django.core.exceptions import ImproperlyConfigured
 from django.db import models
+from django.utils import timezone
+from django.utils.translation import gettext as _
 from modelcluster.forms import ClusterForm, ClusterFormMetaclass, ClusterFormOptions
 from permissionedforms import (
     PermissionedForm,
@@ -81,6 +85,19 @@ register_form_field_override(
     ),
 )
 
+# Slug fields
+register_form_field_override(
+    models.SlugField,
+    override={"widget": widgets.SlugInput},
+)
+
+# Remove the following block when the minimum Django version is >= 5.0.
+if (5, 0) <= DJANGO_VERSION < (6, 0):
+    register_form_field_override(
+        models.URLField,
+        override={"assume_scheme": "https"},
+    )
+
 
 # Callback to allow us to override the default form fields provided for each model field.
 def formfield_for_dbfield(db_field, **kwargs):
@@ -94,32 +111,18 @@ def formfield_for_dbfield(db_field, **kwargs):
 class WagtailAdminModelFormOptions(PermissionedFormOptionsMixin, ClusterFormOptions):
     # Container for the options set in the inner 'class Meta' of a model form, supporting
     # extensions for both ClusterForm ('formsets') and PermissionedForm ('field_permissions').
-    pass
+
+    def __init__(self, options=None):
+        super().__init__(options)
+        self.defer_required_on_fields = getattr(options, "defer_required_on_fields", [])
 
 
 class WagtailAdminModelFormMetaclass(PermissionedFormMetaclass, ClusterFormMetaclass):
     options_class = WagtailAdminModelFormOptions
 
-    # Override the behaviour of the regular ModelForm metaclass -
-    # which handles the translation of model fields to form fields -
-    # to use our own formfield_for_dbfield function to do that translation.
-    # This is done by sneaking a formfield_callback property into the class
-    # being defined (unless the class already provides a formfield_callback
-    # of its own).
-
-    # while we're at it, we'll also set extra_form_count to 0, as we're creating
-    # extra forms in JS
+    # set extra_form_count to 0, as we're creating extra forms in JS
     extra_form_count = 0
 
-    def __new__(cls, name, bases, attrs):
-        if "formfield_callback" not in attrs or attrs["formfield_callback"] is None:
-            attrs["formfield_callback"] = formfield_for_dbfield
-
-        new_class = super(WagtailAdminModelFormMetaclass, cls).__new__(
-            cls, name, bases, attrs
-        )
-        return new_class
-
     @classmethod
     def child_form(cls):
         return WagtailAdminModelForm
@@ -131,8 +134,81 @@ class WagtailAdminModelForm(
     def __init__(self, *args, **kwargs):
         # keep hold of the `for_user` kwarg as well as passing it on to PermissionedForm
         self.for_user = kwargs.get("for_user")
+        self.deferred_required_fields = []
+        self.deferred_formset_min_nums = {}
         super().__init__(*args, **kwargs)
 
+    def defer_required_fields(self):
+        if self.deferred_required_fields or self.deferred_formset_min_nums:
+            # defer_required_fields has already been called
+            return
+
+        for field_name in self._meta.defer_required_on_fields:
+            try:
+                if self.fields[field_name].required:
+                    self.fields[field_name].required = False
+                    self.deferred_required_fields.append(field_name)
+            except KeyError:
+                pass
+
+        for name, formset in self.formsets.items():
+            for form in formset:
+                form.defer_required_fields()
+            if formset.min_num is not None:
+                self.deferred_formset_min_nums[name] = formset.min_num
+                formset.min_num = 0
+
+    def restore_required_fields(self):
+        for name, formset in self.formsets.items():
+            for form in formset:
+                form.restore_required_fields()
+            if name in self.deferred_formset_min_nums:
+                formset.min_num = self.deferred_formset_min_nums[name]
+        self.deferred_formset_min_nums = {}
+
+        for field_name in self.deferred_required_fields:
+            self.fields[field_name].required = True
+        self.deferred_required_fields = []
+
+    class Meta:
+        formfield_callback = formfield_for_dbfield
+
 
 # Now, any model forms built off WagtailAdminModelForm instead of ModelForm should pick up
 # the nice form fields defined in FORM_FIELD_OVERRIDES.
+
+
+class WagtailAdminDraftStateFormMixin:
+    @property
+    def show_schedule_publishing_toggle(self):
+        return "go_live_at" in self.__class__.base_fields
+
+    def clean(self):
+        super().clean()
+
+        # Check scheduled publishing fields
+        go_live_at = self.cleaned_data.get("go_live_at")
+        expire_at = self.cleaned_data.get("expire_at")
+
+        # Go live must be before expire
+        if go_live_at and expire_at:
+            if go_live_at > expire_at:
+                msg = _("Go live date/time must be before expiry date/time")
+                self.add_error("go_live_at", forms.ValidationError(msg))
+                self.add_error("expire_at", forms.ValidationError(msg))
+
+        # Expire at must be in the future
+        if expire_at and expire_at < timezone.now():
+            self.add_error(
+                "expire_at",
+                forms.ValidationError(_("Expiry date/time must be in the future.")),
+            )
+
+        # Don't allow an existing first_published_at to be unset by clearing the field
+        if (
+            "first_published_at" in self.cleaned_data
+            and not self.cleaned_data["first_published_at"]
+        ):
+            del self.cleaned_data["first_published_at"]
+
+        return self.cleaned_data
