@@ -121,12 +121,12 @@ class BlockElementHandler:
         state.has_preceding_nonatomic_block = True
 
     def handle_endtag(self, name, state, contentState):
-        assert (
-            not state.current_inline_styles
-        ), "End of block reached without closing inline style elements"
-        assert (
-            not state.current_entity_ranges
-        ), "End of block reached without closing entity elements"
+        assert not state.current_inline_styles, (
+            "End of block reached without closing inline style elements"
+        )
+        assert not state.current_entity_ranges, (
+            "End of block reached without closing entity elements"
+        )
         state.current_block = None
 
 
@@ -388,7 +388,7 @@ class HtmlToContentStateHandler(HTMLParser):
         if not self.open_elements:
             return  # avoid a pop from an empty list if we have an extra end tag
         expected_name, element_handler = self.open_elements.pop()
-        assert name == expected_name, "Unmatched tags: expected %s, got %s" % (
+        assert name == expected_name, "Unmatched tags: expected {}, got {}".format(
             expected_name,
             name,
         )
