@@ -1,33 +1,65 @@
+from io import StringIO
+
 from django.contrib.auth.models import Permission
-from django.test import TestCase
+from django.core import management
+from django.test import TransactionTestCase
 from django.urls import reverse
+from django.utils.http import urlencode
 
-from wagtail.admin.tests.pages.timestamps import local_datetime
 from wagtail.models import Page
-from wagtail.search.index import SearchField
-from wagtail.test.testapp.models import SimplePage, SingleEventPage
+from wagtail.test.testapp.models import EventIndex, SimplePage, SingleEventPage
 from wagtail.test.utils import WagtailTestUtils
+from wagtail.test.utils.timestamps import local_datetime
 
 
-class TestPageSearch(TestCase, WagtailTestUtils):
+class TestPageSearch(WagtailTestUtils, TransactionTestCase):
+    fixtures = ["test_empty.json"]
+
     def setUp(self):
+        super().setUp()
+        management.call_command(
+            "update_index",
+            backend_name="default",
+            stdout=StringIO(),
+            chunk_size=50,
+        )
         self.user = self.login()
 
-    def get(self, params=None, **extra):
-        return self.client.get(
-            reverse("wagtailadmin_pages:search"), params or {}, **extra
-        )
+    def get(self, params=None, url_name="wagtailadmin_pages:search", **extra):
+        return self.client.get(reverse(url_name), params or {}, **extra)
 
     def test_view(self):
         response = self.get()
         self.assertTemplateUsed(response, "wagtailadmin/pages/search.html")
         self.assertEqual(response.status_code, 200)
 
+        with self.assertNumQueries(22):
+            self.get()
+
     def test_search(self):
+        # Find root page
+        root_page = Page.objects.get(id=2)
+
+        # Create a page
+        new_page = root_page.add_child(
+            instance=SimplePage(
+                title="Hello from Cauldron Lake",
+                slug="bright-falls",
+                content="It's not a lake, it's an ocean",
+                live=True,
+                has_unpublished_changes=False,
+            )
+        )
+
         response = self.get({"q": "Hello"})
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, "wagtailadmin/pages/search.html")
         self.assertEqual(response.context["query_string"], "Hello")
+        next_url = urlencode({"next": reverse("wagtailadmin_pages:search")})
+        expected_new_page_copy_url = (
+            reverse("wagtailadmin_pages:copy", args=(new_page.pk,)) + f"?{next_url}"
+        )
+        self.assertContains(response, f'href="{expected_new_page_copy_url}"')
 
     def test_search_searchable_fields(self):
         # Find root page
@@ -36,8 +68,8 @@ class TestPageSearch(TestCase, WagtailTestUtils):
         # Create a page
         root_page.add_child(
             instance=SimplePage(
-                title="Hi there!",
-                slug="hello-world",
+                title="Greetings!",
+                slug="hello",
                 content="good morning",
                 live=True,
                 has_unpublished_changes=False,
@@ -47,26 +79,38 @@ class TestPageSearch(TestCase, WagtailTestUtils):
         # Confirm the slug is not being searched
         response = self.get({"q": "hello"})
         self.assertNotContains(response, "There is one matching page")
-        search_fields = Page.search_fields
 
-        # Add slug to the search_fields
-        Page.search_fields = Page.search_fields + [
-            SearchField("slug", partial_match=True)
-        ]
-
-        # Confirm the slug is being searched
-        response = self.get({"q": "hello"})
+        # Confirm the title is being searched
+        response = self.get({"q": "greetings"})
         self.assertContains(response, "There is one matching page")
 
-        # Reset the search fields
-        Page.search_fields = search_fields
-
     def test_ajax(self):
-        response = self.get({"q": "Hello"}, HTTP_X_REQUESTED_WITH="XMLHttpRequest")
+        # Find root page
+        root_page = Page.objects.get(id=2)
+
+        # Create a page
+        new_page = root_page.add_child(
+            instance=SimplePage(
+                title="Hello from Cauldron Lake",
+                slug="bright-falls",
+                content="It's not a lake, it's an ocean",
+                live=True,
+                has_unpublished_changes=False,
+            )
+        )
+
+        response = self.get(
+            {"q": "Hello"}, url_name="wagtailadmin_pages:search_results"
+        )
         self.assertEqual(response.status_code, 200)
         self.assertTemplateNotUsed(response, "wagtailadmin/pages/search.html")
         self.assertTemplateUsed(response, "wagtailadmin/pages/search_results.html")
         self.assertEqual(response.context["query_string"], "Hello")
+        next_url = urlencode({"next": reverse("wagtailadmin_pages:search")})
+        expected_new_page_copy_url = (
+            reverse("wagtailadmin_pages:copy", args=(new_page.pk,)) + f"?{next_url}"
+        )
+        self.assertContains(response, f'href="{expected_new_page_copy_url}"')
 
     def test_pagination(self):
         pages = ["0", "1", "-1", "9999", "Not a page"]
@@ -76,11 +120,11 @@ class TestPageSearch(TestCase, WagtailTestUtils):
             self.assertTemplateUsed(response, "wagtailadmin/pages/search.html")
 
     def test_root_can_appear_in_search_results(self):
-        response = self.get({"q": "roo"})
+        response = self.get({"q": "root"})
         self.assertEqual(response.status_code, 200)
         # 'pages' list in the response should contain root
         results = response.context["pages"]
-        self.assertTrue(any([r.slug == "root" for r in results]))
+        self.assertTrue(any(r.slug == "root" for r in results))
 
     def test_search_uses_admin_display_title_from_specific_class(self):
         # SingleEventPage has a custom get_admin_display_title method; explorer should
@@ -208,3 +252,89 @@ class TestPageSearch(TestCase, WagtailTestUtils):
         # Incorrect content_type
         response = self.get({"content_type": "demosite.standardpage.error"})
         self.assertEqual(response.status_code, 404)
+
+    def test_empty_search_renders_content_type_facets(self):
+        root_page = Page.objects.get(id=2)
+        event_index = EventIndex(
+            title="ALL THE EVENTS",
+            intro="It's just a nod to the canon",
+        )
+        root_page.add_child(instance=event_index)
+
+        params = [{"q": ""}, {}]
+        url = reverse("wagtailadmin_pages:search")
+        for param in params:
+            with self.subTest(param=param):
+                response = self.get(param)
+                self.assertEqual(response.status_code, 200)
+                self.assertTemplateUsed(response, "wagtailadmin/pages/search.html")
+                self.assertEqual(response.context["query_string"], "")
+
+                self.assertContains(response, "Page types")
+                self.assertContains(response, "All (3)")
+                # The test fixture contains the root page and the welcome page
+                # with the base page type
+                self.assertContains(response, "Page (2)")
+
+                self.assertContains(response, "ALL THE EVENTS")
+
+                self.assertContains(response, "Event index (1)")
+                self.assertContains(
+                    response,
+                    f"{url}?q=&amp;content_type=tests.eventindex",
+                )
+
+                # The type column should not contain a link to order by content type
+                soup = self.get_soup(response.content)
+                headings = soup.select("main table thead th")
+                type_th = None
+                for heading in headings:
+                    if heading.text.strip() == "Type":
+                        type_th = heading
+                self.assertIsNotNone(type_th)
+                self.assertIsNone(type_th.select_one("a"))
+
+    def test_empty_search_with_content_type_filter(self):
+        root_page = Page.objects.get(id=2)
+        event_index = EventIndex(
+            title="ALL THE EVENTS",
+            intro="It's just a nod to the canon",
+        )
+        new_event = SingleEventPage(
+            title="Lunar event",
+            location="the moon",
+            audience="public",
+            cost="free",
+            date_from="2001-01-01",
+            latest_revision_created_at=local_datetime(2016, 1, 1),
+        )
+        root_page.add_child(instance=event_index)
+        root_page.add_child(instance=new_event)
+
+        params = [
+            {"q": "", "content_type": "tests.singleeventpage"},
+            {"content_type": "tests.singleeventpage"},
+        ]
+        url = reverse("wagtailadmin_pages:search")
+        for param in params:
+            with self.subTest(param=param):
+                response = self.get(param)
+                self.assertEqual(response.status_code, 200)
+                self.assertTemplateUsed(response, "wagtailadmin/pages/search.html")
+                self.assertEqual(response.context["query_string"], "")
+
+                self.assertContains(response, "Page types")
+                self.assertContains(response, "All (4)")
+                # The test fixture contains the root page and the welcome page
+                # with the base page type
+                self.assertContains(response, "Page (2)")
+                self.assertContains(response, "Single event page (1)")
+
+                self.assertContains(response, "Lunar event")
+                self.assertNotContains(response, "ALL THE EVENTS")
+
+                self.assertContains(response, "Event index (1)")
+                self.assertContains(
+                    response,
+                    f"{url}?q=&amp;content_type=tests.eventindex",
+                )
