@@ -17,7 +17,6 @@ from wagtail.test.testapp.models import (
     StreamPage,
 )
 from wagtail.test.utils import WagtailTestUtils
-from wagtail.utils.deprecation import RemovedInWagtail70Warning
 
 
 class TestIssue2599(WagtailTestUtils, TestCase):
@@ -173,7 +172,10 @@ class TestPreview(WagtailTestUtils, TestCase):
         )
         self.assertNotIn(preview_session_key, self.client.session)
 
-        response = self.client.post(preview_url, {**self.post_data, "title": ""})
+        response = self.client.post(
+            preview_url,
+            {**self.post_data, "date_from": "2025-02-31"},
+        )
 
         # Check the JSON response
         self.assertEqual(response.status_code, 200)
@@ -264,6 +266,98 @@ class TestPreview(WagtailTestUtils, TestCase):
         self.assertContains(response, "<li>Parties</li>")
         self.assertContains(response, "<li>Holidays</li>")
 
+    def test_preview_on_create_with_deferred_required_fields(self):
+        preview_url = reverse(
+            "wagtailadmin_pages:preview_on_add",
+            args=("tests", "eventpage", self.home_page.id),
+        )
+
+        preview_session_key = "wagtail-preview-tests-eventpage-{}".format(
+            self.home_page.id
+        )
+        self.assertNotIn(preview_session_key, self.client.session)
+
+        response = self.client.post(
+            preview_url,
+            {
+                "title": "Bare minimum",
+                "slug": "bare-minimum",
+                "carousel_items-TOTAL_FORMS": 0,
+                "carousel_items-INITIAL_FORMS": 0,
+                "speakers-TOTAL_FORMS": 0,
+                "speakers-INITIAL_FORMS": 0,
+                "related_links-TOTAL_FORMS": 0,
+                "related_links-INITIAL_FORMS": 0,
+                "head_counts-TOTAL_FORMS": 0,
+                "head_counts-INITIAL_FORMS": 0,
+            },
+        )
+
+        # Check the JSON response
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(
+            response.content.decode(),
+            {"is_valid": True, "is_available": True},
+        )
+
+        # Check the user can refresh the preview
+        preview_session_key = "wagtail-preview-tests-eventpage-{}".format(
+            self.home_page.id
+        )
+        self.assertIn(preview_session_key, self.client.session)
+
+        response = self.client.get(preview_url)
+
+        # Check the HTML response
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, "tests/event_page.html")
+        self.assertContains(response, "Bare minimum")
+
+    def test_preview_on_create_without_title_and_slug(self):
+        preview_url = reverse(
+            "wagtailadmin_pages:preview_on_add",
+            args=("tests", "eventpage", self.home_page.id),
+        )
+
+        preview_session_key = "wagtail-preview-tests-eventpage-{}".format(
+            self.home_page.id
+        )
+        self.assertNotIn(preview_session_key, self.client.session)
+
+        response = self.client.post(
+            preview_url,
+            {
+                "carousel_items-TOTAL_FORMS": 0,
+                "carousel_items-INITIAL_FORMS": 0,
+                "speakers-TOTAL_FORMS": 0,
+                "speakers-INITIAL_FORMS": 0,
+                "related_links-TOTAL_FORMS": 0,
+                "related_links-INITIAL_FORMS": 0,
+                "head_counts-TOTAL_FORMS": 0,
+                "head_counts-INITIAL_FORMS": 0,
+            },
+        )
+
+        # Check the JSON response
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(
+            response.content.decode(),
+            {"is_valid": True, "is_available": True},
+        )
+
+        # Check the user can refresh the preview
+        preview_session_key = "wagtail-preview-tests-eventpage-{}".format(
+            self.home_page.id
+        )
+        self.assertIn(preview_session_key, self.client.session)
+
+        response = self.client.get(preview_url)
+
+        # Check the HTML response
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, "tests/event_page.html")
+        self.assertContains(response, "Placeholder title")
+
     def test_preview_on_edit_with_m2m_field(self):
         preview_url = reverse(
             "wagtailadmin_pages:preview_on_edit", args=(self.event_page.id,)
@@ -320,6 +414,36 @@ class TestPreview(WagtailTestUtils, TestCase):
         self.assertContains(response, "<li>Parties</li>")
         self.assertContains(response, "<li>Holidays</li>")
 
+        soup = self.get_soup(response.content)
+        userbar = soup.select_one("wagtail-userbar")
+        self.assertIsNotNone(userbar)
+        template = soup.select_one("#wagtail-userbar-template")
+        self.assertIsNotNone(template)
+
+        # Previews use a dummy request object, so the userbar should use
+        # relative URLs to avoid using the wrong host.
+        admin_url = reverse("wagtailadmin_home")
+        admin_link = template.select_one(f'a[href$="{admin_url}"]')
+        self.assertIsNotNone(admin_link)
+        self.assertEqual(admin_link["href"], admin_url)
+
+        css_links = soup.select("link[rel='stylesheet']")
+        self.assertEqual(
+            [link.get("href") for link in css_links],
+            [
+                versioned_static("wagtailadmin/css/core.css"),
+                "/path/to/my/custom.css",
+            ],
+        )
+        scripts = soup.select("script[src]")
+        self.assertEqual(
+            [script.get("src") for script in scripts],
+            [
+                versioned_static("wagtailadmin/js/vendor.js"),
+                versioned_static("wagtailadmin/js/userbar.js"),
+            ],
+        )
+
     def test_preview_on_edit_with_valid_then_invalid_data(self):
         preview_url = reverse(
             "wagtailadmin_pages:preview_on_edit", args=(self.event_page.id,)
@@ -334,7 +458,10 @@ class TestPreview(WagtailTestUtils, TestCase):
         )
 
         # Send an invalid update request
-        response = self.client.post(preview_url, {**self.post_data, "title": ""})
+        response = self.client.post(
+            preview_url,
+            {**self.post_data, "date_to": "1234-56-78"},
+        )
         self.assertEqual(response.status_code, 200)
         self.assertJSONEqual(
             response.content.decode(),
@@ -354,6 +481,82 @@ class TestPreview(WagtailTestUtils, TestCase):
         self.assertContains(response, "<li>Parties</li>")
         self.assertContains(response, "<li>Holidays</li>")
 
+    def test_preview_on_edit_with_deferred_required_fields(self):
+        preview_url = reverse(
+            "wagtailadmin_pages:preview_on_edit", args=(self.event_page.id,)
+        )
+
+        response = self.client.post(
+            preview_url,
+            {
+                "title": "Bare minimum",
+                "slug": "bare-minimum",
+                "carousel_items-TOTAL_FORMS": 0,
+                "carousel_items-INITIAL_FORMS": 0,
+                "speakers-TOTAL_FORMS": 0,
+                "speakers-INITIAL_FORMS": 0,
+                "related_links-TOTAL_FORMS": 0,
+                "related_links-INITIAL_FORMS": 0,
+                "head_counts-TOTAL_FORMS": 0,
+                "head_counts-INITIAL_FORMS": 0,
+            },
+        )
+
+        # Check the JSON response
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(
+            response.content.decode(),
+            {"is_valid": True, "is_available": True},
+        )
+
+        # Check the user can refresh the preview
+        preview_session_key = f"wagtail-preview-{self.event_page.id}"
+        self.assertIn(preview_session_key, self.client.session)
+
+        response = self.client.get(preview_url)
+
+        # Check the HTML response
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, "tests/event_page.html")
+        self.assertContains(response, "Bare minimum")
+
+    def test_preview_on_edit_without_title_and_slug(self):
+        preview_url = reverse(
+            "wagtailadmin_pages:preview_on_edit", args=(self.event_page.id,)
+        )
+
+        response = self.client.post(
+            preview_url,
+            {
+                "carousel_items-TOTAL_FORMS": 0,
+                "carousel_items-INITIAL_FORMS": 0,
+                "speakers-TOTAL_FORMS": 0,
+                "speakers-INITIAL_FORMS": 0,
+                "related_links-TOTAL_FORMS": 0,
+                "related_links-INITIAL_FORMS": 0,
+                "head_counts-TOTAL_FORMS": 0,
+                "head_counts-INITIAL_FORMS": 0,
+            },
+        )
+
+        # Check the JSON response
+        self.assertEqual(response.status_code, 200)
+        self.assertJSONEqual(
+            response.content.decode(),
+            {"is_valid": True, "is_available": True},
+        )
+
+        # Check the user can refresh the preview
+        preview_session_key = f"wagtail-preview-{self.event_page.id}"
+        self.assertIn(preview_session_key, self.client.session)
+
+        response = self.client.get(preview_url)
+
+        # Check the HTML response
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, "tests/event_page.html")
+        self.assertContains(response, "Placeholder title")
+
     def test_preview_on_edit_expiry(self):
         initial_datetime = timezone.now()
         expiry_datetime = initial_datetime + datetime.timedelta(
@@ -746,39 +949,6 @@ class TestEnablePreview(WagtailTestUtils, TestCase):
         self.assertEqual(refresh_button.get("data-controller"), "w-progress")
         self.assertEqual(refresh_button.text.strip(), "Refresh")
 
-    @override_settings(WAGTAIL_AUTO_UPDATE_PREVIEW=False)
-    def test_disable_auto_update_using_deprecated_setting(self):
-        edit_url = self.get_url_on_edit("edit", self.single)
-        preview_url = self.get_url_on_edit("preview_on_edit", self.single)
-        with self.assertWarnsMessage(
-            RemovedInWagtail70Warning,
-            "`WAGTAIL_AUTO_UPDATE_PREVIEW` is deprecated. "
-            "Set `WAGTAIL_AUTO_UPDATE_PREVIEW_INTERVAL = 0` to disable auto-update "
-            "for previews.",
-        ):
-            response = self.client.get(edit_url)
-
-        self.assertEqual(response.status_code, 200)
-
-        soup = self.get_soup(response.content)
-
-        # Should set the interval value on the controller
-        controller = soup.select_one('[data-controller="w-preview"]')
-        self.assertIsNotNone(controller)
-        self.assertEqual(controller.get("data-w-preview-url-value"), preview_url)
-        interval_value = controller.get("data-w-preview-auto-update-interval-value")
-        self.assertEqual(interval_value, "0")
-
-        # Should not render the spinner
-        spinner = controller.select_one('[data-w-preview-target="spinner"]')
-        self.assertIsNone(spinner)
-
-        # Should render the refresh button with the w-progress controller
-        refresh_button = controller.select_one("button")
-        self.assertIsNotNone(refresh_button)
-        self.assertEqual(refresh_button.get("data-controller"), "w-progress")
-        self.assertEqual(refresh_button.text.strip(), "Refresh")
-
     def test_show_preview_on_revisions_list(self):
         latest_revision = self.single.save_revision(log_action=True)
         history_url = self.get_url_on_edit("history", self.single)
