@@ -2,6 +2,7 @@ import json
 import re
 from html import unescape
 
+from bs4 import BeautifulSoup
 from django import forms
 from django.test import TestCase
 from django.test.utils import override_settings
@@ -9,9 +10,10 @@ from django.utils.html import escape
 
 from wagtail.admin import widgets
 from wagtail.admin.forms.tags import TagField
-from wagtail.models import Page
+from wagtail.models import Locale, Page
 from wagtail.test.testapp.forms import AdminStarDateInput
 from wagtail.test.testapp.models import EventPage, RestaurantTag, SimplePage
+from wagtail.utils.deprecation import RemovedInWagtail80Warning
 
 
 class TestAdminPageChooserWidget(TestCase):
@@ -430,9 +432,9 @@ class TestAdminTagWidget(TestCase):
             ],
         )
 
-    @override_settings(TAG_SPACES_ALLOWED=False)
+    @override_settings(WAGTAIL_TAG_SPACES_ALLOWED=False)
     def test_render_js_init_no_spaces_allowed(self):
-        """Checks that the 'w-tag' controller attributes are correctly added to the tag widgets based  on TAG_SPACES_ALLOWED in settings"""
+        """Checks that the 'w-tag' controller attributes are correctly added to the tag widgets based  on WAGTAIL_TAG_SPACES_ALLOWED in settings"""
         widget = widgets.AdminTagWidget()
 
         html = widget.render("tags", None, attrs={"id": "alpha"})
@@ -447,9 +449,9 @@ class TestAdminTagWidget(TestCase):
             ],
         )
 
-    @override_settings(TAG_LIMIT=5)
+    @override_settings(WAGTAIL_TAG_LIMIT=5)
     def test_render_js_init_with_tag_limit(self):
-        """Checks that the 'w-tag' controller attributes are correctly added to the tag widget using options based on TAG_LIMIT in settings"""
+        """Checks that the 'w-tag' controller attributes are correctly added to the tag widget using options based on WAGTAIL_TAG_LIMIT in settings"""
 
         widget = widgets.AdminTagWidget()
 
@@ -517,9 +519,9 @@ class TestAdminTagWidget(TestCase):
             ],
         )
 
-    @override_settings(TAG_SPACES_ALLOWED=True)
+    @override_settings(WAGTAIL_TAG_SPACES_ALLOWED=True)
     def test_tags_help_text_spaces_allowed(self):
-        """Checks that the tags help text html element content is correct when TAG_SPACES_ALLOWED is True"""
+        """Checks that the tags help text html element content is correct when WAGTAIL_TAG_SPACES_ALLOWED is True"""
         widget = widgets.AdminTagWidget()
         help_text = widget.get_context(None, None, {})["widget"]["help_text"]
 
@@ -535,9 +537,9 @@ class TestAdminTagWidget(TestCase):
             html,
         )
 
-    @override_settings(TAG_SPACES_ALLOWED=False)
+    @override_settings(WAGTAIL_TAG_SPACES_ALLOWED=False)
     def test_tags_help_text_no_spaces_allowed(self):
-        """Checks that the tags help text html element content is correct when TAG_SPACES_ALLOWED is False"""
+        """Checks that the tags help text html element content is correct when WAGTAIL_TAG_SPACES_ALLOWED is False"""
         widget = widgets.AdminTagWidget()
         help_text = widget.get_context(None, None, {})["widget"]["help_text"]
 
@@ -552,6 +554,44 @@ class TestAdminTagWidget(TestCase):
             html,
         )
 
+    @override_settings(TAG_LIMIT=3)
+    def test_legacy_tag_limit_setting(self):
+        widget = widgets.AdminTagWidget()
+        with self.assertWarnsMessage(
+            RemovedInWagtail80Warning,
+            "The setting 'TAG_LIMIT' is deprecated. "
+            "Please use 'WAGTAIL_TAG_LIMIT' instead.",
+        ):
+            html = widget.render("tags", None, attrs={"id": "alpha"})
+        params = self.get_js_init_params(html)
+        self.assertEqual(
+            params,
+            [
+                "alpha",
+                "/admin/tag-autocomplete/",
+                {"allowSpaces": True, "tagLimit": 3, "autocompleteOnly": False},
+            ],
+        )
+
+    @override_settings(TAG_SPACES_ALLOWED=False)
+    def test_legacy_tag_spaces_allowed_setting(self):
+        widget = widgets.AdminTagWidget()
+        with self.assertWarnsMessage(
+            RemovedInWagtail80Warning,
+            "The setting 'TAG_SPACES_ALLOWED' is deprecated. "
+            "Please use 'WAGTAIL_TAG_SPACES_ALLOWED' instead.",
+        ):
+            html = widget.render("tags", None, attrs={"id": "alpha"})
+        params = self.get_js_init_params(html)
+        self.assertEqual(
+            params,
+            [
+                "alpha",
+                "/admin/tag-autocomplete/",
+                {"allowSpaces": False, "tagLimit": None, "autocompleteOnly": False},
+            ],
+        )
+
 
 class TestTagField(TestCase):
     def setUp(self):
@@ -659,20 +699,224 @@ class TestFilteredSelect(TestCase):
 
 
 class TestSlugInput(TestCase):
+    def getAttrs(self, *args, **kwargs):
+        return (
+            BeautifulSoup(
+                widgets.SlugInput(*args, **kwargs).render("slug", None),
+                "html.parser",
+            )
+            .find("input")
+            .attrs
+        )
+
     def test_has_data_attr(self):
-        widget = widgets.slug.SlugInput()
+        widget = widgets.SlugInput()
 
         html = widget.render("test", None, attrs={"id": "test-id"})
 
         self.assertInHTML(
-            '<input type="text" name="test" data-controller="w-slug" data-action="blur-&gt;w-slug#slugify w-sync:check-&gt;w-slug#compare w-sync:apply-&gt;w-slug#urlify:prevent" data-w-slug-allow-unicode-value data-w-slug-compare-as-param="urlify" id="test-id">',
+            '<input type="text" name="test" data-controller="w-slug" data-action="blur-&gt;w-slug#slugify w-sync:check-&gt;w-slug#compare w-sync:apply-&gt;w-slug#urlify:prevent" data-w-slug-allow-unicode-value data-w-slug-compare-as-param="urlify" data-w-slug-trim-value="true" id="test-id">',
             html,
         )
 
     @override_settings(WAGTAIL_ALLOW_UNICODE_SLUGS=False)
-    def test_render_data_atrrs_from_settings(self):
-        widget = widgets.slug.SlugInput()
+    def test_render_data_attrs_from_settings(self):
+        widget = widgets.SlugInput()
 
         html = widget.render("test", None, attrs={"id": "test-id"})
 
         self.assertNotIn("data-w-slug-allow-unicode-value", html)
+
+    def test_with_locale_and_formatters_not_provided(self):
+        self.assertEqual(
+            self.getAttrs(),
+            {
+                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
+                "data-controller": "w-slug",
+                "data-w-slug-allow-unicode-value": "",
+                "data-w-slug-compare-as-param": "urlify",
+                # "data-w-slug-formatters-value":... # not included at all
+                # "data-w-slug-locale-value":... # not included at all
+                "data-w-slug-trim-value": "true",
+                "name": "slug",
+                "type": "text",
+            },
+        )
+
+        self.assertEqual(
+            self.getAttrs(formatters=[], locale=None),
+            {
+                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
+                "data-controller": "w-slug",
+                "data-w-slug-allow-unicode-value": "",
+                "data-w-slug-compare-as-param": "urlify",
+                # "data-w-slug-formatters-value":... # not included at all
+                # "data-w-slug-locale-value":... # not included at all
+                "data-w-slug-trim-value": "true",
+                "name": "slug",
+                "type": "text",
+            },
+        )
+
+    # Test formatters argument
+
+    def test_with_formatters_provided_are_escaped(self):
+        self.assertEqual(
+            self.getAttrs(formatters=[(r"\D\s[']+", "'?'")])[
+                "data-w-slug-formatters-value"
+            ],
+            '[[["\\\\D\\\\s[\']+","gu"],"\'?\'"]]',
+        )
+
+    def test_with_formatters_as_string(self):
+        self.assertEqual(
+            self.getAttrs(formatters=[r"\d"])["data-w-slug-formatters-value"],
+            '[[["\\\\d","gu"],""]]',
+        )
+
+        # handling of inline flags
+        self.assertEqual(
+            self.getAttrs(formatters=[r"(?m)^\d+"])["data-w-slug-formatters-value"],
+            '[[["^\\\\d+","gmu"],""]]',
+        )
+
+    def test_with_formatters_as_pattern(self):
+        self.assertEqual(
+            self.getAttrs(formatters=[re.compile(r"\d")])[
+                "data-w-slug-formatters-value"
+            ],
+            '[[["\\\\d","gu"],""]]',
+        )
+
+        # handling of inline flags
+        self.assertEqual(
+            self.getAttrs(
+                formatters=[re.compile(r"(?i)\b(?:and\|or\|the\|in\|of\|to)\b")]
+            )["data-w-slug-formatters-value"],
+            '[[["\\\\b(?:and\\\\|or\\\\|the\\\\|in\\\\|of\\\\|to)\\\\b","giu"],""]]',
+        )
+
+    def test_with_formatters_as_list_like_with_string(self):
+        self.assertEqual(
+            self.getAttrs(formatters=[["ABC"]])["data-w-slug-formatters-value"],
+            '[[["ABC","gu"],""]]',
+        )
+
+        self.assertEqual(
+            self.getAttrs(formatters=[(r"\d",)])["data-w-slug-formatters-value"],
+            '[[["\\\\d","gu"],""]]',
+        )
+
+    def test_with_formatters_as_list_like_with_pattern(self):
+        self.assertEqual(
+            self.getAttrs(formatters=[[re.compile(r"(?i)!")]])[
+                "data-w-slug-formatters-value"
+            ],
+            '[[["!","giu"],""]]',
+        )
+        self.assertEqual(
+            self.getAttrs(formatters=[(re.compile("(?m)^A"),)])[
+                "data-w-slug-formatters-value"
+            ],
+            '[[["^A","gmu"],""]]',
+        )
+
+    def test_with_formatters_with_replace(self):
+        self.assertEqual(
+            self.getAttrs(formatters=[(r"\d", "X")])["data-w-slug-formatters-value"],
+            '[[["\\\\d","gu"],"X"]]',
+        )
+
+        self.assertEqual(
+            self.getAttrs(formatters=[[re.compile(r"\d{1,3}"), "X"]])[
+                "data-w-slug-formatters-value"
+            ],
+            '[[["\\\\d{1,3}","gu"],"X"]]',
+        )
+
+    def test_with_formatters_with_replace_and_flags(self):
+        self.assertEqual(
+            self.getAttrs(
+                formatters=[
+                    [re.compile(r"^(?!blog[-\s])", flags=re.MULTILINE), "blog-", "u"]
+                ]
+            )["data-w-slug-formatters-value"],
+            '[[["^(?!blog[-\\\\s])","mu"],"blog-"]]',
+        )
+
+        self.assertEqual(
+            self.getAttrs(
+                formatters=[[re.compile(r"(?i)a*"), "Z", "g"], [r"the", "", "u"]]
+            )["data-w-slug-formatters-value"],
+            '[[["a*","gi"],"Z"],[["the","u"],""]]',
+        )
+
+    def test_with_multiple_formatters(self):
+        self.assertEqual(
+            self.getAttrs(
+                formatters=[
+                    r"\d",
+                    [r"(?<!\S)Й", "Y"],
+                    [re.compile(r"(?<!\S)Є"), "Ye", "u"],
+                    (r"(?i)the",),
+                ]
+            )["data-w-slug-formatters-value"],
+            '[[["\\\\d","gu"],""],[["(?<!\\\\S)\\u0419","gu"],"Y"],[["(?<!\\\\S)\\u0404","u"],"Ye"],[["the","giu"],""]]',
+        )
+
+    # Test locale argument
+
+    def test_with_locale_provided(self):
+        self.assertEqual(
+            self.getAttrs(locale="uk-UK"),
+            {
+                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
+                "data-controller": "w-slug",
+                "data-w-slug-allow-unicode-value": "",
+                "data-w-slug-compare-as-param": "urlify",
+                "data-w-slug-locale-value": "uk-UK",  # from provided locale
+                "data-w-slug-trim-value": "true",
+                "name": "slug",
+                "type": "text",
+            },
+        )
+
+        french = Locale.objects.create(language_code="fr")
+        self.assertEqual(
+            self.getAttrs(locale=french),
+            {
+                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
+                "data-controller": "w-slug",
+                "data-w-slug-allow-unicode-value": "",
+                "data-w-slug-compare-as-param": "urlify",
+                "data-w-slug-locale-value": "fr",  # from provided locale
+                "data-w-slug-trim-value": "true",
+                "name": "slug",
+                "type": "text",
+            },
+        )
+
+        self.assertEqual(
+            self.getAttrs(locale=False),
+            {
+                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
+                "data-controller": "w-slug",
+                "data-w-slug-allow-unicode-value": "",
+                "data-w-slug-compare-as-param": "urlify",
+                "data-w-slug-locale-value": "und",  # from False (aka 'undetermined')
+                "data-w-slug-trim-value": "true",
+                "name": "slug",
+                "type": "text",
+            },
+        )
+
+    def test_with_locale_blank_override(self):
+        self.assertEqual(
+            self.getAttrs(locale=False)["data-w-slug-locale-value"],
+            "und",
+        )
+
+        self.assertEqual(
+            self.getAttrs(locale="")["data-w-slug-locale-value"],
+            "und",
+        )
