@@ -1,9 +1,6 @@
-# -*- coding: utf-8 -*-
-
 import json
 import unittest
 
-from django import VERSION as DJANGO_VERSION
 from django.conf import settings
 from django.contrib.auth.models import Group, Permission
 from django.core import mail
@@ -21,7 +18,7 @@ from wagtail.test.testapp.models import RestaurantTag
 from wagtail.test.utils import WagtailTestUtils
 
 
-class TestHome(TestCase, WagtailTestUtils):
+class TestHome(WagtailTestUtils, TestCase):
     def setUp(self):
         # Login
         self.login()
@@ -29,7 +26,23 @@ class TestHome(TestCase, WagtailTestUtils):
     def test_simple(self):
         response = self.client.get(reverse("wagtailadmin_home"))
         self.assertEqual(response.status_code, 200)
-        self.assertContains(response, "Welcome to the Test Site Wagtail CMS")
+        self.assertContains(response, "Test Site")
+
+    def test_accessible_skip_link_exists(self):
+        response = self.client.get(reverse("wagtailadmin_home"))
+        self.assertEqual(response.status_code, 200)
+        soup = self.get_soup(response.content)
+
+        # Check that the skip link is present and has the correct attributes
+        skip_link = soup.select_one(".skiplink")
+        self.assertIsNotNone(skip_link)
+        self.assertEqual(skip_link["href"], "#main")
+        self.assertEqual(skip_link["data-controller"], "w-focus")
+        self.assertEqual(skip_link["data-action"], "w-focus#focus:prevent")
+
+        # Check that the main content area has the correct ID
+        main_content = soup.select_one("#main")
+        self.assertIsNotNone(main_content)
 
     def test_admin_menu(self):
         response = self.client.get(reverse("wagtailadmin_home"))
@@ -37,26 +50,23 @@ class TestHome(TestCase, WagtailTestUtils):
         # check that custom menu items (including classname / icon_name) are pulled in
         self.assertContains(
             response,
-            '{"name": "kittens", "label": "Kittens!", "icon_name": "kitten", "classnames": "kitten--test", "url": "http://www.tomroyal.com/teaandkittens/", "attrs": {"data-is-custom": "true"}}',
+            '{"name": "kittens", "label": "Kittens!", "icon_name": "kitten", "classname": "kitten--test", "attrs": {"data-is-custom": "true"}, "url": "http://www.tomroyal.com/teaandkittens/"}',
         )
 
         # Check that the explorer menu item is here, with the right start page.
         self.assertContains(
             response,
-            '[{"name": "explorer", "label": "Pages", "icon_name": "folder-open-inverse", "classnames": "", "url": "/admin/pages/", "attrs": null}, 1]',
+            '[{"name": "explorer", "label": "Pages", "icon_name": "folder-open-inverse", "classname": "", "attrs": {}, "url": "/admin/pages/"}, 1]',
         )
 
-        # There should be a link to the friend admin in on the home page.
-        self.assertContains(response, '"url": "/admin/friendadmin/"')
-
-        # Since we've marked this as not being shown, it shouldn't be shown.
-        self.assertNotContains(response, '"url": "/admin/enemyadmin/"')
+        # There should be a link to the full-featured snippet admin in on the home page.
+        self.assertContains(response, '"url": "/admin/deep/within/the/admin/"')
 
         # check that is_shown is respected on menu items
         response = self.client.get(reverse("wagtailadmin_home") + "?hide-kittens=true")
         self.assertNotContains(
             response,
-            '{"name": "kittens", "label": "Kittens!", "icon_name": "kitten", "classnames": "kitten--test", "url": "http://www.tomroyal.com/teaandkittens/", "attrs": {"data-is-custom": "true"}}',
+            '{"name": "kittens", "label": "Kittens!", "icon_name": "kitten", "classname": "kitten--test", "attrs": {"data-is-custom": "true"}, "url": "http://www.tomroyal.com/teaandkittens/"}',
         )
 
     def test_dashboard_panels(self):
@@ -78,18 +88,11 @@ class TestHome(TestCase, WagtailTestUtils):
         self.assertContains(response, "<li>0 broken links</li>")
 
         # check that media attached to summary items is correctly pulled in
-        if DJANGO_VERSION >= (4, 1):
-            self.assertContains(
-                response,
-                '<link href="/static/testapp/css/broken-links.css" media="all" rel="stylesheet">',
-                html=True,
-            )
-        else:
-            self.assertContains(
-                response,
-                '<link href="/static/testapp/css/broken-links.css" type="text/css" media="all" rel="stylesheet">',
-                html=True,
-            )
+        self.assertContains(
+            response,
+            '<link href="/static/testapp/css/broken-links.css" media="all" rel="stylesheet">',
+            html=True,
+        )
 
     def test_never_cache_header(self):
         # This tests that wagtailadmins global cache settings have been applied correctly
@@ -116,7 +119,7 @@ class TestHome(TestCase, WagtailTestUtils):
         self.assertEqual(response.status_code, 200)
 
 
-class TestEditorHooks(TestCase, WagtailTestUtils):
+class TestEditorHooks(WagtailTestUtils, TestCase):
     def setUp(self):
         self.homepage = Page.objects.get(id=2)
         self.login()
@@ -235,8 +238,80 @@ class TestSendMail(TestCase):
         self.assertEqual(email_message.body, "TEXT content")
         self.assertEqual(email_message.to, ["mr.plain.text@email.com"])
 
+    def test_send_cc(self):
+        send_mail(
+            "Test subject",
+            "Test content",
+            ["nobody@email.com"],
+            "test@email.com",
+            cc=["cc.test@email.com"],
+        )
 
-class TestTagsAutocomplete(TestCase, WagtailTestUtils):
+        # Check that the email was sent
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(mail.outbox[0].subject, "Test subject")
+        self.assertEqual(mail.outbox[0].body, "Test content")
+        self.assertEqual(mail.outbox[0].to, ["nobody@email.com"])
+        self.assertEqual(mail.outbox[0].from_email, "test@email.com")
+        self.assertEqual(mail.outbox[0].cc, ["cc.test@email.com"])
+
+    def test_send_bcc(self):
+        send_mail(
+            "Test subject",
+            "Test content",
+            ["nobody@email.com"],
+            "test@email.com",
+            bcc=["bcc.test@email.com"],
+        )
+
+        # Check that the email was sent
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(mail.outbox[0].subject, "Test subject")
+        self.assertEqual(mail.outbox[0].body, "Test content")
+        self.assertEqual(mail.outbox[0].to, ["nobody@email.com"])
+        self.assertEqual(mail.outbox[0].from_email, "test@email.com")
+        self.assertEqual(mail.outbox[0].bcc, ["bcc.test@email.com"])
+
+    def test_send_reply_to(self):
+        send_mail(
+            "Test subject",
+            "Test content",
+            ["nobody@email.com"],
+            "test@email.com",
+            reply_to=["reply_to.test@email.com"],
+        )
+
+        # Check that the email was sent
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(mail.outbox[0].subject, "Test subject")
+        self.assertEqual(mail.outbox[0].body, "Test content")
+        self.assertEqual(mail.outbox[0].to, ["nobody@email.com"])
+        self.assertEqual(mail.outbox[0].from_email, "test@email.com")
+        self.assertEqual(mail.outbox[0].reply_to, ["reply_to.test@email.com"])
+
+    def test_send_all_extra_fields(self):
+        send_mail(
+            "Test subject",
+            "Test content",
+            ["nobody@email.com"],
+            "test@email.com",
+            cc=["cc.test@email.com"],
+            bcc=["bcc.test@email.com"],
+            reply_to=["reply_to.test@email.com"],
+        )
+
+        # Check that the email was sent
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertEqual(mail.outbox[0].subject, "Test subject")
+        self.assertEqual(mail.outbox[0].body, "Test content")
+        self.assertEqual(mail.outbox[0].to, ["nobody@email.com"])
+        self.assertEqual(mail.outbox[0].from_email, "test@email.com")
+        self.assertEqual(mail.outbox[0].cc, ["cc.test@email.com"])
+        self.assertEqual(mail.outbox[0].bcc, ["bcc.test@email.com"])
+        self.assertEqual(mail.outbox[0].reply_to, ["reply_to.test@email.com"])
+
+
+class TestTagsAutocomplete(WagtailTestUtils, TestCase):
     def setUp(self):
         self.login()
         Tag.objects.create(name="Test", slug="test")
@@ -312,19 +387,43 @@ class TestTagsAutocomplete(TestCase, WagtailTestUtils):
 
         self.assertEqual(data, [])
 
+    def test_tags_autocomplete_limit(self):
+        tags = [Tag(name=f"Tag {i}", slug=f"tag-{i}") for i in range(15)]
+        Tag.objects.bulk_create(tags)
 
-class TestMenuItem(TestCase, WagtailTestUtils):
+        # Send a request to the autocomplete endpoint with a broad search term
+        response = self.client.get(
+            reverse("wagtailadmin_tag_autocomplete"), {"term": "Tag"}
+        )
+
+        # Confirm the response is successful
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response["Content-Type"], "application/json")
+
+        data = json.loads(response.content.decode("utf-8"))
+
+        # The results should be limited to avoid performance issues (#12415)
+        self.assertEqual(len(data), 10)
+        sorted_tags = sorted(tags, key=lambda t: t.name)
+        self.assertEqual(data, [tag.name for tag in sorted_tags[:10]])
+
+
+class TestMenuItem(WagtailTestUtils, TestCase):
     def setUp(self):
         self.login()
         response = self.client.get(reverse("wagtailadmin_home"))
         self.request = response.wsgi_request
 
-    def test_menuitem_reverse_lazy_url_pass(self):
-        menuitem = MenuItem(_("Test"), reverse_lazy("wagtailadmin_home"))
-        self.assertIs(menuitem.is_active(self.request), True)
+    def test_menuitem_with_classname(self):
+        menuitem = MenuItem(
+            _("Test"),
+            reverse_lazy("wagtailadmin_home"),
+            classname="highlight-item",
+        )
+        self.assertEqual(menuitem.classname, "highlight-item")
 
 
-class TestUserPassesTestPermissionDecorator(TestCase, WagtailTestUtils):
+class TestUserPassesTestPermissionDecorator(WagtailTestUtils, TestCase):
     """
     Test for custom user_passes_test permission decorators.
     testapp_bob_only_zone is a view configured to only grant access to users with a first_name of Bob
@@ -363,7 +462,7 @@ class TestUserPassesTestPermissionDecorator(TestCase, WagtailTestUtils):
         self.assertEqual(response.status_code, 403)
 
 
-class TestUserHasAnyPagePermission(TestCase, WagtailTestUtils):
+class TestUserHasAnyPagePermission(WagtailTestUtils, TestCase):
     def test_superuser(self):
         user = self.create_superuser(
             username="superuser", email="admin@example.com", password="p"
@@ -401,7 +500,7 @@ class TestUserHasAnyPagePermission(TestCase, WagtailTestUtils):
         self.assertFalse(user_has_any_page_permission(user))
 
 
-class Test404(TestCase, WagtailTestUtils):
+class Test404(WagtailTestUtils, TestCase):
     def test_admin_404_template_used_append_slash_true(self):
         self.login()
         with self.settings(APPEND_SLASH=True):
@@ -410,6 +509,9 @@ class Test404(TestCase, WagtailTestUtils):
             # Check 404 error after CommonMiddleware redirect
             self.assertEqual(response.status_code, 404)
             self.assertTemplateUsed(response, "wagtailadmin/404.html")
+            soup = self.get_soup(response.content)
+            self.assertFalse(soup.select("script"))
+            self.assertFalse(soup.select("[data-sprite]"))
 
     def test_not_logged_in_redirect(self):
         response = self.client.get("/admin/sdfgdsfgdsfgsdf/")
@@ -420,7 +522,7 @@ class Test404(TestCase, WagtailTestUtils):
         )
 
 
-class TestAdminURLAppendSlash(TestCase, WagtailTestUtils):
+class TestAdminURLAppendSlash(WagtailTestUtils, TestCase):
     def setUp(self):
         # Find root page
         self.root_page = Page.objects.get(id=2)
@@ -435,13 +537,11 @@ class TestAdminURLAppendSlash(TestCase, WagtailTestUtils):
 
             # Check that correct page is returned after CommonMiddleware redirect
             self.assertEqual(response.status_code, 200)
-            self.assertTemplateUsed(response, "wagtailadmin/pages/index.html")
-            self.assertEqual(Page.objects.get(id=1), response.context["parent_page"])
-            self.assertTrue(
-                response.context["pages"]
-                .paginator.object_list.filter(id=self.root_page.id)
-                .exists()
+            self.assertTemplateUsed(
+                response, "wagtailadmin/pages/explorable_index.html"
             )
+            self.assertEqual(Page.objects.get(id=1), response.context["parent_page"])
+            self.assertIn(self.root_page, response.context["pages"])
 
 
 class TestRemoveStaleContentTypes(TestCase):
