@@ -25,7 +25,6 @@ from wagtail.test.testapp.models import (
 )
 from wagtail.test.utils.template_tests import AdminTemplateTestUtils
 from wagtail.test.utils.wagtail_tests import WagtailTestUtils
-from wagtail.utils.deprecation import RemovedInWagtail70Warning
 
 
 class TestModelViewSetGroup(WagtailTestUtils, TestCase):
@@ -239,11 +238,11 @@ class TestCustomColumns(WagtailTestUtils, TestCase):
         self.assertIsNotNone(help)
         self.assertEqual(help.text.strip(), "None")
 
-        success = soup.select_one("td:has(svg.icon-success.w-text-positive-100)")
+        success = soup.select_one("td:has(svg.icon-check.w-text-positive-100)")
         self.assertIsNotNone(success)
         self.assertEqual(success.text.strip(), "True")
 
-        error = soup.select_one("td:has(svg.icon-error.w-text-critical-100)")
+        error = soup.select_one("td:has(svg.icon-cross.w-text-text-error)")
         self.assertIsNotNone(error)
         self.assertEqual(error.text.strip(), "False")
 
@@ -950,41 +949,6 @@ class TestBreadcrumbs(AdminTemplateTestUtils, WagtailTestUtils, TestCase):
         self.assertBreadcrumbsItemsRendered(items, response.content)
 
 
-class TestLegacyPatterns(WagtailTestUtils, TestCase):
-    # RemovedInWagtail70Warning: legacy integer pk-based URLs will be removed
-
-    def setUp(self):
-        self.user = self.login()
-
-    @classmethod
-    def setUpTestData(cls):
-        cls.object = JSONStreamModel.objects.create(
-            body='[{"type": "text", "value": "foo"}]',
-        )
-
-    def test_legacy_edit(self):
-        edit_url = reverse("streammodel:edit", args=(quote(self.object.pk),))
-        legacy_edit_url = "/admin/streammodel/1/"
-        with self.assertWarnsRegex(
-            RemovedInWagtail70Warning,
-            "`/<pk>/` edit view URL pattern has been deprecated in favour of /edit/<pk>/.",
-        ):
-            response = self.client.get(legacy_edit_url)
-        self.assertEqual(edit_url, "/admin/streammodel/edit/1/")
-        self.assertRedirects(response, edit_url, 301)
-
-    def test_legacy_delete(self):
-        delete_url = reverse("streammodel:delete", args=(quote(self.object.pk),))
-        legacy_delete_url = "/admin/streammodel/1/delete/"
-        with self.assertWarnsRegex(
-            RemovedInWagtail70Warning,
-            "`/<pk>/delete/` delete view URL pattern has been deprecated in favour of /delete/<pk>/.",
-        ):
-            response = self.client.get(legacy_delete_url)
-        self.assertEqual(delete_url, "/admin/streammodel/delete/1/")
-        self.assertRedirects(response, delete_url, 301)
-
-
 class TestHistoryView(WagtailTestUtils, TestCase):
     @classmethod
     def setUpTestData(cls):
@@ -1400,7 +1364,7 @@ class TestInspectView(WagtailTestUtils, TestCase):
             reverse("fctoy_alt1:inspect", args=(quote(self.object.pk),))
         )
         expected_fields = ["Name"]
-        expected_values = ["Test Toy"]
+        expected_values = [f"Test Toy ({self.object.pk})"]
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, "wagtailadmin/generic/inspect.html")
         soup = self.get_soup(response.content)
@@ -1501,6 +1465,8 @@ class TestListingButtons(WagtailTestUtils, TestCase):
         actions = soup.select_one("tbody tr td ul.actions")
         more_dropdown = actions.select_one("li [data-controller='w-dropdown']")
         self.assertIsNotNone(more_dropdown)
+        # The aria-label should be on the toggle button, not the dropdown
+        self.assertIsNone(more_dropdown.get("aria-label"))
         more_button = more_dropdown.select_one("button")
         self.assertEqual(
             more_button.attrs.get("aria-label").strip(),
@@ -1510,22 +1476,18 @@ class TestListingButtons(WagtailTestUtils, TestCase):
         expected_buttons = [
             (
                 "Edit",
-                f"Edit '{self.object}'",
                 reverse("feature_complete_toy:edit", args=[quote(self.object.pk)]),
             ),
             (
                 "Copy",
-                f"Copy '{self.object}'",
                 reverse("feature_complete_toy:copy", args=[quote(self.object.pk)]),
             ),
             (
                 "Inspect",
-                f"Inspect '{self.object}'",
                 reverse("feature_complete_toy:inspect", args=[quote(self.object.pk)]),
             ),
             (
                 "Delete",
-                f"Delete '{self.object}'",
                 reverse("feature_complete_toy:delete", args=[quote(self.object.pk)]),
             ),
         ]
@@ -1533,12 +1495,11 @@ class TestListingButtons(WagtailTestUtils, TestCase):
         rendered_buttons = more_dropdown.select("a")
         self.assertEqual(len(rendered_buttons), len(expected_buttons))
 
-        for rendered_button, (label, aria_label, url) in zip(
-            rendered_buttons, expected_buttons
-        ):
+        for rendered_button, (label, url) in zip(rendered_buttons, expected_buttons):
             self.assertEqual(rendered_button.text.strip(), label)
-            self.assertEqual(rendered_button.attrs.get("aria-label"), aria_label)
             self.assertEqual(rendered_button.attrs.get("href"), url)
+            # Should not render aria-label in favor of the button text
+            self.assertIsNone(rendered_button.attrs.get("aria-label"))
 
     def test_title_cell_not_link_to_edit_view_when_no_edit_permission(self):
         self.user.is_superuser = False
@@ -1629,17 +1590,14 @@ class TestListingButtons(WagtailTestUtils, TestCase):
         expected_buttons = [
             (
                 "Edit",
-                f"Edit '{self.object}'",
                 reverse("fctoy_alt1:edit", args=[quote(self.object.pk)]),
             ),
             (
                 "Inspect",
-                f"Inspect '{self.object}'",
                 reverse("fctoy_alt1:inspect", args=[quote(self.object.pk)]),
             ),
             (
                 "Delete",
-                f"Delete '{self.object}'",
                 reverse("fctoy_alt1:delete", args=[quote(self.object.pk)]),
             ),
         ]
@@ -1647,12 +1605,11 @@ class TestListingButtons(WagtailTestUtils, TestCase):
         rendered_buttons = more_dropdown.select("a")
         self.assertEqual(len(rendered_buttons), len(expected_buttons))
 
-        for rendered_button, (label, aria_label, url) in zip(
-            rendered_buttons, expected_buttons
-        ):
+        for rendered_button, (label, url) in zip(rendered_buttons, expected_buttons):
             self.assertEqual(rendered_button.text.strip(), label)
-            self.assertEqual(rendered_button.attrs.get("aria-label"), aria_label)
             self.assertEqual(rendered_button.attrs.get("href"), url)
+            # Should not render aria-label in favor of the button text
+            self.assertIsNone(rendered_button.attrs.get("aria-label"))
 
     def test_dropdown_not_rendered_when_no_child_buttons_exist(self):
         self.user.is_superuser = False
@@ -1871,3 +1828,325 @@ class TestHeaderButtons(WagtailTestUtils, TestCase):
             [(a.text.strip(), a.get("href")) for a in header_buttons],
             expected_buttons,
         )
+
+
+class TestIndexViewReordering(WagtailTestUtils, TestCase):
+    def setUp(self):
+        self.user = self.login()
+        self.obj1 = FeatureCompleteToy.objects.create(name="Toy 1", sort_order=0)
+        self.obj2 = FeatureCompleteToy.objects.create(name="Toy 2", sort_order=1)
+        self.obj3 = FeatureCompleteToy.objects.create(name="Toy 3", sort_order=2)
+
+    def test_header_button_rendered(self):
+        index_url = reverse("feature_complete_toy:index")
+        custom_ordering_url = index_url + "?ordering=sort_order"
+        response = self.client.get(index_url)
+        self.assertEqual(response.status_code, 200)
+        soup = self.get_soup(response.content)
+        button = soup.select_one(
+            f".w-slim-header .w-dropdown a[href='{custom_ordering_url}']"
+        )
+        self.assertIsNotNone(button)
+        self.assertEqual(button.text.strip(), "Sort item order")
+
+        # Reordering feature disabled when not sorting by sort_order
+        table = soup.select_one("main table")
+        self.assertIsNotNone(table)
+        self.assertFalse(table.get("data-controller"))
+        first_th = soup.select_one("main thead th:first-child")
+        self.assertIsNotNone(first_th)
+        self.assertEqual(first_th.text.strip(), "Name")
+
+    def test_show_ordering_column(self):
+        index_url = reverse("feature_complete_toy:index")
+        custom_ordering_url = index_url + "?ordering=sort_order"
+        response = self.client.get(custom_ordering_url)
+        self.assertEqual(response.status_code, 200)
+        soup = self.get_soup(response.content)
+
+        # The table should have the w-orderable controller
+        table = soup.select_one("main table")
+        self.assertIsNotNone(table)
+        self.assertEqual(table.get("data-controller"), "w-orderable")
+        self.assertEqual(
+            table.get("data-w-orderable-message-value"),
+            "'__LABEL__' has been moved successfully.",
+        )
+        self.assertEqual(
+            table.get("data-w-orderable-url-value"),
+            reverse("feature_complete_toy:reorder", args=[999999]),
+        )
+
+        # The ordering column added as the first column
+        first_th = table.select_one("thead th:first-child")
+        self.assertIsNotNone(first_th)
+        self.assertEqual(first_th.text.strip(), "Sort")
+
+        # All rows have the corresponding attributes for reordering
+        rows = table.select("tbody tr")
+        self.assertEqual(len(rows), 3)
+        expected = [
+            {
+                "id": f"item_{quote(toy.pk)}",
+                "data-w-orderable-item-id": quote(toy.pk),
+                "data-w-orderable-item-label": str(toy),
+                "data-w-orderable-target": "item",
+            }
+            for toy in [self.obj1, self.obj2, self.obj3]
+        ]
+        for row, expected_attrs in zip(rows, expected):
+            for attr, value in expected_attrs.items():
+                self.assertEqual(row.get(attr), value)
+            handle = row.select_one("td button[data-w-orderable-target='handle']")
+            self.assertIsNotNone(handle)
+
+    def test_reordering_disabled_with_explicit_sort_order_field_none(self):
+        # The model has a sort_order_field, but the viewset explicitly sets
+        # sort_order_field = None
+        index_url = reverse("fctoy-alt2:index")
+        custom_ordering_url = index_url + "?ordering=sort_order"
+        response = self.client.get(custom_ordering_url)
+        self.assertEqual(response.status_code, 200)
+        soup = self.get_soup(response.content)
+
+        # Header button for enabling reordering should not be rendered
+        button = soup.select_one(
+            f".w-slim-header .w-dropdown a[href='{custom_ordering_url}']"
+        )
+        self.assertIsNone(button)
+
+        # Reordering feature not enabled
+        table = soup.select_one("main table")
+        self.assertIsNotNone(table)
+        self.assertFalse(table.get("data-controller"))
+        first_th = soup.select_one("main thead th:first-child")
+        self.assertIsNotNone(first_th)
+        self.assertEqual(first_th.text.strip(), "Feature complete toy")
+
+    def test_reordering_disabled_with_no_sort_order_field(self):
+        # This model has no sort_order_field defined on the model nor the viewset
+        JSONStreamModel.objects.create()
+        index_url = reverse("streammodel:index")
+        custom_ordering_url = index_url + "?ordering="
+        response = self.client.get(custom_ordering_url)
+        self.assertEqual(response.status_code, 200)
+        soup = self.get_soup(response.content)
+
+        # Header button for enabling reordering should not be rendered
+        button = soup.select_one(
+            f".w-slim-header .w-dropdown a[href^='{custom_ordering_url}']"
+        )
+        self.assertIsNone(button)
+
+        # Reordering feature not enabled
+        table = soup.select_one("main table")
+        self.assertIsNotNone(table)
+        self.assertFalse(table.get("data-controller"))
+        first_th = soup.select_one("main thead th:first-child")
+        self.assertIsNotNone(first_th)
+        self.assertEqual(first_th.text.strip(), "JSON stream model")
+
+    def test_reordering_disabled_with_insufficient_permission(self):
+        self.user.is_superuser = False
+        self.user.save()
+        admin_permission = Permission.objects.get(
+            content_type__app_label="wagtailadmin", codename="access_admin"
+        )
+        view_permission = Permission.objects.get(
+            content_type__app_label=self.obj1._meta.app_label,
+            codename=get_permission_codename("view", self.obj1._meta),
+        )
+        self.user.user_permissions.add(admin_permission, view_permission)
+
+        index_url = reverse("feature_complete_toy:index")
+        custom_ordering_url = index_url + "?ordering=sort_order"
+        response = self.client.get(custom_ordering_url)
+        self.assertEqual(response.status_code, 200)
+        soup = self.get_soup(response.content)
+
+        # Header button for enabling reordering should not be rendered
+        button = soup.select_one(
+            f".w-slim-header .w-dropdown a[href='{custom_ordering_url}']"
+        )
+        self.assertIsNone(button)
+
+        # Reordering feature not enabled
+        table = soup.select_one("main table")
+        self.assertIsNotNone(table)
+        self.assertFalse(table.get("data-controller"))
+        first_th = soup.select_one("main thead th:first-child")
+        self.assertIsNotNone(first_th)
+        self.assertEqual(first_th.text.strip(), "Name")
+
+    def test_minimal_permission(self):
+        self.user.is_superuser = False
+        self.user.save()
+        admin_permission = Permission.objects.get(
+            content_type__app_label="wagtailadmin", codename="access_admin"
+        )
+        change_permission = Permission.objects.get(
+            content_type__app_label=self.obj1._meta.app_label,
+            codename=get_permission_codename("change", self.obj1._meta),
+        )
+        self.user.user_permissions.add(admin_permission, change_permission)
+
+        self.test_header_button_rendered()
+        self.test_show_ordering_column()
+
+
+class TestCreateViewReordering(WagtailTestUtils, TestCase):
+    def setUp(self):
+        self.user = self.login()
+        FeatureCompleteToy.objects.bulk_create(
+            [
+                FeatureCompleteToy(name="Toy 1", sort_order=0),
+                FeatureCompleteToy(name="Toy 2", sort_order=1),
+                FeatureCompleteToy(name="Toy 3", sort_order=2),
+            ]
+        )
+
+    def test_create_sets_max_sort_order(self):
+        response = self.client.post(
+            reverse("feature_complete_toy:add"),
+            data={"name": "New Toy", "release_date": "2025-08-17"},
+        )
+        self.assertRedirects(response, reverse("feature_complete_toy:index"))
+        new_toy = FeatureCompleteToy.objects.get(name="New Toy")
+        self.assertEqual(new_toy.sort_order, 3)
+
+    def test_create_does_not_set_max_sort_order_without_sort_order_field(self):
+        response = self.client.post(
+            reverse("fctoy-alt2:add"),
+            data={"name": "New Toy", "release_date": "2025-08-17", "strid": "foo"},
+        )
+        self.assertRedirects(response, reverse("fctoy-alt2:index"))
+        new_toy = FeatureCompleteToy.objects.get(name="New Toy")
+        self.assertIsNone(new_toy.sort_order)
+
+
+class TestReorderView(WagtailTestUtils, TestCase):
+    def setUp(self):
+        self.user = self.login()
+        # We don't do any normalization, so the sort_order values may not be
+        # consecutive integers (e.g. after an item is deleted), and the update
+        # logic may cause the sort_order values to be negative or larger than
+        # the number of items in the queryset.
+        self.obj1 = FeatureCompleteToy.objects.create(name="Toy 1", sort_order=-3)
+        self.obj2 = FeatureCompleteToy.objects.create(name="Toy 2", sort_order=4)
+        self.obj3 = FeatureCompleteToy.objects.create(name="Toy 3", sort_order=9)
+
+    def get_url(self, obj):
+        return reverse("feature_complete_toy:reorder", args=(quote(obj.pk),))
+
+    def assertOrder(self, objs):
+        self.assertSequenceEqual(
+            [
+                (obj, obj.sort_order)
+                for obj in FeatureCompleteToy.objects.order_by("sort_order")
+            ],
+            objs,
+        )
+
+    def test_reorder_view_disabled_without_sort_order_field(self):
+        # The alt2 viewset explicitly sets sort_order_field to `None`,
+        # so the reorder view should not be available
+        with self.assertRaises(NoReverseMatch):
+            reverse("fctoy-alt2:reorder", args=(quote(self.obj1.pk),))
+
+        # This model has no sort_order_field on the model nor the viewset
+        obj = JSONStreamModel.objects.create()
+        with self.assertRaises(NoReverseMatch):
+            reverse("streammodel:reorder", args=(quote(obj.pk),))
+
+    def test_get_request_does_not_alter_order(self):
+        response = self.client.get(self.get_url(self.obj1))
+        self.assertEqual(response.status_code, 405)
+
+        # Ensure item order does not change
+        self.assertOrder([(self.obj1, -3), (self.obj2, 4), (self.obj3, 9)])
+
+    def test_post_request_without_position_argument_moves_to_the_end(self):
+        response = self.client.post(self.get_url(self.obj1))
+        self.assertEqual(response.status_code, 200)
+
+        # The item will be moved to the last position by taking the sort_order
+        # of the last item, and the sort_order of the other items updated by -1
+        self.assertOrder([(self.obj2, 3), (self.obj3, 8), (self.obj1, 9)])
+
+    def test_post_request_with_non_integer_position_moves_to_the_end(self):
+        response = self.client.post(self.get_url(self.obj1) + "?position=good")
+        self.assertEqual(response.status_code, 200)
+
+        # The item will be moved to the last position by taking the sort_order
+        # of the last item, and the sort_order of the other items updated by -1
+        self.assertOrder([(self.obj2, 3), (self.obj3, 8), (self.obj1, 9)])
+
+    def test_move_position_up(self):
+        # Move obj3 to the first position
+        response = self.client.post(self.get_url(self.obj3) + "?position=0")
+        self.assertEqual(response.status_code, 200)
+
+        # Check if obj3 is now the first item by taking obj1's sort_order and
+        # incrementing sort_order of the other items after it by 1
+        self.assertOrder([(self.obj3, -3), (self.obj1, -2), (self.obj2, 5)])
+
+    def test_move_position_down(self):
+        # Move obj1 to the second position
+        response = self.client.post(self.get_url(self.obj1) + "?position=1")
+        self.assertEqual(response.status_code, 200)
+
+        # Check if obj1 is now the second item by taking obj2's sort_order
+        # and decrementing sort_order of the other items before it by 1
+        self.assertOrder([(self.obj2, 3), (self.obj1, 4), (self.obj3, 9)])
+
+    def test_move_position_to_same_position(self):
+        # Move obj1 to position 0 (where it already is)
+        response = self.client.post(self.get_url(self.obj1) + "?position=0")
+        self.assertEqual(response.status_code, 200)
+
+        # Ensure item order does not change
+        self.assertOrder([(self.obj1, -3), (self.obj2, 4), (self.obj3, 9)])
+
+    def test_move_position_with_invalid_target_position(self):
+        response = self.client.post(self.get_url(self.obj1) + "?position=99")
+        self.assertEqual(response.status_code, 200)
+
+        # The item will be moved to the last position by taking the sort_order
+        # of the last item, and the sort_order of the other items updated by -1
+        self.assertOrder([(self.obj2, 3), (self.obj3, 8), (self.obj1, 9)])
+
+    def test_insufficient_permission(self):
+        self.user.is_superuser = False
+        self.user.save()
+        admin_permission = Permission.objects.get(
+            content_type__app_label="wagtailadmin", codename="access_admin"
+        )
+        view_permission = Permission.objects.get(
+            content_type__app_label=self.obj1._meta.app_label,
+            codename=get_permission_codename("view", self.obj1._meta),
+        )
+        self.user.user_permissions.add(admin_permission, view_permission)
+
+        response = self.client.post(self.get_url(self.obj1) + "?position=1")
+        self.assertEqual(response.status_code, 302)
+        self.assertRedirects(response, reverse("wagtailadmin_home"))
+        self.assertOrder([(self.obj1, -3), (self.obj2, 4), (self.obj3, 9)])
+
+    def test_minimal_permission(self):
+        self.user.is_superuser = False
+        self.user.save()
+        admin_permission = Permission.objects.get(
+            content_type__app_label="wagtailadmin", codename="access_admin"
+        )
+        change_permission = Permission.objects.get(
+            content_type__app_label=self.obj1._meta.app_label,
+            codename=get_permission_codename("change", self.obj1._meta),
+        )
+        self.user.user_permissions.add(admin_permission, change_permission)
+
+        response = self.client.post(self.get_url(self.obj1) + "?position=1")
+        self.assertEqual(response.status_code, 200)
+
+        # Check if obj1 is now the second item by taking obj2's sort_order
+        # and decrementing sort_order of the other items before it by 1
+        self.assertOrder([(self.obj2, 3), (self.obj1, 4), (self.obj3, 9)])
