@@ -4,6 +4,7 @@ from collections import OrderedDict
 from collections.abc import Mapping
 
 from django.contrib.admin.utils import quote
+from django.contrib.humanize.templatetags.humanize import intcomma
 from django.forms import MediaDefiningClass
 from django.template.loader import get_template
 from django.templatetags.l10n import unlocalize
@@ -13,7 +14,8 @@ from django.utils.text import capfirst
 from django.utils.translation import gettext, gettext_lazy
 
 from wagtail.admin.ui.components import Component
-from wagtail.coreutils import multigetattr
+from wagtail.coreutils import get_locales_display_names, multigetattr
+from wagtail.models import Locale
 
 
 class BaseColumn(metaclass=MediaDefiningClass):
@@ -141,6 +143,7 @@ class Column(BaseColumn):
     """A column that displays a single field of data from the model"""
 
     cell_template_name = "wagtailadmin/tables/cell.html"
+    empty_value_display = ""
 
     def get_value(self, instance):
         """
@@ -157,13 +160,26 @@ class Column(BaseColumn):
 
     def get_cell_context_data(self, instance, parent_context):
         context = super().get_cell_context_data(instance, parent_context)
-        value = self.get_value(instance)
+        context["raw_value"] = value = self.get_value(instance)
         if isinstance(value, int) and not isinstance(value, bool):
             # To prevent errors arising from USE_THOUSAND_SEPARATOR, we require all numbers output
             # on templates to be explicitly localized or unlocalized. For numeric table cells, we
             # unlocalize them by default; developers may subclass Column to obtain formatted numbers.
             value = unlocalize(value)
-        context["value"] = value
+
+        if not str(value).strip():
+            context["value"] = self.empty_value_display
+        else:
+            context["value"] = value
+        return context
+
+
+class NumberColumn(Column):
+    """A specialised column that displays numbers with locale-aware formatting"""
+
+    def get_cell_context_data(self, instance, parent_context):
+        context = super().get_cell_context_data(instance, parent_context)
+        context["value"] = intcomma(context["raw_value"])
         return context
 
 
@@ -185,6 +201,7 @@ class TitleColumn(Column):
     """A column where data is styled as a title and wrapped in a link or <label>"""
 
     cell_template_name = "wagtailadmin/tables/title_cell.html"
+    empty_value_display = gettext_lazy("(blank)")
 
     def __init__(
         self,
@@ -299,6 +316,31 @@ class LiveStatusTagColumn(StatusTagColumn):
         )
 
 
+class LocaleColumn(Column):
+    """Represents a Locale label."""
+
+    cell_template_name = "wagtailadmin/tables/locale_cell.html"
+
+    def __init__(self, **kwargs):
+        super().__init__(
+            "locale_id",
+            label=kwargs.pop("label", gettext("Locale")),
+            sort_key=kwargs.pop("sort_key", "locale"),
+            classname=kwargs.pop("classname", "w-text-16"),
+            **kwargs,
+        )
+
+    def get_cell_context_data(self, instance, parent_context):
+        context = super().get_cell_context_data(instance, parent_context)
+        value = self.get_value(instance)
+        if isinstance(value, int):
+            value = get_locales_display_names().get(value)
+        elif isinstance(value, Locale):
+            value = value.get_display_name()
+        context["value"] = value
+        return context
+
+
 class DateColumn(Column):
     """Outputs a date in human-readable format"""
 
@@ -371,6 +413,10 @@ class BulkActionsCheckboxColumn(BaseColumn):
         return context
 
 
+class UsageCountColumn(Column):
+    cell_template_name = "wagtailadmin/tables/usage_count_column_cell.html"
+
+
 class ReferencesColumn(Column):
     cell_template_name = "wagtailadmin/tables/references_cell.html"
 
