@@ -3,11 +3,12 @@ from abc import ABC, abstractmethod
 from django import forms
 from django.db import transaction
 from django.shortcuts import get_list_or_404, redirect
+from django.utils.functional import classproperty
 from django.views.generic import FormView
 
 from wagtail import hooks
 from wagtail.admin import messages
-from wagtail.admin.views.pages.utils import get_valid_next_url_from_request
+from wagtail.admin.utils import get_valid_next_url_from_request
 
 
 class BulkAction(ABC, FormView):
@@ -28,7 +29,6 @@ class BulkAction(ABC, FormView):
 
     extras = {}
     action_priority = 100
-    models = []
     classes = set()
 
     form_class = forms.Form
@@ -50,6 +50,10 @@ class BulkAction(ABC, FormView):
                 )
             )
 
+    @classproperty
+    def models(cls):
+        return []
+
     @classmethod
     def get_queryset(cls, model, object_ids):
         return get_list_or_404(model, pk__in=object_ids)
@@ -69,8 +73,9 @@ class BulkAction(ABC, FormView):
 
     @classmethod
     def get_default_model(cls):
-        if len(cls.models) == 1:
-            return cls.models[0]
+        models = cls.models
+        if len(models) == 1:
+            return models[0]
         raise Exception(
             "Cannot get default model if number of models is greater than 1"
         )
@@ -106,8 +111,12 @@ class BulkAction(ABC, FormView):
                 objects.append(obj)
         return objects, {"items_with_no_access": items_with_no_access}
 
+    def annotate_items(self, items):
+        return items
+
     def get_context_data(self, **kwargs):
         items, items_with_no_access = self.get_actionable_objects()
+        items = self.annotate_items(items)
         _items = []
         for item in items:
             _items.append(self.object_context(item))
@@ -117,6 +126,7 @@ class BulkAction(ABC, FormView):
             **items_with_no_access,
             "next": self.next_url,
             "submit_url": self.request.path + "?" + self.request.META["QUERY_STRING"],
+            "model_opts": self.model._meta,
         }
 
     def prepare_action(self, objects, objects_without_access):
@@ -129,6 +139,7 @@ class BulkAction(ABC, FormView):
         request = self.request
         self.cleaned_form = form
         objects, objects_without_access = self.get_actionable_objects()
+        self.actionable_objects = self.annotate_items(objects)
         resp = self.prepare_action(objects, objects_without_access)
         if hasattr(resp, "status_code"):
             return resp
