@@ -24,7 +24,7 @@ class PublishBulkAction(PageBulkAction):
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
         context["has_draft_descendants"] = any(
-            map(lambda x: x["draft_descendant_count"], context["items"])
+            item["draft_descendant_count"] for item in context["items"]
         )
         return context
 
@@ -48,7 +48,11 @@ class PublishBulkAction(PageBulkAction):
 
             if include_descendants:
                 for draft_descendant_page in (
-                    page.get_descendants().not_live().defer_streamfields().specific()
+                    page.get_descendants()
+                    .not_live()
+                    .defer_streamfields()
+                    .specific()
+                    .iterator()
                 ):
                     if (
                         user is None
@@ -67,35 +71,15 @@ class PublishBulkAction(PageBulkAction):
 
     def get_success_message(self, num_parent_objects, num_child_objects):
         include_descendants = self.cleaned_form.cleaned_data["include_descendants"]
-        if num_parent_objects == 1:
-            if include_descendants:
-                if num_child_objects == 0:
-                    success_message = _("1 page has been published")
-                else:
-                    success_message = ngettext(
-                        "1 page and %(num_child_objects)d child page have been published",
-                        "1 page and %(num_child_objects)d child pages have been published",
-                        num_child_objects,
-                    ) % {"num_child_objects": num_child_objects}
-            else:
-                success_message = _("1 page has been published")
+        if include_descendants and num_child_objects > 0:
+            # Translators: This forms a message such as "1 page and 3 child pages have been published"
+            return _("%(parent_pages)s and %(child_pages)s have been published") % {
+                "parent_pages": self.get_parent_page_text(num_parent_objects),
+                "child_pages": self.get_child_page_text(num_child_objects),
+            }
         else:
-            if include_descendants:
-                if num_child_objects == 0:
-                    success_message = _(
-                        "%(num_parent_objects)d pages have been published"
-                    ) % {"num_parent_objects": num_parent_objects}
-                else:
-                    success_message = ngettext(
-                        "%(num_parent_objects)d pages and %(num_child_objects)d child page have been published",
-                        "%(num_parent_objects)d pages and %(num_child_objects)d child pages have been published",
-                        num_child_objects,
-                    ) % {
-                        "num_child_objects": num_child_objects,
-                        "num_parent_objects": num_parent_objects,
-                    }
-            else:
-                success_message = _(
-                    "%(num_parent_objects)d pages have been published"
-                ) % {"num_parent_objects": num_parent_objects}
-        return success_message
+            return ngettext(
+                "%(num_parent_objects)d page has been published",
+                "%(num_parent_objects)d pages have been published",
+                num_parent_objects,
+            ) % {"num_parent_objects": num_parent_objects}
