@@ -1,5 +1,6 @@
 from django.shortcuts import redirect
 from django.template.response import TemplateResponse
+from django.urls import reverse
 from django.utils.translation import gettext as _
 
 from wagtail import hooks
@@ -8,7 +9,7 @@ from wagtail.actions.create_alias import CreatePageAliasAction
 from wagtail.admin import messages
 from wagtail.admin.auth import user_has_any_page_permission, user_passes_test
 from wagtail.admin.forms.pages import CopyForm
-from wagtail.admin.views.pages.utils import get_valid_next_url_from_request
+from wagtail.admin.utils import get_valid_next_url_from_request
 from wagtail.models import Page
 
 
@@ -22,8 +23,8 @@ def copy(request, page_id):
     # Check if the user has permission to publish subpages on the parent
     can_publish = parent_page.permissions_for_user(request.user).can_publish_subpage()
 
-    # Create the form
-    form = CopyForm(
+    form_class = getattr(page.specific_class, "copy_form_class", CopyForm)
+    form = form_class(
         request.POST or None, user=request.user, page=page, can_publish=can_publish
     )
 
@@ -78,20 +79,26 @@ def copy(request, page_id):
                 new_page = action.execute()
 
             # Give a success message back to the user
+            edit_button = messages.button(
+                reverse("wagtailadmin_pages:edit", args=(new_page.id,)),
+                _("Edit"),
+            )
             if form.cleaned_data.get("copy_subpages"):
                 messages.success(
                     request,
-                    _("Page '{0}' and {1} subpages copied.").format(
-                        page.specific_deferred.get_admin_display_title(),
-                        new_page.get_descendants().count(),
-                    ),
+                    _("Page '%(page_title)s' and %(subpages_count)s subpages copied.")
+                    % {
+                        "page_title": page.specific_deferred.get_admin_display_title(),
+                        "subpages_count": new_page.get_descendants().count(),
+                    },
+                    buttons=[edit_button],
                 )
             else:
                 messages.success(
                     request,
-                    _("Page '{0}' copied.").format(
-                        page.specific_deferred.get_admin_display_title()
-                    ),
+                    _("Page '%(page_title)s' copied.")
+                    % {"page_title": page.specific_deferred.get_admin_display_title()},
+                    buttons=[edit_button],
                 )
 
             for fn in hooks.get_hooks("after_copy_page"):
