@@ -8,15 +8,32 @@ from django.shortcuts import get_object_or_404, redirect
 from django.template.response import TemplateResponse
 from django.urls import reverse
 from django.utils import timezone
+from django.utils.functional import cached_property
 from django.utils.translation import gettext as _
-from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
+from django.utils.translation import gettext_lazy
+from django.views.generic.base import View
 
-from wagtail.admin import messages, signals
+from wagtail.admin import messages
 from wagtail.admin.action_menu import PageActionMenu
-from wagtail.admin.ui.side_panels import PageSidePanels
+from wagtail.admin.telepath import JSContext
+from wagtail.admin.ui.components import MediaContainer
+from wagtail.admin.ui.side_panels import (
+    ChecksSidePanel,
+    CommentsSidePanel,
+    PageStatusSidePanel,
+    PreviewSidePanel,
+)
+from wagtail.admin.utils import get_valid_next_url_from_request
 from wagtail.admin.views.generic import HookResponseMixin
-from wagtail.admin.views.pages.utils import get_valid_next_url_from_request
-from wagtail.models import Locale, Page, PageSubscription, UserPagePermissionsProxy
+from wagtail.admin.views.generic.base import WagtailAdminTemplateMixin
+from wagtail.models import (
+    BaseViewRestriction,
+    Locale,
+    Page,
+    PageSubscription,
+    PageViewRestriction,
+)
+from wagtail.signals import init_new_page
 
 
 def add_subpage(request, parent_page_id):
@@ -54,8 +71,9 @@ def add_subpage(request, parent_page_id):
     )
 
 
-class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
+class CreateView(WagtailAdminTemplateMixin, HookResponseMixin, View):
     template_name = "wagtailadmin/pages/create.html"
+    page_title = gettext_lazy("New")
 
     def dispatch(
         self, request, content_type_app_name, content_type_model_name, parent_page_id
@@ -95,17 +113,25 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
             return response
 
         self.locale = self.parent_page.locale
-
-        # If the parent page is the root page. The user may specify any locale they like
-        if self.parent_page.is_root():
-            selected_locale = request.GET.get("locale", None) or request.POST.get(
-                "locale", None
-            )
-            if selected_locale:
-                self.locale = get_object_or_404(Locale, language_code=selected_locale)
-
         self.page = self.page_class(owner=self.request.user)
         self.page.locale = self.locale
+
+        if getattr(settings, "WAGTAIL_I18N_ENABLED", False):
+            # If the parent page is the root page. The user may specify any locale they like
+            if self.parent_page.is_root():
+                selected_locale = request.GET.get("locale", None) or request.POST.get(
+                    "locale", None
+                )
+                if selected_locale:
+                    self.locale = get_object_or_404(
+                        Locale, language_code=selected_locale
+                    )
+            self.page.locale = self.locale
+            self.translations = self.get_translations()
+        else:
+            self.locale = None
+            self.translations = []
+
         self.edit_handler = self.page_class.get_edit_handler()
         self.form_class = self.edit_handler.get_form_class()
 
@@ -127,25 +153,43 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
             parent_page=self.parent_page,
             for_user=self.request.user,
         )
+        if self.action_name == "save":
+            self.form.defer_required_fields()
 
         if self.form.is_valid():
             return self.form_valid(self.form)
         else:
+            self.form.restore_required_fields()
             return self.form_invalid(self.form)
 
-    def form_valid(self, form):
+    @cached_property
+    def action_name_and_method(self):
         if (
             bool(self.request.POST.get("action-publish"))
             and self.parent_page_perms.can_publish_subpage()
         ):
-            return self.publish_action()
+            return ("publish", self.publish_action)
         elif (
             bool(self.request.POST.get("action-submit"))
             and self.parent_page.has_workflow
         ):
-            return self.submit_action()
+            return ("submit", self.submit_action)
         else:
-            return self.save_action()
+            return ("save", self.save_action)
+
+    @property
+    def action_name(self):
+        return self.action_name_and_method[0]
+
+    @property
+    def action_method(self):
+        return self.action_name_and_method[1]
+
+    def form_valid(self, form):
+        return self.action_method()
+
+    def get_page_subtitle(self):
+        return self.page_class.get_verbose_name()
 
     def get_edit_message_button(self):
         return messages.button(
@@ -162,6 +206,37 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
     def get_view_live_message_button(self):
         return messages.button(self.page.url, _("View live"), new_window=False)
 
+    def set_default_privacy_setting(self):
+        # privacy setting options BaseViewRestriction.RESTRICTION_CHOICES
+        default_privacy_setting = self.page.get_default_privacy_setting(self.request)
+
+        if default_privacy_setting["type"] == BaseViewRestriction.NONE:
+            # default privacy setting is public no need to do anything
+            pass
+        elif default_privacy_setting["type"] == BaseViewRestriction.LOGIN:
+            PageViewRestriction.objects.create(
+                page=self.page,
+                restriction_type=BaseViewRestriction.LOGIN,
+            )
+        elif default_privacy_setting["type"] == BaseViewRestriction.PASSWORD:
+            PageViewRestriction.objects.create(
+                page=self.page,
+                restriction_type=BaseViewRestriction.PASSWORD,
+                password=default_privacy_setting["password"],
+            )
+        elif default_privacy_setting["type"] == BaseViewRestriction.GROUPS:
+            # Create a page view restriction for groups
+            groups_page_restriction = PageViewRestriction.objects.create(
+                page=self.page,
+                restriction_type=BaseViewRestriction.GROUPS,
+            )
+            # add groups to the page view restriction
+            groups_page_restriction.groups.set(default_privacy_setting["groups"])
+        else:
+            raise ValueError(
+                f"Invalid privacy setting {default_privacy_setting.get('type')!r}"
+            )
+
     def save_action(self):
         self.page = self.form.save(commit=False)
         self.page.live = False
@@ -169,8 +244,11 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
         # Save page
         self.parent_page.add_child(instance=self.page)
 
+        # Set page privacy setting
+        self.set_default_privacy_setting()
+
         # Save revision
-        self.page.save_revision(user=self.request.user, log_action=False)
+        self.page.save_revision(user=self.request.user, log_action=True, clean=False)
 
         # Save subscription settings
         self.subscription.page = self.page
@@ -179,7 +257,8 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
         # Notification
         messages.success(
             self.request,
-            _("Page '{0}' created.").format(self.page.get_admin_display_title()),
+            _("Page '%(page_title)s' created.")
+            % {"page_title": self.page.get_admin_display_title()},
         )
 
         response = self.run_hook("after_create_page", self.request, self.page)
@@ -195,8 +274,11 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
         # Save page
         self.parent_page.add_child(instance=self.page)
 
+        # Set page privacy setting
+        self.set_default_privacy_setting()
+
         # Save revision
-        revision = self.page.save_revision(user=self.request.user, log_action=False)
+        revision = self.page.save_revision(user=self.request.user, log_action=True)
 
         # Save subscription settings
         self.subscription.page = self.page
@@ -220,9 +302,8 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
         if self.page.go_live_at and self.page.go_live_at > timezone.now():
             messages.success(
                 self.request,
-                _("Page '{0}' created and scheduled for publishing.").format(
-                    self.page.get_admin_display_title()
-                ),
+                _("Page '%(page_title)s' created and scheduled for publishing.")
+                % {"page_title": self.page.get_admin_display_title()},
                 buttons=[self.get_edit_message_button()],
             )
         else:
@@ -232,9 +313,8 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
             buttons.append(self.get_edit_message_button())
             messages.success(
                 self.request,
-                _("Page '{0}' created and published.").format(
-                    self.page.get_admin_display_title()
-                ),
+                _("Page '%(page_title)s' created and published.")
+                % {"page_title": self.page.get_admin_display_title()},
                 buttons=buttons,
             )
 
@@ -251,8 +331,11 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
         # Save page
         self.parent_page.add_child(instance=self.page)
 
+        # Set page privacy setting
+        self.set_default_privacy_setting()
+
         # Save revision
-        self.page.save_revision(user=self.request.user, log_action=False)
+        self.page.save_revision(user=self.request.user, log_action=True)
 
         # Submit
         workflow = self.page.get_workflow()
@@ -271,9 +354,8 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
 
         messages.success(
             self.request,
-            _("Page '{0}' created and submitted for moderation.").format(
-                self.page.get_admin_display_title()
-            ),
+            _("Page '%(page_title)s' created and submitted for moderation.")
+            % {"page_title": self.page.get_admin_display_title()},
             buttons=buttons,
         )
 
@@ -301,7 +383,7 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
     def form_invalid(self, form):
         messages.validation_error(
             self.request,
-            _("The page could not be created due to validation errors"),
+            _("The page could not be created due to validation errors."),
             self.form,
         )
         self.has_unsaved_changes = True
@@ -309,9 +391,7 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
         return self.render_to_response(self.get_context_data())
 
     def get(self, request):
-        signals.init_new_page.send(
-            sender=CreateView, page=self.page, parent=self.parent_page
-        )
+        init_new_page.send(sender=CreateView, page=self.page, parent=self.parent_page)
         self.form = self.form_class(
             instance=self.page,
             subscription=self.subscription,
@@ -322,6 +402,43 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
 
         return self.render_to_response(self.get_context_data())
 
+    def get_preview_url(self):
+        return reverse(
+            "wagtailadmin_pages:preview_on_add",
+            args=[
+                self.page_content_type.app_label,
+                self.page_content_type.model,
+                self.parent_page.id,
+            ],
+        )
+
+    def get_side_panels(self):
+        side_panels = [
+            PageStatusSidePanel(
+                self.page,
+                self.request,
+                show_schedule_publishing_toggle=self.form.show_schedule_publishing_toggle,
+                locale=self.locale,
+                translations=self.translations,
+                parent_page=self.parent_page,
+            ),
+        ]
+        if self.page.is_previewable():
+            side_panels.append(
+                PreviewSidePanel(
+                    self.page, self.request, preview_url=self.get_preview_url()
+                )
+            )
+            side_panels.append(
+                ChecksSidePanel(
+                    self.page,
+                    self.request,
+                )
+            )
+        if self.form.show_comments_toggle:
+            side_panels.append(CommentsSidePanel(self.page, self.request))
+        return MediaContainer(side_panels)
+
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
         bound_panel = self.edit_handler.get_bound_panel(
@@ -334,12 +451,14 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
             lock=None,
             locked_for_user=False,
         )
-        side_panels = PageSidePanels(
-            self.request,
-            self.page,
-            preview_enabled=True,
-            comments_enabled=self.form.show_comments_toggle,
-        )
+        side_panels = self.get_side_panels()
+
+        js_context = JSContext()
+        edit_handler_data = js_context.pack(bound_panel)
+
+        media = MediaContainer(
+            [bound_panel, self.form, action_menu, side_panels, js_context]
+        ).media
 
         context.update(
             {
@@ -347,68 +466,58 @@ class CreateView(TemplateResponseMixin, ContextMixin, HookResponseMixin, View):
                 "page_class": self.page_class,
                 "parent_page": self.parent_page,
                 "edit_handler": bound_panel,
+                "edit_handler_data": edit_handler_data,
                 "action_menu": action_menu,
                 "side_panels": side_panels,
                 "form": self.form,
                 "next": self.next_url,
                 "has_unsaved_changes": self.has_unsaved_changes,
-                "locale": None,
-                "translations": [],
-                "media": bound_panel.media
-                + self.form.media
-                + action_menu.media
-                + side_panels.media,
+                "locale": self.locale,
+                "media": media,
             }
         )
 
-        if getattr(settings, "WAGTAIL_I18N_ENABLED", False):
-            # Pages can be created in any language at the root level
-            if self.parent_page.is_root():
-                translations = [
-                    {
-                        "locale": locale,
-                        "url": reverse(
-                            "wagtailadmin_pages:add",
-                            args=[
-                                self.page_content_type.app_label,
-                                self.page_content_type.model,
-                                self.parent_page.id,
-                            ],
-                        )
-                        + "?"
-                        + urlencode({"locale": locale.language_code}),
-                    }
-                    for locale in Locale.objects.all()
-                ]
-
-            else:
-                user_perms = UserPagePermissionsProxy(self.request.user)
-                translations = [
-                    {
-                        "locale": translation.locale,
-                        "url": reverse(
-                            "wagtailadmin_pages:add",
-                            args=[
-                                self.page_content_type.app_label,
-                                self.page_content_type.model,
-                                translation.id,
-                            ],
-                        ),
-                    }
-                    for translation in self.parent_page.get_translations()
-                    .only("id", "locale")
-                    .select_related("locale")
-                    if user_perms.for_page(translation).can_add_subpage()
-                    and self.page_class
-                    in translation.specific_class.creatable_subpage_models()
-                    and self.page_class.can_create_at(translation)
-                ]
-
-            context.update(
-                {
-                    "locale": self.locale,
-                    "translations": translations,
-                }
-            )
-
         return context
+
+    def get_translations(self):
+        # Pages can be created in any language at the root level
+        if self.parent_page.is_root():
+            return [
+                {
+                    "locale": locale,
+                    "url": reverse(
+                        "wagtailadmin_pages:add",
+                        args=[
+                            self.page_content_type.app_label,
+                            self.page_content_type.model,
+                            self.parent_page.id,
+                        ],
+                    )
+                    + "?"
+                    + urlencode({"locale": locale.language_code}),
+                }
+                # Do not show the switcher for the current locale
+                for locale in Locale.objects.exclude(pk=self.locale.pk)
+            ]
+
+        else:
+            return [
+                {
+                    "locale": translation.locale,
+                    "url": reverse(
+                        "wagtailadmin_pages:add",
+                        args=[
+                            self.page_content_type.app_label,
+                            self.page_content_type.model,
+                            translation.id,
+                        ],
+                    ),
+                }
+                for translation in self.parent_page.get_translations()
+                .only("id", "locale")
+                .select_related("locale")
+                if translation.permissions_for_user(self.request.user).can_add_subpage()
+                and self.page_class
+                in translation.specific_class.creatable_subpage_models()
+                and self.page_class.can_create_at(translation)
+            ]
