@@ -23,14 +23,14 @@ from wagtail.admin.ui.components import MediaContainer
 from wagtail.admin.ui.side_panels import (
     PageStatusSidePanel,
 )
-from wagtail.admin.ui.tables import Column, DateColumn
+from wagtail.admin.ui.tables import DateColumn
 from wagtail.admin.ui.tables.pages import (
     BulkActionsColumn,
     NavigateToChildrenColumn,
-    OrderingColumn,
     PageStatusColumn,
     PageTable,
     PageTitleColumn,
+    PageTypeColumn,
     ParentPageColumn,
 )
 from wagtail.admin.views import generic
@@ -147,11 +147,11 @@ class PageListingMixin:
             sort_key="latest_revision_created_at",
             width="12%",
         ),
-        Column(
+        PageTypeColumn(
             "type",
             label=_("Type"),
             accessor="page_type_display_name",
-            sort_key="content_type",
+            sort_key="content_type__model",
             width="12%",
         ),
         PageStatusColumn(
@@ -309,6 +309,9 @@ class ExplorableIndexView(IndexView):
     index_results_url_name = "wagtailadmin_explore_results"
     page_title = _("Exploring")
     filterset_class = GenericPageFilterSet
+    # This is not a real field on the model, but it allows reuse of ordering
+    # logic from generic IndexView
+    sort_order_field = "ord"
 
     @classproperty
     def columns(cls):
@@ -393,39 +396,14 @@ class ExplorableIndexView(IndexView):
         if permissions.can_view_revisions():
             return reverse("wagtailadmin_pages:history", args=[self.parent_page.id])
 
+    def get_reorder_url(self):
+        return reverse("wagtailadmin_pages:set_page_position", args=[999999])
+
     def get_table_kwargs(self):
         kwargs = super().get_table_kwargs()
-        kwargs["use_row_ordering_attributes"] = self.show_ordering_column
         kwargs["parent_page"] = self.parent_page
-
-        if self.show_ordering_column:
-            kwargs["caption"] = _(
-                "Focus on the drag button and press up or down arrows to move the item, then press enter to submit the change."
-            )
-            kwargs["attrs"] = {
-                "data-controller": "w-orderable",
-                "data-w-orderable-active-class": "w-orderable--active",
-                "data-w-orderable-chosen-class": "w-orderable__item--active",
-                "data-w-orderable-container-value": "tbody",
-                "data-w-orderable-message-value": _(
-                    "'%(page_title)s' has been moved successfully."
-                )
-                % {"page_title": "__LABEL__"},
-                "data-w-orderable-url-value": reverse(
-                    "wagtailadmin_pages:set_page_position", args=[999999]
-                ),
-            }
         return kwargs
 
-    def get_valid_orderings(self):
-        valid_orderings = super().get_valid_orderings()
-
-        if not self.is_searching:
-            # ordering by page order is only available when not searching
-            valid_orderings.append("ord")
-
-        return valid_orderings
-
     def get_ordering(self):
         if self.is_searching and not self.is_explicitly_ordered:
             # default to ordering by relevance
@@ -439,22 +417,10 @@ class ExplorableIndexView(IndexView):
 
         return ordering
 
-    def get_paginate_by(self, queryset):
-        if self.ordering == "ord":
-            # Don't paginate if sorting by page order - all pages must be shown to
-            # allow drag-and-drop reordering
-            return None
-        else:
-            return self.paginate_by
-
     def get_page_subtitle(self):
         return self.parent_page.get_admin_display_title()
 
     def get_context_data(self, **kwargs):
-        self.show_ordering_column = self.ordering == "ord"
-        if self.show_ordering_column:
-            self.columns = self.columns.copy()
-            self.columns[0] = OrderingColumn("ordering", width="80px", sort_key="ord")
         context = super().get_context_data(**kwargs)
 
         if self.is_searching:
