@@ -4,6 +4,8 @@ from django.shortcuts import get_object_or_404, redirect
 from django.template.response import TemplateResponse
 from django.urls import reverse
 from django.utils.translation import gettext as _
+from django.views.generic.base import TemplateView
+from django.views.generic.edit import FormMixin
 
 from wagtail import hooks
 from wagtail.actions.move_page import MovePageAction
@@ -12,39 +14,51 @@ from wagtail.admin.forms.pages import MoveForm
 from wagtail.models import Page
 
 
-def move_choose_destination(request, page_to_move_id):
-    page_to_move = get_object_or_404(Page, id=page_to_move_id)
-    page_perms = page_to_move.permissions_for_user(request.user)
-    if not page_perms.can_move():
-        raise PermissionDenied
+class MoveChooseDestination(TemplateView, FormMixin):
+    template_name = "wagtailadmin/pages/move_choose_destination.html"
+    form_class = MoveForm
 
-    target_parent_models = set(page_to_move.specific_class.allowed_parent_page_models())
+    def setup(self, request, *args, **kwargs):
+        super().setup(request, *args, **kwargs)
+        self.page_to_move = get_object_or_404(Page, id=kwargs["page_to_move_id"])
+        self.page_perms = self.page_to_move.permissions_for_user(request.user)
 
-    move_form = MoveForm(
-        request.POST or None,
-        page_to_move=page_to_move,
-        target_parent_models=target_parent_models,
-    )
+        if not self.page_perms.can_move():
+            raise PermissionDenied
 
-    if request.method == "POST":
-        if move_form.is_valid():
-            # Receive the new parent page (this should never be empty)
-            if move_form.cleaned_data["new_parent_page"]:
-                new_parent_page = move_form.cleaned_data["new_parent_page"]
-                return redirect(
-                    "wagtailadmin_pages:move_confirm",
-                    page_to_move.id,
-                    new_parent_page.id,
-                )
+    def get_form_kwargs(self):
+        kwargs = super().get_form_kwargs()
+        target_parent_models = set(
+            self.page_to_move.specific_class.allowed_parent_page_models()
+        )
+        kwargs["target_parent_models"] = target_parent_models
+        kwargs["page_to_move"] = self.page_to_move
 
-    return TemplateResponse(
-        request,
-        "wagtailadmin/pages/move_choose_destination.html",
-        {
-            "page_to_move": page_to_move,
-            "move_form": move_form,
-        },
-    )
+        return kwargs
+
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context["page_to_move"] = self.page_to_move
+        context["move_form"] = self.get_form()
+
+        return context
+
+    def form_valid(self, form):
+        # Receive the new parent page (this should never be empty)
+        if form.cleaned_data["new_parent_page"]:
+            new_parent_page = form.cleaned_data["new_parent_page"]
+            return redirect(
+                "wagtailadmin_pages:move_confirm",
+                self.page_to_move.id,
+                new_parent_page.id,
+            )
+        return super().form_valid(form)
+
+    def post(self, request, *args, **kwargs):
+        form = self.get_form()
+        if form.is_valid():
+            return self.form_valid(form)
+        return self.form_invalid(form)
 
 
 def move_confirm(request, page_to_move_id, destination_id):
@@ -56,8 +70,9 @@ def move_confirm(request, page_to_move_id, destination_id):
         messages.error(
             request,
             _(
-                "The slug '{0}' is already in use at the selected parent page. Make sure the slug is unique and try again"
-            ).format(page_to_move.slug),
+                "The slug '%(page_slug)s' is already in use at the selected parent page. Make sure the slug is unique and try again."
+            )
+            % {"page_slug": page_to_move.slug},
         )
         return redirect(
             "wagtailadmin_pages:move",
@@ -107,7 +122,8 @@ def move_confirm(request, page_to_move_id, destination_id):
 
         messages.success(
             request,
-            _("Page '{0}' moved.").format(page_to_move.get_admin_display_title()),
+            _("Page '%(page_title)s' moved.")
+            % {"page_title": page_to_move.get_admin_display_title()},
             buttons=[
                 messages.button(
                     reverse("wagtailadmin_pages:edit", args=(page_to_move.id,)),
