@@ -1,7 +1,10 @@
+import uuid
+
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import PermissionDenied
 from django.http import Http404
 from django.shortcuts import get_object_or_404
+from django.utils.translation import gettext
 
 from wagtail.admin.views.generic.preview import PreviewOnEdit as GenericPreviewOnEdit
 from wagtail.models import Page
@@ -43,6 +46,16 @@ class PreviewOnEdit(GenericPreviewOnEdit):
                 for_user=self.request.user,
             )
 
+        query_dict = query_dict.copy()
+
+        if not query_dict.get("title"):
+            query_dict["title"] = gettext("Placeholder title")
+
+        # Generate a random slug if one is not provided, use UUID to ensure
+        # uniqueness without needing to hit the database
+        if not query_dict.get("slug"):
+            query_dict["slug"] = str(uuid.uuid4())
+
         return form_class(
             query_dict,
             instance=self.object,
@@ -95,9 +108,9 @@ class PreviewOnCreate(PreviewOnEdit):
 
     def get_form(self, query_dict):
         form = super().get_form(query_dict)
-        if form.is_valid():
+        if self.validate_form(form):
             # Ensures our unsaved page has a suitable url.
             form.instance.set_url_path(form.parent_page)
 
-            form.instance.full_clean()
+            form.instance.minimal_clean()
         return form
