@@ -1,8 +1,8 @@
 from django.conf import settings
 from django.contrib.auth.models import Permission
-from django.urls import reverse
+from django.urls import reverse, reverse_lazy
+from django.utils.functional import cached_property
 from django.utils.http import urlencode
-from django.utils.translation import gettext
 from django.utils.translation import gettext_lazy as _
 from draftjs_exporter.dom import DOM
 
@@ -14,8 +14,15 @@ from wagtail.admin.admin_url_finder import (
 )
 from wagtail.admin.auth import user_has_any_page_permission
 from wagtail.admin.forms.collections import GroupCollectionManagementPermissionFormSet
-from wagtail.admin.menu import MenuItem, SubmenuMenuItem, reports_menu, settings_menu
-from wagtail.admin.navigation import get_explorable_root_page
+from wagtail.admin.menu import (
+    DismissibleMenuItem,
+    DismissibleSubmenuMenuItem,
+    MenuItem,
+    SubmenuMenuItem,
+    help_menu,
+    reports_menu,
+    settings_menu,
+)
 from wagtail.admin.rich_text.converters.contentstate import link_entity
 from wagtail.admin.rich_text.converters.editor_html import (
     LinkTypeRule,
@@ -33,6 +40,7 @@ from wagtail.admin.rich_text.converters.html_to_contentstate import (
 )
 from wagtail.admin.search import SearchArea
 from wagtail.admin.site_summary import PagesSummaryItem
+from wagtail.admin.ui.menus.pages import PageMenuItem
 from wagtail.admin.ui.sidebar import (
     PageExplorerMenuItem as PageExplorerMenuItemComponent,
 )
@@ -44,13 +52,19 @@ from wagtail.admin.views.pages.bulk_actions import (
     UnpublishBulkAction,
 )
 from wagtail.admin.viewsets import viewsets
-from wagtail.admin.widgets import Button, ButtonWithDropdownFromHook, PageListingButton
-from wagtail.models import Collection, Page, Task, UserPagePermissionsProxy, Workflow
+from wagtail.admin.widgets import ButtonWithDropdownFromHook
+from wagtail.models import Collection, Page, Task, Workflow
 from wagtail.permissions import (
     collection_permission_policy,
+    page_permission_policy,
     task_permission_policy,
     workflow_permission_policy,
 )
+from wagtail.templatetags.wagtailcore_tags import (
+    wagtail_feature_release_editor_guide_link,
+    wagtail_feature_release_whats_new_link,
+)
+from wagtail.utils.version import get_main_version
 from wagtail.whitelist import allow_without_attributes, attribute_rule, check_url
 
 
@@ -60,7 +74,7 @@ class ExplorerMenuItem(MenuItem):
 
     def get_context(self, request):
         context = super().get_context(request)
-        start_page = get_explorable_root_page(request.user)
+        start_page = page_permission_policy.explorable_root_instance(request.user)
 
         if start_page:
             context["start_page_id"] = start_page.id
@@ -68,7 +82,7 @@ class ExplorerMenuItem(MenuItem):
         return context
 
     def render_component(self, request):
-        start_page = get_explorable_root_page(request.user)
+        start_page = page_permission_policy.explorable_root_instance(request.user)
 
         if start_page:
             return PageExplorerMenuItemComponent(
@@ -77,7 +91,7 @@ class ExplorerMenuItem(MenuItem):
                 self.url,
                 start_page.id,
                 icon_name=self.icon_name,
-                classnames=self.classnames,
+                classname=self.classname,
             )
         else:
             return super().render_component(request)
@@ -101,14 +115,20 @@ class SettingsMenuItem(SubmenuMenuItem):
             self.label,
             self.menu.render_component(request),
             icon_name=self.icon_name,
-            classnames=self.classnames,
-            footer_text="Wagtail v." + __version__,
+            classname=self.classname,
+            footer_text="Wagtail v" + __version__,
         )
 
 
 @hooks.register("register_admin_menu_item")
 def register_settings_menu():
-    return SettingsMenuItem(_("Settings"), settings_menu, icon_name="cogs", order=10000)
+    return SettingsMenuItem(
+        _("Settings"),
+        settings_menu,
+        name="settings",
+        icon_name="cogs",
+        order=10000,
+    )
 
 
 @hooks.register("register_permissions")
@@ -154,6 +174,7 @@ def register_collections_menu_item():
     return CollectionsMenuItem(
         _("Collections"),
         reverse("wagtailadmin_collections:index"),
+        name="collections",
         icon_name="folder-open-1",
         order=700,
     )
@@ -184,6 +205,7 @@ def register_workflows_menu_item():
     return WorkflowsMenuItem(
         _("Workflows"),
         reverse("wagtailadmin_workflows:index"),
+        name="workflows",
         icon_name="tasks",
         order=100,
     )
@@ -194,258 +216,170 @@ def register_workflow_tasks_menu_item():
     return WorkflowTasksMenuItem(
         _("Workflow tasks"),
         reverse("wagtailadmin_workflows:task_index"),
+        name="workflow-tasks",
         icon_name="thumbtack",
         order=150,
     )
 
 
 @hooks.register("register_page_listing_buttons")
-def page_listing_buttons(page, page_perms, next_url=None):
-    if page_perms.can_edit():
-        yield PageListingButton(
-            _("Edit"),
-            reverse("wagtailadmin_pages:edit", args=[page.id]),
-            attrs={
-                "aria-label": _("Edit '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=10,
-        )
-    if page.has_unpublished_changes and page.is_previewable():
-        yield PageListingButton(
-            _("View draft"),
-            reverse("wagtailadmin_pages:view_draft", args=[page.id]),
-            attrs={
-                "aria-label": _("Preview draft version of '%(title)s'")
-                % {"title": page.get_admin_display_title()},
-                "rel": "noreferrer",
-            },
-            priority=20,
-        )
-    if page.live and page.url:
-        yield PageListingButton(
-            _("View live"),
-            page.url,
-            attrs={
-                "rel": "noreferrer",
-                "aria-label": _("View live version of '%(title)s'")
-                % {"title": page.get_admin_display_title()},
-            },
-            priority=30,
-        )
-    if page_perms.can_add_subpage():
-        yield PageListingButton(
-            _("Add child page"),
-            reverse("wagtailadmin_pages:add_subpage", args=[page.id]),
-            attrs={
-                "aria-label": _("Add a child page to '%(title)s' ")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=40,
-        )
-
+def page_listing_buttons(page, user, next_url=None):
     yield ButtonWithDropdownFromHook(
-        _("More"),
+        "",
         hook_name="register_page_listing_more_buttons",
         page=page,
-        page_perms=page_perms,
+        user=user,
         next_url=next_url,
+        icon_name="dots-horizontal",
         attrs={
-            "target": "_blank",
-            "rel": "noreferrer",
-            "title": _("View more options for '%(title)s'")
+            "aria-label": _("More options for '%(title)s'")
             % {"title": page.get_admin_display_title()},
         },
         priority=50,
     )
 
 
+class PageListingEditButton(PageMenuItem):
+    label = _("Edit")
+    icon_name = "edit"
+    url_name = "wagtailadmin_pages:edit"
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_edit()
+
+
+class PageListingViewDraftButton(PageMenuItem):
+    label = _("View draft")
+    icon_name = "draft"
+    url_name = "wagtailadmin_pages:view_draft"
+    link_rel = "noreferrer"
+
+    def is_shown(self, user):
+        return self.page.has_unpublished_changes and self.page.is_previewable()
+
+
+class PageListingViewLiveButton(PageMenuItem):
+    label = _("View live")
+    icon_name = "doc-empty"
+    link_rel = "noreferrer"
+
+    def is_shown(self, user):
+        return self.page.live and self.page.url
+
+
+class PageListingAddChildPageButton(PageMenuItem):
+    label = _("Add child page")
+    icon_name = "circle-plus"
+    url_name = "wagtailadmin_pages:add_subpage"
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_add_subpage()
+
+
+class PageListingMoveButton(PageMenuItem):
+    label = _("Move")
+    icon_name = "arrow-right-full"
+    url_name = "wagtailadmin_pages:move"
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_move()
+
+
+class PageListingCopyButton(PageMenuItem):
+    label = _("Copy")
+    icon_name = "copy"
+    url_name = "wagtailadmin_pages:copy"
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_copy()
+
+
+class PageListingDeleteButton(PageMenuItem):
+    label = _("Delete")
+    icon_name = "bin"
+
+    @cached_property
+    def url(self):
+        if self.page:
+            url = reverse("wagtailadmin_pages:delete", args=[self.page.id])
+            if self.next_url:
+                if self.next_url == reverse(
+                    "wagtailadmin_explore", args=[self.page.id]
+                ):
+                    # cannot redirect to the explore view after deleting the page
+                    pass
+                elif self.next_url == reverse(
+                    "wagtailadmin_pages:edit", args=[self.page.id]
+                ):
+                    # cannot redirect to the edit view after deleting the page
+                    pass
+                else:
+                    # OK to add the 'next' parameter
+                    url += "?" + urlencode({"next": self.next_url})
+            return url
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_delete()
+
+
+class PageListingUnpublishButton(PageMenuItem):
+    label = _("Unpublish")
+    icon_name = "download"
+    url_name = "wagtailadmin_pages:unpublish"
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_unpublish()
+
+
+class PageListingHistoryButton(PageMenuItem):
+    label = _("History")
+    icon_name = "history"
+    url_name = "wagtailadmin_pages:history"
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_view_revisions()
+
+
+class PageListingSortMenuOrderButton(PageMenuItem):
+    label = _("Sort menu order")
+    icon_name = "list-ul"
+
+    def is_shown(self, user):
+        return self.page.permissions_for_user(user).can_reorder_children()
+
+    @cached_property
+    def url(self):
+        return reverse("wagtailadmin_explore", args=[self.page.id]) + "?ordering=ord"
+
+
 @hooks.register("register_page_listing_more_buttons")
-def page_listing_more_buttons(page, page_perms, next_url=None):
-    if page_perms.can_move():
-        yield Button(
-            _("Move"),
-            reverse("wagtailadmin_pages:move", args=[page.id]),
-            attrs={
-                "title": _("Move page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=10,
-        )
-    if page_perms.can_copy():
-        url = reverse("wagtailadmin_pages:copy", args=[page.id])
-        if next_url:
-            url += "?" + urlencode({"next": next_url})
-
-        yield Button(
-            _("Copy"),
-            url,
-            attrs={
-                "title": _("Copy page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=20,
-        )
-    if page_perms.can_delete():
-        url = reverse("wagtailadmin_pages:delete", args=[page.id])
-        include_next_url = True
-
-        # After deleting the page, it is impossible to redirect to it.
-        if next_url == reverse("wagtailadmin_explore", args=[page.id]):
-            include_next_url = False
-
-        if next_url and include_next_url:
-            url += "?" + urlencode({"next": next_url})
-
-        yield Button(
-            _("Delete"),
-            url,
-            attrs={
-                "title": _("Delete page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=30,
-        )
-    if page_perms.can_unpublish():
-        url = reverse("wagtailadmin_pages:unpublish", args=[page.id])
-        if next_url:
-            url += "?" + urlencode({"next": next_url})
-
-        yield Button(
-            _("Unpublish"),
-            url,
-            attrs={
-                "title": _("Unpublish page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=40,
-        )
-    if page_perms.can_view_revisions():
-        yield Button(
-            _("History"),
-            reverse("wagtailadmin_pages:history", args=[page.id]),
-            attrs={
-                "title": _("View page history for '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=50,
-        )
-
-    if page_perms.can_reorder_children():
-        yield Button(
-            _("Sort menu order"),
-            "?ordering=ord",
-            attrs={
-                "title": _("Change ordering of child pages of '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=60,
-        )
+def page_listing_more_buttons(page, user, next_url=None):
+    yield PageListingEditButton(page=page, next_url=next_url, priority=2)
+    yield PageListingViewDraftButton(page=page, priority=4)
+    yield PageListingViewLiveButton(page=page, url=page.url, priority=6)
+    yield PageListingAddChildPageButton(page=page, next_url=next_url, priority=8)
+    yield PageListingMoveButton(page=page, priority=10)
+    yield PageListingCopyButton(page=page, next_url=next_url, priority=20)
+    yield PageListingDeleteButton(page=page, next_url=next_url, priority=30)
+    yield PageListingUnpublishButton(page=page, next_url=next_url, priority=40)
+    yield PageListingHistoryButton(page=page, priority=50)
+    yield PageListingSortMenuOrderButton(page=page, priority=60)
 
 
 @hooks.register("register_page_header_buttons")
-def page_header_buttons(page, page_perms, next_url=None):
-    if page_perms.can_edit():
-        yield Button(
-            _("Edit"),
-            reverse("wagtailadmin_pages:edit", args=[page.id]),
-            icon_name="edit",
-            attrs={
-                "title": _("Edit '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=10,
-        )
-    if page_perms.can_move():
-        yield Button(
-            _("Move"),
-            reverse("wagtailadmin_pages:move", args=[page.id]),
-            icon_name="arrow-right-full",
-            attrs={
-                "title": _("Move page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=20,
-        )
-    if page_perms.can_copy():
-        url = reverse("wagtailadmin_pages:copy", args=[page.id])
-        if next_url:
-            url += "?" + urlencode({"next": next_url})
+def page_header_buttons(page, user, view_name, next_url=None):
+    yield PageListingEditButton(page=page, priority=10)
 
-        yield Button(
-            _("Copy"),
-            url,
-            icon_name="copy",
-            attrs={
-                "title": _("Copy page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=30,
-        )
-    if page_perms.can_add_subpage():
-        yield Button(
-            _("Add child page"),
-            reverse("wagtailadmin_pages:add_subpage", args=[page.id]),
-            icon_name="circle-plus",
-            attrs={
-                "aria-label": _("Add a child page to '%(title)s' ")
-                % {"title": page.get_admin_display_title()},
-            },
-            priority=40,
-        )
-    if page_perms.can_delete():
-        url = reverse("wagtailadmin_pages:delete", args=[page.id])
+    # "add child" is a separate primary action on the index page
+    if view_name != "index":
+        yield PageListingAddChildPageButton(page=page, priority=15)
 
-        include_next_url = True
-
-        # After deleting the page, it is impossible to redirect to it.
-        if next_url == reverse("wagtailadmin_explore", args=[page.id]):
-            include_next_url = False
-
-        if next_url == reverse("wagtailadmin_pages:edit", args=[page.id]):
-            include_next_url = False
-
-        if next_url and include_next_url:
-            url += "?" + urlencode({"next": next_url})
-
-        yield Button(
-            _("Delete"),
-            url,
-            icon_name="bin",
-            attrs={
-                "title": _("Delete page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=50,
-        )
-    if page_perms.can_unpublish():
-        url = reverse("wagtailadmin_pages:unpublish", args=[page.id])
-        if next_url:
-            url += "?" + urlencode({"next": next_url})
-
-        yield Button(
-            _("Unpublish"),
-            url,
-            icon_name="download-alt",
-            attrs={
-                "title": _("Unpublish page '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=60,
-        )
-    if page_perms.can_reorder_children():
-        url = reverse("wagtailadmin_explore", args=[page.id])
-        url += "?ordering=ord"
-        yield Button(
-            _("Sort menu order"),
-            url,
-            icon_name="list-ul",
-            attrs={
-                "title": _("Change ordering of child pages of '%(title)s'")
-                % {"title": page.get_admin_display_title()}
-            },
-            priority=70,
-        )
+    yield PageListingMoveButton(page=page, priority=20)
+    yield PageListingCopyButton(page=page, next_url=next_url, priority=30)
+    yield PageListingDeleteButton(page=page, next_url=next_url, priority=50)
+    yield PageListingUnpublishButton(page=page, next_url=next_url, priority=60)
+    yield PageListingHistoryButton(page=page, priority=65)
+    yield PageListingSortMenuOrderButton(page=page, priority=70)
 
 
 @hooks.register("register_admin_urls")
@@ -533,7 +467,7 @@ def register_core_features(features):
             {
                 "icon": "h1",
                 "type": "header-one",
-                "description": gettext("Heading %(level)d") % {"level": 1},
+                "description": _("Heading 1"),
             }
         ),
     )
@@ -554,7 +488,7 @@ def register_core_features(features):
             {
                 "icon": "h2",
                 "type": "header-two",
-                "description": gettext("Heading %(level)d") % {"level": 2},
+                "description": _("Heading 2"),
             }
         ),
     )
@@ -575,7 +509,7 @@ def register_core_features(features):
             {
                 "icon": "h3",
                 "type": "header-three",
-                "description": gettext("Heading %(level)d") % {"level": 3},
+                "description": _("Heading 3"),
             }
         ),
     )
@@ -596,7 +530,7 @@ def register_core_features(features):
             {
                 "icon": "h4",
                 "type": "header-four",
-                "description": gettext("Heading %(level)d") % {"level": 4},
+                "description": _("Heading 4"),
             }
         ),
     )
@@ -617,7 +551,7 @@ def register_core_features(features):
             {
                 "icon": "h5",
                 "type": "header-five",
-                "description": gettext("Heading %(level)d") % {"level": 5},
+                "description": _("Heading 5"),
             }
         ),
     )
@@ -638,7 +572,7 @@ def register_core_features(features):
             {
                 "icon": "h6",
                 "type": "header-six",
-                "description": gettext("Heading %(level)d") % {"level": 6},
+                "description": _("Heading 6"),
             }
         ),
     )
@@ -659,7 +593,7 @@ def register_core_features(features):
             {
                 "type": "unordered-list-item",
                 "icon": "list-ul",
-                "description": gettext("Bulleted list"),
+                "description": _("Bulleted list"),
             }
         ),
     )
@@ -683,7 +617,7 @@ def register_core_features(features):
             {
                 "type": "ordered-list-item",
                 "icon": "list-ol",
-                "description": gettext("Numbered list"),
+                "description": _("Numbered list"),
             }
         ),
     )
@@ -707,7 +641,7 @@ def register_core_features(features):
             {
                 "type": "blockquote",
                 "icon": "openquote",
-                "description": gettext("Blockquote"),
+                "description": _("Blockquote"),
             }
         ),
     )
@@ -729,7 +663,7 @@ def register_core_features(features):
             {
                 "type": "BOLD",
                 "icon": "bold",
-                "description": gettext("Bold"),
+                "description": _("Bold"),
             }
         ),
     )
@@ -751,7 +685,7 @@ def register_core_features(features):
             {
                 "type": "ITALIC",
                 "icon": "italic",
-                "description": gettext("Italic"),
+                "description": _("Italic"),
             }
         ),
     )
@@ -774,7 +708,7 @@ def register_core_features(features):
             {
                 "type": "LINK",
                 "icon": "link",
-                "description": gettext("Link"),
+                "description": _("Link"),
                 # We want to enforce constraints on which links can be pasted into rich text.
                 # Keep only the attributes Wagtail needs.
                 "attributes": ["url", "id", "parentId"],
@@ -782,6 +716,21 @@ def register_core_features(features):
                     # Keep pasted links with http/https protocol, and not-pasted links (href = undefined).
                     "href": "^(http:|https:|undefined$)",
                 },
+                "chooserUrls": {
+                    "pageChooser": reverse_lazy("wagtailadmin_choose_page"),
+                    "externalLinkChooser": reverse_lazy(
+                        "wagtailadmin_choose_page_external_link"
+                    ),
+                    "emailLinkChooser": reverse_lazy(
+                        "wagtailadmin_choose_page_email_link"
+                    ),
+                    "phoneLinkChooser": reverse_lazy(
+                        "wagtailadmin_choose_page_phone_link"
+                    ),
+                    "anchorLinkChooser": reverse_lazy(
+                        "wagtailadmin_choose_page_anchor_link"
+                    ),
+                },
             },
             js=[
                 "wagtailadmin/js/page-chooser-modal.js",
@@ -806,7 +755,7 @@ def register_core_features(features):
             {
                 "type": "SUPERSCRIPT",
                 "icon": "superscript",
-                "description": gettext("Superscript"),
+                "description": _("Superscript"),
             }
         ),
     )
@@ -827,7 +776,7 @@ def register_core_features(features):
             {
                 "type": "SUBSCRIPT",
                 "icon": "subscript",
-                "description": gettext("Subscript"),
+                "description": _("Subscript"),
             }
         ),
     )
@@ -848,7 +797,7 @@ def register_core_features(features):
             {
                 "type": "STRIKETHROUGH",
                 "icon": "strikethrough",
-                "description": gettext("Strikethrough"),
+                "description": _("Strikethrough"),
             }
         ),
     )
@@ -869,7 +818,7 @@ def register_core_features(features):
             {
                 "type": "CODE",
                 "icon": "code",
-                "description": gettext("Code"),
+                "description": _("Code"),
             }
         ),
     )
@@ -887,29 +836,45 @@ def register_core_features(features):
 
 class LockedPagesMenuItem(MenuItem):
     def is_shown(self, request):
-        return UserPagePermissionsProxy(request.user).can_remove_locks()
+        return page_permission_policy.user_has_permission(request.user, "unlock")
 
 
 class WorkflowReportMenuItem(MenuItem):
     def is_shown(self, request):
-        return getattr(settings, "WAGTAIL_WORKFLOW_ENABLED", True)
+        return getattr(
+            settings, "WAGTAIL_WORKFLOW_ENABLED", True
+        ) and page_permission_policy.user_has_any_permission(
+            request.user, ["add", "change", "publish"]
+        )
 
 
 class SiteHistoryReportMenuItem(MenuItem):
     def is_shown(self, request):
-        return UserPagePermissionsProxy(request.user).explorable_pages().exists()
+        return page_permission_policy.explorable_root_instance(request.user) is not None
 
 
 class AgingPagesReportMenuItem(MenuItem):
     def is_shown(self, request):
-        return getattr(settings, "WAGTAIL_AGING_PAGES_ENABLED", True)
+        return getattr(
+            settings, "WAGTAIL_AGING_PAGES_ENABLED", True
+        ) and page_permission_policy.user_has_any_permission(
+            request.user, ["add", "change", "publish"]
+        )
+
+
+class PageTypesReportMenuItem(MenuItem):
+    def is_shown(self, request):
+        return page_permission_policy.user_has_any_permission(
+            request.user, ["add", "change", "publish"]
+        )
 
 
 @hooks.register("register_reports_menu_item")
 def register_locked_pages_menu_item():
     return LockedPagesMenuItem(
-        _("Locked Pages"),
+        _("Locked pages"),
         reverse("wagtailadmin_reports:locked_pages"),
+        name="locked-pages",
         icon_name="lock",
         order=700,
     )
@@ -920,6 +885,7 @@ def register_workflow_report_menu_item():
     return WorkflowReportMenuItem(
         _("Workflows"),
         reverse("wagtailadmin_reports:workflow"),
+        name="workflows",
         icon_name="tasks",
         order=800,
     )
@@ -930,6 +896,7 @@ def register_workflow_tasks_report_menu_item():
     return WorkflowReportMenuItem(
         _("Workflow tasks"),
         reverse("wagtailadmin_reports:workflow_tasks"),
+        name="workflow-tasks",
         icon_name="thumbtack",
         order=900,
     )
@@ -940,6 +907,7 @@ def register_site_history_report_menu_item():
     return SiteHistoryReportMenuItem(
         _("Site history"),
         reverse("wagtailadmin_reports:site_history"),
+        name="site-history",
         icon_name="history",
         order=1000,
     )
@@ -950,38 +918,112 @@ def register_aging_pages_report_menu_item():
     return AgingPagesReportMenuItem(
         _("Aging pages"),
         reverse("wagtailadmin_reports:aging_pages"),
+        name="aging-pages",
         icon_name="time",
         order=1100,
     )
 
 
+@hooks.register("register_reports_menu_item")
+def register_page_types_report_menu_item():
+    return PageTypesReportMenuItem(
+        _("Page types usage"),
+        reverse("wagtailadmin_reports:page_types_usage"),
+        name="page-types-usage",
+        icon_name="doc-empty-inverse",
+        order=1200,
+    )
+
+
 @hooks.register("register_admin_menu_item")
 def register_reports_menu():
-    return SubmenuMenuItem(_("Reports"), reports_menu, icon_name="site", order=9000)
+    return SubmenuMenuItem(
+        _("Reports"),
+        reports_menu,
+        name="reports",
+        icon_name="site",
+        order=9000,
+    )
+
+
+@hooks.register("register_help_menu_item")
+def register_whats_new_in_wagtail_version_menu_item():
+    version = get_main_version(include_patch=False)
+    return DismissibleMenuItem(
+        _("What's new in Wagtail %(version)s") % {"version": version},
+        wagtail_feature_release_whats_new_link(),
+        icon_name="help",
+        order=1000,
+        attrs={"target": "_blank", "rel": "noreferrer"},
+        name=f"whats-new-in-wagtail-{version}",
+    )
+
+
+@hooks.register("register_help_menu_item")
+def register_editors_guide_menu_item():
+    return DismissibleMenuItem(
+        _("Editor Guide"),
+        wagtail_feature_release_editor_guide_link(),
+        icon_name="help",
+        order=1100,
+        attrs={"target": "_blank", "rel": "noreferrer"},
+        name="editor-guide",
+    )
+
+
+@hooks.register("register_help_menu_item")
+def register_keyboard_shortcuts_menu_item():
+    """
+    Triggers the keyboard shortcuts dialog to open when clicked
+    while preventing the default link click action.
+    """
+
+    return MenuItem(
+        _("Shortcuts"),
+        icon_name="keyboard",
+        order=1200,
+        attrs={
+            "role": "button",  # Ensure screen readers announce this as a button
+            "data-a11y-dialog-show": "keyboard-shortcuts-dialog",
+            "data-action": "w-action#noop:prevent:stop",
+            "data-controller": "w-kbd w-action",
+            "data-w-kbd-key-value": "?",
+        },
+        name="keyboard-shortcuts-trigger",
+        url="#",
+    )
+
+
+@hooks.register("register_admin_menu_item")
+def register_help_menu():
+    return DismissibleSubmenuMenuItem(
+        _("Help"),
+        help_menu,
+        name="help",
+        icon_name="help",
+        order=11000,
+    )
 
 
 @hooks.register("register_icons")
 def register_icons(icons):
     for icon in [
-        "angle-double-left.svg",
-        "angle-double-right.svg",
-        "arrow-down-big.svg",
         "arrow-down.svg",
         "arrow-right-full.svg",
         "arrow-left.svg",
         "arrow-right.svg",
-        "arrow-up-big.svg",
         "arrow-up.svg",
-        "arrows-up-down.svg",
         "bars.svg",
         "bin.svg",
         "bold.svg",
         "breadcrumb-expand.svg",
-        "chain-broken.svg",
+        "calendar.svg",
+        "calendar-alt.svg",
+        "calendar-check.svg",
         "check.svg",
-        "chevron-down.svg",
         "circle-check.svg",
         "circle-plus.svg",
+        "circle-xmark.svg",
         "clipboard-list.svg",
         "code.svg",
         "cog.svg",
@@ -993,21 +1035,19 @@ def register_icons(icons):
         "comment-add.svg",
         "comment-add-reversed.svg",
         "cross.svg",
+        "crosshairs.svg",
         "cut.svg",
         "date.svg",
+        "decimal.svg",
         "desktop.svg",
         "doc-empty-inverse.svg",
         "doc-empty.svg",
         "doc-full-inverse.svg",
-        "doc-full.svg",  # aka file-text-alt
-        "dots-vertical.svg",
+        "doc-full.svg",
         "dots-horizontal.svg",
-        "download-alt.svg",
         "download.svg",
         "draft.svg",
-        "duplicate.svg",
         "edit.svg",
-        "ellipsis-v.svg",
         "expand-right.svg",
         "error.svg",
         "folder-inverse.svg",
@@ -1015,7 +1055,9 @@ def register_icons(icons):
         "folder-open-inverse.svg",
         "folder.svg",
         "form.svg",
+        "glasses.svg",
         "globe.svg",
+        "grid.svg",
         "grip.svg",
         "group.svg",
         "h1.svg",
@@ -1027,10 +1069,11 @@ def register_icons(icons):
         "help.svg",
         "history.svg",
         "home.svg",
-        "horizontalrule.svg",
-        "image.svg",  # aka picture
+        "image.svg",
         "info-circle.svg",
         "italic.svg",
+        "key.svg",
+        "keyboard.svg",
         "link.svg",
         "link-external.svg",
         "list-ol.svg",
@@ -1041,6 +1084,7 @@ def register_icons(icons):
         "logout.svg",
         "mail.svg",
         "media.svg",
+        "minus.svg",
         "mobile-alt.svg",
         "no-view.svg",
         "openquote.svg",
@@ -1050,18 +1094,18 @@ def register_icons(icons):
         "password.svg",
         "pick.svg",
         "pilcrow.svg",
-        "placeholder.svg",  # aka marquee
+        "placeholder.svg",
         "plus-inverse.svg",
         "plus.svg",
         "radio-empty.svg",
         "radio-full.svg",
         "redirect.svg",
-        "repeat.svg",
-        "reset.svg",
+        "regex.svg",
         "resubmit.svg",
         "rotate.svg",
         "search.svg",
         "site.svg",
+        "sliders.svg",
         "snippet.svg",
         "spinner.svg",
         "strikethrough.svg",
@@ -1073,20 +1117,17 @@ def register_icons(icons):
         "tag.svg",
         "tasks.svg",
         "thumbtack.svg",
+        "thumbtack-crossed.svg",
         "tick-inverse.svg",
-        "tick.svg",
         "time.svg",
         "title.svg",
-        "undo.svg",
-        "uni52.svg",  # Is this a redundant icon?
         "upload.svg",
         "user.svg",
         "view.svg",
-        "wagtail-inverse.svg",
         "wagtail.svg",
         "warning.svg",
     ]:
-        icons.append("wagtailadmin/icons/{}".format(icon))
+        icons.append(f"wagtailadmin/icons/{icon}")
     return icons
 
 
@@ -1097,10 +1138,10 @@ def add_pages_summary_item(request, items):
 
 class PageAdminURLFinder:
     def __init__(self, user):
-        self.page_perms = user and UserPagePermissionsProxy(user)
+        self.user = user
 
     def get_edit_url(self, instance):
-        if self.page_perms and not self.page_perms.for_page(instance).can_edit():
+        if self.user and not instance.permissions_for_user(self.user).can_edit():
             return None
         else:
             return reverse("wagtailadmin_pages:edit", args=(instance.pk,))
