@@ -18,14 +18,14 @@ class FilteredSelect(forms.Select):
     see wagtailadmin/js/filtered-select.js for an example of how these attributes are configured.
     """
 
-    def __init__(self, attrs=None, choices=(), filter_field=''):
+    def __init__(self, attrs=None, choices=(), filter_field=""):
         super().__init__(attrs, choices)
         self.filter_field = filter_field
 
     def build_attrs(self, base_attrs, extra_attrs=None):
         my_attrs = {
-            'data-widget': 'filtered-select',
-            'data-filter-field': self.filter_field,
+            "data-widget": "filtered-select",
+            "data-filter-field": self.filter_field,
         }
         if extra_attrs:
             my_attrs.update(extra_attrs)
@@ -49,7 +49,7 @@ class FilteredSelect(forms.Select):
                 filter_value = None
 
             if option_value is None:
-                option_value = ''
+                option_value = ""
 
             subgroup = []
             if isinstance(option_label, (list, tuple)):
@@ -72,31 +72,51 @@ class FilteredSelect(forms.Select):
                     (subvalue, sublabel) = choice
                     filter_value = None
 
-                selected = (
-                    str(subvalue) in value
-                    and (not has_selected or self.allow_multiple_selected)
+                selected = str(subvalue) in value and (
+                    not has_selected or self.allow_multiple_selected
                 )
                 has_selected |= selected
 
-                subgroup.append(self.create_option(
-                    name, subvalue, sublabel, selected, index, subindex=subindex,
-                    filter_value=filter_value
-                ))
+                subgroup.append(
+                    self.create_option(
+                        name,
+                        subvalue,
+                        sublabel,
+                        selected,
+                        index,
+                        subindex=subindex,
+                        filter_value=filter_value,
+                    )
+                )
                 if subindex is not None:
                     subindex += 1
         return groups
 
-    def create_option(self, name, value, label, selected, index, subindex=None, attrs=None, filter_value=None):
+    def create_option(
+        self,
+        name,
+        value,
+        label,
+        selected,
+        index,
+        subindex=None,
+        attrs=None,
+        filter_value=None,
+    ):
         option = super().create_option(
             name, value, label, selected, index, subindex=subindex, attrs=attrs
         )
         if filter_value is not None:
-            option['attrs']['data-filter-value'] = ','.join([str(val) for val in filter_value])
+            option["attrs"]["data-filter-value"] = ",".join(
+                [str(val) for val in filter_value]
+            )
 
         return option
 
     @property
     def media(self):
-        return forms.Media(js=[
-            versioned_static('wagtailadmin/js/filtered-select.js'),
-        ])
+        return forms.Media(
+            js=[
+                versioned_static("wagtailadmin/js/filtered-select.js"),
+            ]
+        )
