@@ -2,4 +2,4 @@ from django.forms import widgets
 
 
 class SwitchInput(widgets.CheckboxInput):
-    template_name = 'wagtailadmin/widgets/switch.html'
+    template_name = "wagtailadmin/widgets/switch.html"
