@@ -6,7 +6,7 @@ from rest_framework import relations, serializers
 from rest_framework.fields import Field, SkipField
 from taggit.managers import _TaggableManager
 
-from wagtail.core import fields as wagtailcore_fields
+from wagtail import fields as wagtailcore_fields
 
 from .utils import get_object_detail_url
 
@@ -18,12 +18,13 @@ class TypeField(Field):
     Example:
     "type": "wagtailimages.Image"
     """
+
     def get_attribute(self, instance):
         return instance
 
     def to_representation(self, obj):
-        name = type(obj)._meta.app_label + '.' + type(obj).__name__
-        self.context['view'].seen_types[name] = type(obj)
+        name = type(obj)._meta.app_label + "." + type(obj).__name__
+        self.context["view"].seen_types[name] = type(obj)
         return name
 
 
@@ -34,8 +35,11 @@ class DetailUrlField(Field):
     Example:
     "detail_url": "http://api.example.com/v1/images/1/"
     """
+
     def get_attribute(self, instance):
-        url = get_object_detail_url(self.context['router'], self.context['request'], type(instance), instance.pk)
+        url = get_object_detail_url(
+            self.context["router"], self.context["request"], type(instance), instance.pk
+        )
 
         if url:
             return url
@@ -54,6 +58,7 @@ class PageHtmlUrlField(Field):
     Example:
     "html_url": "http://www.example.com/blog/blog-post/"
     """
+
     def get_attribute(self, instance):
         return instance
 
@@ -75,14 +80,15 @@ class PageTypeField(Field):
     Example:
     "type": "blog.BlogPage"
     """
+
     def get_attribute(self, instance):
         return instance
 
     def to_representation(self, page):
         if page.specific_class is None:
             return None
-        name = page.specific_class._meta.app_label + '.' + page.specific_class.__name__
-        self.context['view'].seen_types[name] = page.specific_class
+        name = page.specific_class._meta.app_label + "." + page.specific_class.__name__
+        self.context["view"].seen_types[name] = page.specific_class
         return name
 
 
@@ -90,6 +96,7 @@ class PageLocaleField(Field):
     """
     Serializes the "locale" field for pages.
     """
+
     def get_attribute(self, instance):
         return instance
 
@@ -111,8 +118,9 @@ class RelatedField(relations.RelatedField):
         }
     }
     """
+
     def __init__(self, *args, **kwargs):
-        self.serializer_class = kwargs.pop('serializer_class')
+        self.serializer_class = kwargs.pop("serializer_class")
         super().__init__(*args, **kwargs)
 
     def to_representation(self, value):
@@ -129,14 +137,39 @@ class PageParentField(relations.RelatedField):
 
     The representation is the same as the RelatedField class.
     """
+
     def get_attribute(self, instance):
         parent = instance.get_parent()
 
-        if self.context['base_queryset'].filter(id=parent.id).exists():
+        if self.context["base_queryset"].filter(id=parent.id).exists():
             return parent
 
     def to_representation(self, value):
-        serializer_class = get_serializer_class(value.__class__, ['id', 'type', 'detail_url', 'html_url', 'title'], meta_fields=['type', 'detail_url', 'html_url'], base=PageSerializer)
+        serializer_class = get_serializer_class(
+            value.__class__,
+            ["id", "type", "detail_url", "html_url", "title"],
+            meta_fields=["type", "detail_url", "html_url"],
+            base=PageSerializer,
+        )
+        serializer = serializer_class(context=self.context)
+        return serializer.to_representation(value)
+
+
+class PageAliasOfField(relations.RelatedField):
+    """
+    Serializes the "alias_of" field on Page objects.
+    """
+
+    def get_attribute(self, instance):
+        return instance.alias_of
+
+    def to_representation(self, value):
+        serializer_class = get_serializer_class(
+            value.__class__,
+            ["id", "type", "detail_url", "html_url", "title"],
+            meta_fields=["type", "detail_url", "html_url"],
+            base=PageSerializer,
+        )
         serializer = serializer_class(context=self.context)
         return serializer.to_representation(value)
 
@@ -179,16 +212,16 @@ class ChildRelationField(Field):
         }
     ]
     """
+
     def __init__(self, *args, **kwargs):
-        self.serializer_class = kwargs.pop('serializer_class')
+        self.serializer_class = kwargs.pop("serializer_class")
         super().__init__(*args, **kwargs)
 
     def to_representation(self, value):
         serializer = self.serializer_class(context=self.context)
 
         return [
-            serializer.to_representation(child_object)
-            for child_object in value.all()
+            serializer.to_representation(child_object) for child_object in value.all()
         ]
 
 
@@ -227,6 +260,7 @@ class StreamField(Field):
     by an integer (the ID of the related object) but elsewhere in the API,
     foreign objects are nested objects with id and meta as attributes.
     """
+
     def to_representation(self, value):
         return value.stream_block.get_api_representation(value, self.context)
 
@@ -243,16 +277,21 @@ class TagsField(Field):
 
     "tags": ["bird", "wagtail"]
     """
+
     def to_representation(self, value):
-        return list(value.all().order_by('name').values_list('name', flat=True))
+        return list(value.all().order_by("name").values_list("name", flat=True))
 
 
 class BaseSerializer(serializers.ModelSerializer):
     # Add StreamField to serializer_field_mapping
-    serializer_field_mapping = serializers.ModelSerializer.serializer_field_mapping.copy()
-    serializer_field_mapping.update({
-        wagtailcore_fields.StreamField: StreamField,
-    })
+    serializer_field_mapping = (
+        serializers.ModelSerializer.serializer_field_mapping.copy()
+    )
+    serializer_field_mapping.update(
+        {
+            wagtailcore_fields.StreamField: StreamField,
+        }
+    )
     serializer_related_field = RelatedField
 
     # Meta fields
@@ -264,12 +303,14 @@ class BaseSerializer(serializers.ModelSerializer):
         fields = [field for field in self.fields.values() if not field.write_only]
 
         # Split meta fields from core fields
-        meta_fields = [field for field in fields if field.field_name in self.meta_fields]
+        meta_fields = [
+            field for field in fields if field.field_name in self.meta_fields
+        ]
         fields = [field for field in fields if field.field_name not in self.meta_fields]
 
         # Make sure id is always first. This will be filled in later
-        if 'id' in [field.field_name for field in fields]:
-            data['id'] = None
+        if "id" in [field.field_name for field in fields]:
+            data["id"] = None
 
         # Serialise meta fields
         meta = OrderedDict()
@@ -287,12 +328,12 @@ class BaseSerializer(serializers.ModelSerializer):
                 meta[field.field_name] = field.to_representation(attribute)
 
         if meta:
-            data['meta'] = meta
+            data["meta"] = meta
 
         # Serialise core fields
         for field in fields:
             try:
-                if field.field_name == 'admin_display_title':
+                if field.field_name == "admin_display_title":
                     instance = instance.specific_deferred
 
                 attribute = field.get_attribute(instance)
@@ -317,8 +358,10 @@ class BaseSerializer(serializers.ModelSerializer):
         return super().build_property_field(field_name, model_class)
 
     def build_relational_field(self, field_name, relation_info):
-        field_class, field_kwargs = super().build_relational_field(field_name, relation_info)
-        field_kwargs['serializer_class'] = self.child_serializer_classes[field_name]
+        field_class, field_kwargs = super().build_relational_field(
+            field_name, relation_info
+        )
+        field_kwargs["serializer_class"] = self.child_serializer_classes[field_name]
         return field_class, field_kwargs
 
 
@@ -327,24 +370,37 @@ class PageSerializer(BaseSerializer):
     locale = PageLocaleField(read_only=True)
     html_url = PageHtmlUrlField(read_only=True)
     parent = PageParentField(read_only=True)
+    alias_of = PageAliasOfField(read_only=True)
 
     def build_relational_field(self, field_name, relation_info):
         # Find all relation fields that point to child class and make them use
         # the ChildRelationField class.
         if relation_info.to_many:
-            model = getattr(self.Meta, 'model')
+            model = self.Meta.model
             child_relations = {
                 child_relation.field.remote_field.related_name: child_relation.related_model
                 for child_relation in get_all_child_relations(model)
             }
 
-            if field_name in child_relations and field_name in self.child_serializer_classes:
-                return ChildRelationField, {'serializer_class': self.child_serializer_classes[field_name]}
+            if (
+                field_name in child_relations
+                and field_name in self.child_serializer_classes
+            ):
+                return ChildRelationField, {
+                    "serializer_class": self.child_serializer_classes[field_name]
+                }
 
         return super().build_relational_field(field_name, relation_info)
 
 
-def get_serializer_class(model, field_names, meta_fields, field_serializer_overrides=None, child_serializer_classes=None, base=BaseSerializer):
+def get_serializer_class(
+    model,
+    field_names,
+    meta_fields,
+    field_serializer_overrides=None,
+    child_serializer_classes=None,
+    base=BaseSerializer,
+):
     model_ = model
 
     class Meta:
@@ -352,12 +408,12 @@ def get_serializer_class(model, field_names, meta_fields, field_serializer_overr
         fields = list(field_names)
 
     attrs = {
-        'Meta': Meta,
-        'meta_fields': list(meta_fields),
-        'child_serializer_classes': child_serializer_classes or {},
+        "Meta": Meta,
+        "meta_fields": list(meta_fields),
+        "child_serializer_classes": child_serializer_classes or {},
     }
 
     if field_serializer_overrides:
         attrs.update(field_serializer_overrides)
 
-    return type(str(model_.__name__ + 'Serializer'), (base, ), attrs)
+    return type(str(model_.__name__ + "Serializer"), (base,), attrs)
