@@ -1,10 +1,12 @@
 from collections import OrderedDict
 
+from django.apps import apps
 from django.conf import settings
 from django.core.exceptions import FieldDoesNotExist
 from django.http import Http404
 from django.shortcuts import redirect
 from django.urls import path, reverse
+from django.utils.functional import classproperty
 from modelcluster.fields import ParentalKey
 from rest_framework import status
 from rest_framework.renderers import BrowsableAPIRenderer, JSONRenderer
@@ -35,7 +37,16 @@ from .utils import (
 
 
 class BaseAPIViewSet(GenericViewSet):
-    renderer_classes = [JSONRenderer, BrowsableAPIRenderer]
+    @classproperty
+    def renderer_classes(cls):
+        renderers = [JSONRenderer]
+
+        # Only add BrowsableAPIRenderer if rest_framework is installed
+        # (which provides the necessary templates and static files)
+        if apps.is_installed("rest_framework"):
+            renderers.append(BrowsableAPIRenderer)
+
+        return renderers
 
     pagination_class = WagtailPagination
     base_serializer_class = BaseSerializer
@@ -56,6 +67,7 @@ class BaseAPIViewSet(GenericViewSet):
             "format",
         ]
     )
+    find_query_parameters = frozenset(["id"])
     body_fields = ["id"]
     meta_fields = ["type", "detail_url"]
     listing_default_fields = ["id", "type", "detail_url"]
@@ -113,7 +125,12 @@ class BaseAPIViewSet(GenericViewSet):
                 )
             )
 
-        return redirect(url)
+        # Retain all query parameters except ones only used to find the object
+        query = request.GET.copy()
+        for param in self.find_query_parameters:
+            query.pop(param, None)
+
+        return redirect(f"{url}?{query.urlencode()}")
 
     def find_object(self, queryset, request):
         """
@@ -238,7 +255,7 @@ class BaseAPIViewSet(GenericViewSet):
             for field in cls.detail_only_fields:
                 try:
                     all_fields.remove(field)
-                except KeyError:
+                except ValueError:
                     pass
 
         # Get list of configured fields
@@ -305,10 +322,10 @@ class BaseAPIViewSet(GenericViewSet):
                 child_endpoint_class = (
                     child_endpoint_class[1] if child_endpoint_class else BaseAPIViewSet
                 )
-                child_serializer_classes[
-                    field_name
-                ] = child_endpoint_class._get_serializer_class(
-                    router, child_model, child_sub_fields, nested=True
+                child_serializer_classes[field_name] = (
+                    child_endpoint_class._get_serializer_class(
+                        router, child_model, child_sub_fields, nested=True
+                    )
                 )
 
             else:
@@ -435,6 +452,11 @@ class PagesAPIViewSet(BaseAPIViewSet):
             "site",
         ]
     )
+    find_query_parameters = BaseAPIViewSet.find_query_parameters.union(
+        [
+            "html_path",
+        ]
+    )
     body_fields = BaseAPIViewSet.body_fields + [
         "title",
     ]
@@ -555,20 +577,24 @@ class PagesAPIViewSet(BaseAPIViewSet):
 
         # Allow pages to be filtered to a specific type
         try:
-            models = page_models_from_string(
-                request.GET.get("type", "wagtailcore.Page")
-            )
+            models_type = request.GET.get("type", None)
+            models = models_type and page_models_from_string(models_type) or []
         except (LookupError, ValueError):
             raise BadRequestError("type doesn't exist")
 
         if not models:
-            return self.get_base_queryset()
+            if self.model == Page:
+                return self.get_base_queryset()
+            else:
+                return self.model.objects.filter(
+                    pk__in=self.get_base_queryset().values_list("pk", flat=True)
+                )
 
         elif len(models) == 1:
             # If a single page type has been specified, swap out the Page-based queryset for one based on
             # the specific page model so that we can filter on any custom APIFields defined on that model
             return models[0].objects.filter(
-                id__in=self.get_base_queryset().values_list("id", flat=True)
+                pk__in=self.get_base_queryset().values_list("pk", flat=True)
             )
 
         else:  # len(models) > 1
