@@ -6,6 +6,18 @@ from django.utils.html import conditional_escape
 from django.utils.translation import gettext_lazy as _
 
 from wagtail.admin.forms import WagtailAdminPageForm
+from wagtail.compat import URLField
+
+
+def _split_by_newline_or_comma(value):
+    """
+    Split the given string either by new lines, or if no new line is present then
+    by comma.
+    """
+    if len(lines := value.strip().splitlines()) > 1:
+        return [line.strip().rstrip(",").strip() for line in lines]
+    else:
+        return list(map(str.strip, value.split(",")))
 
 
 class BaseForm(django.forms.Form):
@@ -28,7 +40,8 @@ class FormBuilder:
         return django.forms.CharField(**options)
 
     def create_multiline_field(self, field, options):
-        return django.forms.CharField(widget=django.forms.Textarea, **options)
+        options.setdefault("widget", django.forms.Textarea)
+        return django.forms.CharField(**options)
 
     def create_date_field(self, field, options):
         return django.forms.DateField(**options)
@@ -40,7 +53,7 @@ class FormBuilder:
         return django.forms.EmailField(**options)
 
     def create_url_field(self, field, options):
-        return django.forms.URLField(**options)
+        return URLField(**options)
 
     def create_number_field(self, field, options):
         return django.forms.DecimalField(**options)
@@ -55,20 +68,21 @@ class FormBuilder:
 
     def create_radio_field(self, field, options):
         options["choices"] = self.get_formatted_field_choices(field)
-        return django.forms.ChoiceField(widget=django.forms.RadioSelect, **options)
+        options.setdefault("widget", django.forms.RadioSelect)
+        return django.forms.ChoiceField(**options)
 
     def create_checkboxes_field(self, field, options):
         options["choices"] = self.get_formatted_field_choices(field)
         options["initial"] = self.get_formatted_field_initial(field)
-        return django.forms.MultipleChoiceField(
-            widget=django.forms.CheckboxSelectMultiple, **options
-        )
+        options.setdefault("widget", django.forms.CheckboxSelectMultiple)
+        return django.forms.MultipleChoiceField(**options)
 
     def create_checkbox_field(self, field, options):
         return django.forms.BooleanField(**options)
 
     def create_hidden_field(self, field, options):
-        return django.forms.CharField(widget=django.forms.HiddenInput, **options)
+        options.setdefault("widget", django.forms.HiddenInput)
+        return django.forms.CharField(**options)
 
     def get_create_field_function(self, type):
         """
@@ -100,19 +114,7 @@ class FormBuilder:
         Split the provided choices into a list, separated by new lines.
         If no new lines in the provided choices, split by commas.
         """
-
-        if "\n" in field.choices:
-            choices = (
-                (
-                    x.strip().rstrip(",").strip(),
-                    x.strip().rstrip(",").strip(),
-                )
-                for x in field.choices.split("\r\n")
-            )
-        else:
-            choices = ((x.strip(), x.strip()) for x in field.choices.split(","))
-
-        return choices
+        return [(x, x) for x in _split_by_newline_or_comma(field.choices)]
 
     def get_formatted_field_initial(self, field):
         """
@@ -120,15 +122,7 @@ class FormBuilder:
         Split the supplied default values into a list, separated by new lines.
         If no new lines in the provided default values, split by commas.
         """
-
-        if "\n" in field.default_value:
-            values = [
-                x.strip().rstrip(",").strip() for x in field.default_value.split("\r\n")
-            ]
-        else:
-            values = [x.strip() for x in field.default_value.split(",")]
-
-        return values
+        return _split_by_newline_or_comma(field.default_value)
 
     @property
     def formfields(self):
@@ -197,31 +191,61 @@ class WagtailAdminFormPageForm(WagtailAdminPageForm):
             for form in forms:
                 form.is_valid()
 
-            # Use existing clean_name or generate for new fields,
-            # raise an error if there are duplicate resolved clean names.
-            # Note: `clean_name` is set in `FormField.save`.
+            seen_names = set()
+            duplicate_names = set()
 
-            clean_names = [
-                f.instance.clean_name or f.instance.get_field_clean_name()
-                for f in forms
-            ]
-            duplicate_clean_name = next(
-                (n for n in clean_names if clean_names.count(n) > 1), None
-            )
-            if duplicate_clean_name:
-                duplicate_form_field = next(
-                    f
-                    for f in self.formsets[related_name].forms
-                    if f.instance.get_field_clean_name() == duplicate_clean_name
-                )
-                duplicate_form_field.add_error(
-                    "label",
-                    django.forms.ValidationError(
-                        _(
-                            "There is another field with the label %(label_name)s, please change one of them."
-                        )
-                        % {"label_name": duplicate_form_field.instance.label}
-                    ),
+            # Find duplicates
+            for form in forms:
+                if form.cleaned_data.get("DELETE", False):
+                    continue
+
+                # Use existing clean_name or generate for new fields,
+                # raise an error if there are duplicate resolved clean names.
+                # Note: `clean_name` is set in `FormField.save`.
+                clean_name = (
+                    form.instance.clean_name or form.instance.get_field_clean_name()
                 )
 
+                if clean_name in seen_names:
+                    duplicate_names.add(clean_name)
+                else:
+                    seen_names.add(clean_name)
+
+            # Add validation errors to forms with duplicate clean names
+            if duplicate_names:
+                for form in forms:
+                    if form.cleaned_data.get("DELETE", False):
+                        continue
+
+                    clean_name = (
+                        form.instance.clean_name or form.instance.get_field_clean_name()
+                    )
+
+                    if clean_name in duplicate_names:
+                        if form.instance.clean_name:
+                            form.add_error(
+                                "label",
+                                django.forms.ValidationError(
+                                    _(
+                                        "The name '%(duplicate_clean_name)s' is "
+                                        "already assigned to this field and cannot be "
+                                        "changed. Please enter a different label for "
+                                        "the conflicting field."
+                                    )
+                                    % {"duplicate_clean_name": clean_name}
+                                ),
+                            )
+                        else:
+                            form.add_error(
+                                "label",
+                                django.forms.ValidationError(
+                                    _(
+                                        "Field name '%(duplicate_clean_name)s' "
+                                        "conflicts with another field. Please "
+                                        "change the label."
+                                    )
+                                    % {"duplicate_clean_name": clean_name}
+                                ),
+                            )
+
         return cleaned_data
