@@ -1,7 +1,6 @@
 import datetime
 import os
 
-from django.conf import settings
 from django.core.serializers.json import DjangoJSONEncoder
 from django.core.validators import validate_email
 from django.db import models
@@ -11,6 +10,7 @@ from django.utils.translation import gettext_lazy as _
 
 from wagtail.admin.mail import send_mail
 from wagtail.admin.panels import FieldPanel
+from wagtail.api import APIField
 from wagtail.contrib.forms.utils import get_field_clean_name
 from wagtail.models import Orderable, Page
 
@@ -93,6 +93,8 @@ class AbstractFormField(Orderable):
     field_type = models.CharField(
         verbose_name=_("field type"), max_length=16, choices=FORM_FIELD_CHOICES
     )
+    # field_type must be populated for previews to build the form field.
+    field_type.required_on_save = True
     required = models.BooleanField(verbose_name=_("required"), default=True)
     choices = models.TextField(
         verbose_name=_("choices"),
@@ -121,6 +123,16 @@ class AbstractFormField(Orderable):
         FieldPanel("default_value", classname="formbuilder-default"),
     ]
 
+    api_fields = [
+        APIField("clean_name"),
+        APIField("label"),
+        APIField("field_type"),
+        APIField("help_text"),
+        APIField("required"),
+        APIField("choices"),
+        APIField("default_value"),
+    ]
+
     def get_field_clean_name(self):
         """
         Prepare an ascii safe lower_snake_case variant of the field name to use as the field key.
@@ -152,10 +164,8 @@ class AbstractFormField(Orderable):
         ordering = ["sort_order"]
 
 
-class AbstractForm(Page):
-    """
-    A Form Page. Pages implementing a form should inherit from it
-    """
+class FormMixin:
+    """A mixin that adds form builder functionality to the page."""
 
     base_form_class = WagtailAdminFormPageForm
 
@@ -169,9 +179,6 @@ class AbstractForm(Page):
             name, ext = os.path.splitext(self.template)
             self.landing_page_template = name + "_landing" + ext
 
-    class Meta:
-        abstract = True
-
     def get_form_fields(self):
         """
         Form page expects `form_fields` to be declared.
@@ -209,7 +216,7 @@ class AbstractForm(Page):
 
         return form_class(*args, **form_params)
 
-    def get_landing_page_template(self, request, *args, **kwargs):
+    def get_landing_page_template(self, *args, **kwargs):
         return self.landing_page_template
 
     def get_submission_class(self):
@@ -258,10 +265,11 @@ class AbstractForm(Page):
         """
         Returns list submissions view for admin.
 
-        `list_submissions_view_class` can bse set to provide custom view class.
+        `list_submissions_view_class` can be set to provide custom view class.
         Your class must be inherited from SubmissionsListView.
         """
-        view = self.get_submissions_list_view_class().as_view()
+        results_only = kwargs.pop("results_only", False)
+        view = self.get_submissions_list_view_class().as_view(results_only=results_only)
         return view(request, form_page=self, *args, **kwargs)
 
     def serve(self, request, *args, **kwargs):
@@ -304,17 +312,23 @@ def validate_to_address(value):
         validate_email(address.strip())
 
 
-class AbstractEmailForm(AbstractForm):
-    """
-    A Form Page that sends email. Pages implementing a form to be send to an email should inherit from it
-    """
+class AbstractForm(FormMixin, Page):
+    """A Form Page. Pages implementing a form should inherit from it."""
+
+    class Meta:
+        abstract = True
+
+
+class EmailFormMixin(models.Model):
+    """A mixin that adds email sending functionality to the form."""
 
     to_address = models.CharField(
         verbose_name=_("to address"),
         max_length=255,
         blank=True,
         help_text=_(
-            "Optional - form submissions will be emailed to these addresses. Separate multiple addresses by comma."
+            "Optional - form submissions will be emailed to these addresses. "
+            "Separate multiple addresses by comma."
         ),
         validators=[validate_to_address],
     )
@@ -351,15 +365,24 @@ class AbstractEmailForm(AbstractForm):
             if isinstance(value, list):
                 value = ", ".join(value)
 
-            # Format dates and datetimes with SHORT_DATE(TIME)_FORMAT
+            # Format dates and datetime(s) with SHORT_DATE(TIME)_FORMAT
             if isinstance(value, datetime.datetime):
-                value = date_format(value, settings.SHORT_DATETIME_FORMAT)
+                value = date_format(value, "SHORT_DATETIME_FORMAT")
             elif isinstance(value, datetime.date):
-                value = date_format(value, settings.SHORT_DATE_FORMAT)
+                value = date_format(value, "SHORT_DATE_FORMAT")
 
-            content.append("{}: {}".format(field.label, value))
+            content.append(f"{field.label}: {value}")
 
         return "\n".join(content)
 
     class Meta:
         abstract = True
+
+
+class AbstractEmailForm(EmailFormMixin, FormMixin, Page):
+    """
+    A Form Page that sends email. Inherit from it if your form sends an email.
+    """
+
+    class Meta:
+        abstract = True
