@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-import unittest
-
-from django import VERSION as DJANGO_VERSION
 from django.core import mail
 from django.core.exceptions import ValidationError
 from django.test import TestCase, override_settings
@@ -18,6 +14,7 @@ from wagtail.test.testapp.models import (
     CustomFormPageSubmission,
     ExtendedFormField,
     FormField,
+    FormFieldWithCustomSubmission,
     FormPageWithCustomFormBuilder,
     JadeFormPage,
 )
@@ -45,10 +42,6 @@ class TestFormSubmission(TestCase):
         # check that variables defined in get_context are passed through to the template (#1429)
         self.assertContains(response, "<p>hello world</p>")
 
-    @unittest.skipIf(
-        (4, 0) <= DJANGO_VERSION < (4, 0, 2),
-        "help_text is erroneously escaped in Django 4.0 - 4.0.1: https://code.djangoproject.com/ticket/33419",
-    )
     @override_settings(WAGTAILFORMS_HELP_TEXT_ALLOW_HTML=True)
     def test_get_form_without_help_text_escaping(self):
         response = self.client.get("/contact-us/")
@@ -182,8 +175,28 @@ class TestFormSubmission(TestCase):
         with self.assertRaises(ValidationError):
             make_form_page(from_address="not an email")
 
+    def test_string_representation_form_submission(self):
+        """
+        Ensure that a form submission can be logged / printed without error.
+        Broke when converting field to JSON - see #8927
+        """
 
-class TestFormWithCustomSubmission(TestCase, WagtailTestUtils):
+        self.client.post(
+            "/contact-us/",
+            {
+                "your_email": "bob@example.com",
+                "your_message": "hello world",
+                "your_choices": {},
+            },
+        )
+
+        self.assertGreaterEqual(FormSubmission.objects.count(), 1)
+
+        submission = FormSubmission.objects.first()
+        self.assertIn("hello world", str(submission))
+
+
+class TestFormWithCustomSubmission(WagtailTestUtils, TestCase):
     def setUp(self):
         # Create a form page
         self.form_page = make_form_page_with_custom_submission()
@@ -459,7 +472,7 @@ class TestFormSubmissionWithMultipleRecipients(TestCase):
 
 
 class TestFormSubmissionWithMultipleRecipientsAndWithCustomSubmission(
-    TestCase, WagtailTestUtils
+    WagtailTestUtils, TestCase
 ):
     def setUp(self):
         # Create a form page
@@ -545,9 +558,8 @@ class TestFormWithRedirect(TestCase):
         )
 
 
-class TestFormPageWithCustomFormBuilder(TestCase, WagtailTestUtils):
+class TestFormPageWithCustomFormBuilder(WagtailTestUtils, TestCase):
     def setUp(self):
-
         home_page = Page.objects.get(url_path="/home/")
         form_page = home_page.add_child(
             instance=FormPageWithCustomFormBuilder(
@@ -591,11 +603,13 @@ class TestFormPageWithCustomFormBuilder(TestCase, WagtailTestUtils):
             html=True,
         )
         # check ip address field has rendered
-        self.assertContains(
-            response,
-            '<input type="text" name="device_ip_address" required id="id_device_ip_address" />',
-            html=True,
-        )
+        # (not comparing HTML directly because https://docs.djangoproject.com/en/5.1/releases/5.1.5/
+        # added a maxlength attribute)
+        soup = self.get_soup(response.content)
+        input = soup.find("input", {"name": "device_ip_address"})
+        self.assertEqual(input["type"], "text")
+        self.assertEqual(input["required"], "")
+        self.assertEqual(input["id"], "id_device_ip_address")
 
     def test_post_invalid_form(self):
         response = self.client.post(
@@ -758,8 +772,34 @@ class TestCleanedDataEmails(TestCase):
         self.assertEqual(len(mail.outbox), 3)
         self.assertIn("Datetime: 12/21/1910 9:19 p.m.", mail.outbox[2].body)
 
+    @override_settings(USE_I18N=True, LANGUAGE_CODE="de")
+    def test_date_localization(self):
+        self.client.post(
+            "/contact-us/",
+            {
+                "date": "2017-12-31",
+            },
+        )
 
-class TestIssue798(TestCase, WagtailTestUtils):
+        # Check the email
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertIn("Date: 31.12.2017", mail.outbox[0].body)
+
+    @override_settings(USE_I18N=True, LANGUAGE_CODE="de")
+    def test_datetime_localization(self):
+        self.client.post(
+            "/contact-us/",
+            {
+                "datetime": "1910-12-21 21:19:12",
+            },
+        )
+
+        # Check the email
+        self.assertEqual(len(mail.outbox), 1)
+        self.assertIn("Datetime: 21.12.1910 21:19", mail.outbox[0].body)
+
+
+class TestIssue798(WagtailTestUtils, TestCase):
     fixtures = ["test.json"]
 
     def setUp(self):
@@ -808,3 +848,56 @@ class TestNonHtmlExtension(TestCase):
         self.assertEqual(
             form_page.landing_page_template, "tests/form_page_landing.jade"
         )
+
+
+class TestFormFieldCleanNameCreation(WagtailTestUtils, TestCase):
+    fixtures = ["test.json"]
+
+    def setUp(self):
+        self.login(username="siteeditor", password="password")
+        self.form_page = Page.objects.get(
+            url_path="/home/contact-us-one-more-time/"
+        ).specific
+
+    def test_form_field_clean_name_creation(self):
+        """creating a new field should use clean_name format (anyascii snake_case)"""
+
+        field = FormFieldWithCustomSubmission.objects.create(
+            page=self.form_page,
+            label="Telefón-nummer",
+            field_type="number",
+        )
+
+        self.assertEqual(field.clean_name, "telefon_nummer")
+
+
+class TestFormFieldCleanNameCreationOverride(WagtailTestUtils, TestCase):
+    def setUp(self):
+        # Create a form page
+        home_page = Page.objects.get(url_path="/home/")
+
+        self.form_page = home_page.add_child(
+            instance=FormPageWithCustomFormBuilder(
+                title="Richiesta Gelato",
+                slug="ice-cream-request",
+                to_address="scoops@pro-eis.co.it",
+                from_address="scoops@pro-eis.co.it",
+                subject="Gelato in arrivo",
+            )
+        )
+
+    def test_form_field_clean_name_override(self):
+        """
+        Creating a new field should use the overridden method
+        See ExtendedFormField get_field_clean_name method
+        """
+
+        field = ExtendedFormField.objects.create(
+            page=self.form_page,
+            sort_order=1,
+            label="quanti ge·là·to?",
+            field_type="number",  # only number fields will add the ID as a prefix to the clean_name
+            required=True,
+        )
+
+        self.assertEqual(field.clean_name, "number_field--quanti_gelato")
