@@ -5,7 +5,7 @@ import requests
 from azure.mgmt.cdn import CdnManagementClient
 from azure.mgmt.frontdoor import FrontDoorManagementClient
 from django.core.exceptions import ImproperlyConfigured
-from django.test import TestCase
+from django.test import SimpleTestCase, TestCase
 from django.test.utils import override_settings
 
 from wagtail.contrib.frontend_cache.backends import (
@@ -18,7 +18,7 @@ from wagtail.contrib.frontend_cache.backends import (
 )
 from wagtail.contrib.frontend_cache.utils import get_backends
 from wagtail.models import Page
-from wagtail.test.testapp.models import EventIndex
+from wagtail.test.testapp.models import EventIndex, EventPage
 
 from .utils import (
     PurgeBatch,
@@ -28,8 +28,17 @@ from .utils import (
     purge_urls_from_cache,
 )
 
+EVENTPAGE_URLS = {
+    "http://localhost/events/final-event/",
+    "http://localhost/events/christmas/",
+    "http://localhost/events/saint-patrick/",
+    "http://localhost/events/tentative-unpublished-event/",
+    "http://localhost/events/someone-elses-event/",
+    "http://localhost/secret-plans/steal-underpants/",
+}
 
-class TestBackendConfiguration(TestCase):
+
+class TestBackendConfiguration(SimpleTestCase):
     def test_default(self):
         backends = get_backends()
 
@@ -81,6 +90,8 @@ class TestBackendConfiguration(TestCase):
                 "cloudfront": {
                     "BACKEND": "wagtail.contrib.frontend_cache.backends.CloudfrontBackend",
                     "DISTRIBUTION_ID": "frontend",
+                    "AWS_ACCESS_KEY_ID": "my-access-key-id",
+                    "AWS_SECRET_ACCESS_KEY": "my-secret-access-key",
                 },
             }
         )
@@ -90,6 +101,12 @@ class TestBackendConfiguration(TestCase):
 
         self.assertEqual(backends["cloudfront"].cloudfront_distribution_id, "frontend")
 
+        credentials = backends["cloudfront"].client._request_signer._credentials
+
+        self.assertEqual(credentials.method, "explicit")
+        self.assertEqual(credentials.access_key, "my-access-key-id")
+        self.assertEqual(credentials.secret_key, "my-secret-access-key")
+
     def test_azure_cdn(self):
         backends = get_backends(
             backend_settings={
@@ -149,8 +166,8 @@ class TestBackendConfiguration(TestCase):
         self.assertEqual(set(backends.keys()), {"azure_cdn"})
         client = backends["azure_cdn"]._get_client()
         self.assertIsInstance(client, CdnManagementClient)
-        self.assertEqual(client.config.subscription_id, "fake-subscription-id")
-        self.assertIs(client.config.credentials, mock_credentials)
+        self.assertEqual(client._config.subscription_id, "fake-subscription-id")
+        self.assertIs(client._config.credential, mock_credentials)
 
     def test_azure_front_door_get_client(self):
         mock_credentials = mock.MagicMock()
@@ -168,11 +185,11 @@ class TestBackendConfiguration(TestCase):
         client = backends["azure_front_door"]._get_client()
         self.assertEqual(set(backends.keys()), {"azure_front_door"})
         self.assertIsInstance(client, FrontDoorManagementClient)
-        self.assertEqual(client.config.subscription_id, "fake-subscription-id")
-        self.assertIs(client.config.credentials, mock_credentials)
+        self.assertEqual(client._config.subscription_id, "fake-subscription-id")
+        self.assertIs(client._config.credential, mock_credentials)
 
     @mock.patch(
-        "wagtail.contrib.frontend_cache.backends.AzureCdnBackend._make_purge_call"
+        "wagtail.contrib.frontend_cache.backends.azure.AzureCdnBackend._make_purge_call"
     )
     def test_azure_cdn_purge(self, make_purge_call_mock):
         backends = get_backends(
@@ -214,7 +231,7 @@ class TestBackendConfiguration(TestCase):
         self.assertEqual(call_args[1], ["/home/events/christmas/?test=1", "/blog/"])
 
     @mock.patch(
-        "wagtail.contrib.frontend_cache.backends.AzureFrontDoorBackend._make_purge_call"
+        "wagtail.contrib.frontend_cache.backends.azure.AzureFrontDoorBackend._make_purge_call"
     )
     def test_azure_front_door_purge(self, make_purge_call_mock):
         backends = get_backends(
@@ -285,7 +302,7 @@ class TestBackendConfiguration(TestCase):
             log_output.output[0],
         )
 
-    @mock.patch("wagtail.contrib.frontend_cache.backends.urlopen")
+    @mock.patch("wagtail.contrib.frontend_cache.backends.http.urlopen")
     def _test_http_with_side_effect(self, urlopen_mock, urlopen_side_effect):
         # given a backends configuration with one HTTP backend
         backends = get_backends(
@@ -323,26 +340,75 @@ class TestBackendConfiguration(TestCase):
             )
 
     @mock.patch(
-        "wagtail.contrib.frontend_cache.backends.CloudfrontBackend._create_invalidation"
+        "wagtail.contrib.frontend_cache.backends.cloudfront.CloudfrontBackend._create_invalidation"
     )
-    def test_cloudfront_distribution_id_mapping(self, _create_invalidation):
+    def test_cloudfront_purge_with_query_string(self, mock_create_invalidation):
+        """Test that CloudFront invalidation includes query strings when purging URLs."""
         backends = get_backends(
             backend_settings={
                 "cloudfront": {
                     "BACKEND": "wagtail.contrib.frontend_cache.backends.CloudfrontBackend",
-                    "DISTRIBUTION_ID": {
-                        "www.wagtail.org": "frontend",
-                    },
+                    "DISTRIBUTION_ID": "frontend",
+                    "AWS_ACCESS_KEY_ID": "test-access-key",
+                    "AWS_SECRET_ACCESS_KEY": "test-secret-key",
                 },
             }
         )
-        backends.get("cloudfront").purge(
-            "http://www.wagtail.org/home/events/christmas/"
-        )
-        backends.get("cloudfront").purge("http://torchbox.com/blog/")
 
-        _create_invalidation.assert_called_once_with(
-            "frontend", ["/home/events/christmas/"]
+        backends["cloudfront"].purge(
+            "http://www.example.com/path/to/page?query=value&another=param"
+        )
+        mock_create_invalidation.assert_called_once_with(
+            "frontend", ["/path/to/page?query=value&another=param"]
+        )
+
+    @mock.patch(
+        "wagtail.contrib.frontend_cache.backends.cloudfront.CloudfrontBackend._create_invalidation"
+    )
+    def test_cloudfront_purge_root_with_query_string(self, mock_create_invalidation):
+        """Test that CloudFront invalidation handles root URLs with query strings."""
+        backends = get_backends(
+            backend_settings={
+                "cloudfront": {
+                    "BACKEND": "wagtail.contrib.frontend_cache.backends.CloudfrontBackend",
+                    "DISTRIBUTION_ID": "frontend",
+                    "AWS_ACCESS_KEY_ID": "test-access-key",
+                    "AWS_SECRET_ACCESS_KEY": "test-secret-key",
+                },
+            }
+        )
+
+        backends["cloudfront"].purge("http://www.example.com?query=value")
+        mock_create_invalidation.assert_called_once_with("frontend", ["/?query=value"])
+
+    @mock.patch(
+        "wagtail.contrib.frontend_cache.backends.cloudfront.CloudfrontBackend._create_invalidation"
+    )
+    def test_cloudfront_purge_batch_with_query_strings(self, mock_create_invalidation):
+        """Test that CloudFront invalidation batch includes query strings."""
+        backends = get_backends(
+            backend_settings={
+                "cloudfront": {
+                    "BACKEND": "wagtail.contrib.frontend_cache.backends.CloudfrontBackend",
+                    "DISTRIBUTION_ID": "frontend",
+                    "AWS_ACCESS_KEY_ID": "test-access-key",
+                    "AWS_SECRET_ACCESS_KEY": "test-secret-key",
+                },
+            }
+        )
+
+        backends["cloudfront"].purge_batch(
+            [
+                "http://www.example.com/path/to/page?query=value",
+                "http://www.example.com/another/path",
+                "http://www.example.com?root=query",
+            ]
+        )
+        mock_create_invalidation.assert_called_once()
+        self.assertEqual(mock_create_invalidation.call_args.args[0], "frontend")
+        self.assertEqual(
+            sorted(mock_create_invalidation.call_args.args[1]),
+            ["/?root=query", "/another/path", "/path/to/page?query=value"],
         )
 
     def test_multiple(self):
@@ -392,26 +458,20 @@ class TestBackendConfiguration(TestCase):
         self.assertEqual(backends["default"].cache_netloc, "localhost:8000")
 
 
-PURGED_URLS = []
+PURGED_URLS = set()
 
 
 class MockBackend(BaseBackend):
-    def __init__(self, config):
-        pass
-
     def purge(self, url):
-        PURGED_URLS.append(url)
+        PURGED_URLS.add(url)
 
 
 class MockCloudflareBackend(CloudflareBackend):
-    def __init__(self, config):
-        pass
-
     def _purge_urls(self, urls):
         if len(urls) > self.CHUNK_SIZE:
             raise Exception("Cloudflare backend is not chunking requests as expected")
 
-        PURGED_URLS.extend(urls)
+        PURGED_URLS.update(urls)
 
 
 @override_settings(
@@ -419,73 +479,195 @@ class MockCloudflareBackend(CloudflareBackend):
         "varnish": {
             "BACKEND": "wagtail.contrib.frontend_cache.tests.MockBackend",
         },
-    }
+    },
+    CACHES={
+        "default": {
+            "BACKEND": "django.core.cache.backends.dummy.DummyCache",
+        }
+    },
 )
 class TestCachePurgingFunctions(TestCase):
-
     fixtures = ["test.json"]
 
     def setUp(self):
-        # Reset PURGED_URLS to an empty list
-        PURGED_URLS[:] = []
+        PURGED_URLS.clear()
 
     def test_purge_url_from_cache(self):
-        purge_url_from_cache("http://localhost/foo")
-        self.assertEqual(PURGED_URLS, ["http://localhost/foo"])
+        with self.captureOnCommitCallbacks(execute=True):
+            purge_url_from_cache("http://localhost/foo")
+        self.assertEqual(PURGED_URLS, {"http://localhost/foo"})
 
     def test_purge_urls_from_cache(self):
-        purge_urls_from_cache(["http://localhost/foo", "http://localhost/bar"])
-        self.assertEqual(PURGED_URLS, ["http://localhost/foo", "http://localhost/bar"])
+        with self.captureOnCommitCallbacks(execute=True):
+            purge_urls_from_cache(["http://localhost/foo", "http://localhost/bar"])
+        self.assertEqual(PURGED_URLS, {"http://localhost/foo", "http://localhost/bar"})
 
     def test_purge_page_from_cache(self):
         page = EventIndex.objects.get(url_path="/home/events/")
-        purge_page_from_cache(page)
+        with self.captureOnCommitCallbacks(execute=True):
+            with self.assertNumQueries(1):
+                # Because no cache object is provided, a query is needed to
+                # fetch site root paths in order to derive page urls
+                purge_page_from_cache(page)
         self.assertEqual(
-            PURGED_URLS, ["http://localhost/events/", "http://localhost/events/past/"]
+            PURGED_URLS,
+            {"http://localhost/events/", "http://localhost/events/past/"},
         )
 
-    def test_purge_pages_from_cache(self):
-        purge_pages_from_cache(EventIndex.objects.all())
-        self.assertEqual(
-            PURGED_URLS, ["http://localhost/events/", "http://localhost/events/past/"]
-        )
-
-    def test_purge_batch(self):
-        batch = PurgeBatch()
+    def test_purge_page_from_cache_with_shared_cache_object(self):
         page = EventIndex.objects.get(url_path="/home/events/")
-        batch.add_page(page)
-        batch.add_url("http://localhost/foo")
-        batch.purge()
+
+        # Ensure site root paths are already cached, which should result in
+        # zero additional queries being incurred by this test
+        page._get_relevant_site_root_paths()
+
+        with self.captureOnCommitCallbacks(execute=True):
+            # Because site root paths are already available via the cache_object,
+            # no further queries should be needed to derive page urls
+            with self.assertNumQueries(0):
+                purge_page_from_cache(page, cache_object=page)
 
         self.assertEqual(
             PURGED_URLS,
-            [
+            {"http://localhost/events/", "http://localhost/events/past/"},
+        )
+
+    def test_purge_pages_from_cache(self):
+        pages = list(Page.objects.all().type(EventPage))
+        with self.captureOnCommitCallbacks(execute=True):
+            with self.assertNumQueries(1):
+                # Because no cache object is provided, a query is needed to
+                # fetch site root paths in order to derive page urls
+                purge_pages_from_cache(pages)
+        self.assertEqual(PURGED_URLS, EVENTPAGE_URLS)
+
+    def test_purge_pages_from_cache_with_shared_cache_object(self):
+        pages = list(Page.objects.all().type(EventPage))
+
+        # Use the first page as the cache object for the operation
+        cache_object = pages[0]
+
+        # Ensure site root paths are already cached, which should result in
+        # zero additional queries being incurred by this test
+        cache_object._get_relevant_site_root_paths()
+
+        with self.captureOnCommitCallbacks(execute=True):
+            # Because site root paths are already available via the cache_object,
+            # no further queries should be needed to derive page urls
+            with self.assertNumQueries(0):
+                purge_pages_from_cache(pages, cache_object=cache_object)
+
+        self.assertEqual(PURGED_URLS, EVENTPAGE_URLS)
+
+    def test_purge_batch(self):
+        page = EventIndex.objects.get(url_path="/home/events/")
+        batch = PurgeBatch()
+
+        # Because no cache object is provided, a query is needed to
+        # fetch site root paths in order to derive page urls
+        with self.assertNumQueries(1):
+            batch.add_page(page)
+
+        batch.add_url("http://localhost/foo")
+
+        with self.captureOnCommitCallbacks(execute=True):
+            batch.purge()
+
+        self.assertEqual(
+            PURGED_URLS,
+            {
                 "http://localhost/events/",
                 "http://localhost/events/past/",
                 "http://localhost/foo",
-            ],
+            },
         )
 
+    def test_purge_batch_with_multiple_pages(self):
+        pages = list(Page.objects.all().type(EventPage))
+        batch = PurgeBatch()
+
+        # Because the batch has no cache object, a query is needed to
+        # fetch site root paths in order to derive page urls
+        with self.assertNumQueries(1):
+            batch.add_pages(pages)
+
+        with self.captureOnCommitCallbacks(execute=True):
+            batch.purge()
+
+        self.assertEqual(PURGED_URLS, EVENTPAGE_URLS)
+
+    def test_multiple_purge_batches_with_shared_cache_object(self):
+        pages = list(Page.objects.all().type(EventPage))
+
+        # Use the first page as the cache object for the batch
+        cache_object = pages[0]
+
+        # Ensure site root paths are already cached, which should result in
+        # zero additional queries being incurred by this test
+        cache_object._get_relevant_site_root_paths()
+
+        batch = PurgeBatch(cache_object=cache_object)
+
+        with self.assertNumQueries(0):
+            # Because site root paths are already available via the cache_object,
+            # no queries should be needed to derive page urls
+            batch.add_pages(pages)
+
+        with self.captureOnCommitCallbacks(execute=True):
+            batch.purge()
+
+        self.assertEqual(PURGED_URLS, EVENTPAGE_URLS)
+        PURGED_URLS.clear()
+
+    @override_settings(
+        WAGTAILFRONTENDCACHE={
+            "varnish": {
+                "BACKEND": "wagtail.contrib.frontend_cache.tests.MockBackend",
+                "HOSTNAMES": ["example.com"],
+            },
+        }
+    )
+    def test_invalidate_specific_location(self):
+        with self.assertLogs(
+            logger="wagtail.frontendcache",
+            level="INFO",
+        ) as log_output:
+            with self.captureOnCommitCallbacks(execute=True):
+                purge_url_from_cache("http://localhost/foo")
+
+        self.assertEqual(PURGED_URLS, set())
+        self.assertIn(
+            "Unable to find purge backend for localhost",
+            log_output.output[0],
+        )
+
+        with self.captureOnCommitCallbacks(execute=True):
+            purge_url_from_cache("http://example.com/foo")
+        self.assertEqual(PURGED_URLS, {"http://example.com/foo"})
+
 
 @override_settings(
     WAGTAILFRONTENDCACHE={
         "cloudflare": {
             "BACKEND": "wagtail.contrib.frontend_cache.tests.MockCloudflareBackend",
+            "ZONEID": "zone",
+            "BEARER_TOKEN": "token",
         },
     }
 )
 class TestCloudflareCachePurgingFunctions(TestCase):
     def setUp(self):
         # Reset PURGED_URLS to an empty list
-        PURGED_URLS[:] = []
+        PURGED_URLS.clear()
 
     def test_cloudflare_purge_batch_chunked(self):
-        batch = PurgeBatch()
-        urls = ["https://localhost/foo{}".format(i) for i in range(1, 65)]
-        batch.add_urls(urls)
-        batch.purge()
+        with self.captureOnCommitCallbacks(execute=True):
+            batch = PurgeBatch()
+            urls = [f"https://localhost/foo{i}" for i in range(1, 65)]
+            batch.add_urls(urls)
+            batch.purge()
 
-        self.assertCountEqual(PURGED_URLS, urls)
+        self.assertCountEqual(PURGED_URLS, set(urls))
 
 
 @override_settings(
@@ -496,33 +678,35 @@ class TestCloudflareCachePurgingFunctions(TestCase):
     }
 )
 class TestCachePurgingSignals(TestCase):
-
     fixtures = ["test.json"]
 
     def setUp(self):
         # Reset PURGED_URLS to an empty list
-        PURGED_URLS[:] = []
+        PURGED_URLS.clear()
 
     def test_purge_on_publish(self):
-        page = EventIndex.objects.get(url_path="/home/events/")
-        page.save_revision().publish()
+        with self.captureOnCommitCallbacks(execute=True):
+            page = EventIndex.objects.get(url_path="/home/events/")
+            page.save_revision().publish()
         self.assertEqual(
-            PURGED_URLS, ["http://localhost/events/", "http://localhost/events/past/"]
+            PURGED_URLS, {"http://localhost/events/", "http://localhost/events/past/"}
         )
 
     def test_purge_on_unpublish(self):
-        page = EventIndex.objects.get(url_path="/home/events/")
-        page.unpublish()
+        with self.captureOnCommitCallbacks(execute=True):
+            page = EventIndex.objects.get(url_path="/home/events/")
+            page.unpublish()
         self.assertEqual(
-            PURGED_URLS, ["http://localhost/events/", "http://localhost/events/past/"]
+            PURGED_URLS, {"http://localhost/events/", "http://localhost/events/past/"}
         )
 
     def test_purge_with_unroutable_page(self):
-        root = Page.objects.get(url_path="/")
-        page = EventIndex(title="new top-level page")
-        root.add_child(instance=page)
-        page.save_revision().publish()
-        self.assertEqual(PURGED_URLS, [])
+        with self.captureOnCommitCallbacks(execute=True):
+            root = Page.objects.get(url_path="/")
+            page = EventIndex(title="new top-level page")
+            root.add_child(instance=page)
+            page.save_revision().publish()
+        self.assertEqual(PURGED_URLS, set())
 
     @override_settings(
         ROOT_URLCONF="wagtail.test.urls_multilang",
@@ -530,20 +714,20 @@ class TestCachePurgingSignals(TestCase):
         WAGTAILFRONTENDCACHE_LANGUAGES=["en", "fr", "pt-br"],
     )
     def test_purge_on_publish_in_multilang_env(self):
-        PURGED_URLS[:] = []  # reset PURGED_URLS to the empty list
-        page = EventIndex.objects.get(url_path="/home/events/")
-        page.save_revision().publish()
+        with self.captureOnCommitCallbacks(execute=True):
+            page = EventIndex.objects.get(url_path="/home/events/")
+            page.save_revision().publish()
 
         self.assertEqual(
             PURGED_URLS,
-            [
+            {
                 "http://localhost/en/events/",
                 "http://localhost/en/events/past/",
                 "http://localhost/fr/events/",
                 "http://localhost/fr/events/past/",
                 "http://localhost/pt-br/events/",
                 "http://localhost/pt-br/events/past/",
-            ],
+            },
         )
 
     @override_settings(
@@ -553,18 +737,18 @@ class TestCachePurgingSignals(TestCase):
         WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")],
     )
     def test_purge_on_publish_with_i18n_enabled(self):
-        PURGED_URLS[:] = []  # reset PURGED_URLS to the empty list
-        page = EventIndex.objects.get(url_path="/home/events/")
-        page.save_revision().publish()
+        with self.captureOnCommitCallbacks(execute=True):
+            page = EventIndex.objects.get(url_path="/home/events/")
+            page.save_revision().publish()
 
         self.assertEqual(
             PURGED_URLS,
-            [
+            {
                 "http://localhost/en/events/",
                 "http://localhost/en/events/past/",
                 "http://localhost/fr/events/",
                 "http://localhost/fr/events/past/",
-            ],
+            },
         )
 
     @override_settings(
@@ -573,13 +757,13 @@ class TestCachePurgingSignals(TestCase):
         WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")],
     )
     def test_purge_on_publish_without_i18n_enabled(self):
-        # It should ignore WAGTAIL_CONTENT_LANGUAGES as WAGTAIL_I18N_ENABLED isn't set
-        PURGED_URLS[:] = []  # reset PURGED_URLS to the empty list
-        page = EventIndex.objects.get(url_path="/home/events/")
-        page.save_revision().publish()
+        with self.captureOnCommitCallbacks(execute=True):
+            # It should ignore WAGTAIL_CONTENT_LANGUAGES as WAGTAIL_I18N_ENABLED isn't set
+            page = EventIndex.objects.get(url_path="/home/events/")
+            page.save_revision().publish()
         self.assertEqual(
             PURGED_URLS,
-            ["http://localhost/en/events/", "http://localhost/en/events/past/"],
+            {"http://localhost/en/events/", "http://localhost/en/events/past/"},
         )
 
 
@@ -593,13 +777,13 @@ class TestPurgeBatchClass(TestCase):
         batch = PurgeBatch()
         batch.add_url("http://localhost/foo")
 
-        self.assertEqual(batch.urls, ["http://localhost/foo"])
+        self.assertEqual(batch.urls, {"http://localhost/foo"})
 
     def test_add_urls(self):
         batch = PurgeBatch()
         batch.add_urls(["http://localhost/foo", "http://localhost/bar"])
 
-        self.assertEqual(batch.urls, ["http://localhost/foo", "http://localhost/bar"])
+        self.assertEqual(batch.urls, {"http://localhost/foo", "http://localhost/bar"})
 
     def test_add_page(self):
         page = EventIndex.objects.get(url_path="/home/events/")
@@ -608,7 +792,7 @@ class TestPurgeBatchClass(TestCase):
         batch.add_page(page)
 
         self.assertEqual(
-            batch.urls, ["http://localhost/events/", "http://localhost/events/past/"]
+            batch.urls, {"http://localhost/events/", "http://localhost/events/past/"}
         )
 
     def test_add_pages(self):
@@ -616,7 +800,7 @@ class TestPurgeBatchClass(TestCase):
         batch.add_pages(EventIndex.objects.all())
 
         self.assertEqual(
-            batch.urls, ["http://localhost/events/", "http://localhost/events/past/"]
+            batch.urls, {"http://localhost/events/", "http://localhost/events/past/"}
         )
 
     def test_multiple_calls(self):
@@ -629,14 +813,14 @@ class TestPurgeBatchClass(TestCase):
 
         self.assertEqual(
             batch.urls,
-            [
+            {
                 "http://localhost/events/",
                 "http://localhost/events/past/",
                 "http://localhost/foo",
-            ],
+            },
         )
 
-    @mock.patch("wagtail.contrib.frontend_cache.backends.requests.delete")
+    @mock.patch("wagtail.contrib.frontend_cache.backends.cloudflare.requests.delete")
     def test_http_error_on_cloudflare_purge_batch(self, requests_delete_mock):
         backend_settings = {
             "cloudflare": {
@@ -656,13 +840,12 @@ class TestPurgeBatchClass(TestCase):
         )
         requests_delete_mock.side_effect = http_error
 
-        page = EventIndex.objects.get(url_path="/home/events/")
-
         batch = PurgeBatch()
-        batch.add_page(page)
+        batch.add_url("http://localhost/events/")
 
         with self.assertLogs(level="ERROR") as log_output:
-            batch.purge(backend_settings=backend_settings)
+            with self.captureOnCommitCallbacks(execute=True):
+                batch.purge(backend_settings=backend_settings)
 
         self.assertIn(
             "Couldn't purge 'http://localhost/events/' from Cloudflare. HTTPError: 500",
