@@ -23,10 +23,10 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
+
 # Copied from: https://raw.githubusercontent.com/django-import-export/django-import-export/5795e114210adf250ac6e146db2fa413f38875de/import_export/tmp_storages.py
 import os
 import tempfile
-
 from uuid import uuid4
 
 from django.core.cache import cache
@@ -35,14 +35,13 @@ from django.core.files.storage import default_storage
 
 
 class BaseStorage:
-
     def __init__(self, name=None):
         self.name = name
 
-    def save(self, data, mode='w'):
+    def save(self, data, mode="w"):
         raise NotImplementedError
 
-    def read(self, read_mode='r'):
+    def read(self, read_mode="r"):
         raise NotImplementedError
 
     def remove(self):
@@ -50,8 +49,7 @@ class BaseStorage:
 
 
 class TempFolderStorage(BaseStorage):
-
-    def open(self, mode='r'):
+    def open(self, mode="r"):
         if self.name:
             return open(self.get_full_path(), mode)
         else:
@@ -59,11 +57,11 @@ class TempFolderStorage(BaseStorage):
             self.name = tmp_file.name
             return tmp_file
 
-    def save(self, data, mode='w'):
+    def save(self, data, mode="w"):
         with self.open(mode=mode) as file:
             file.write(data)
 
-    def read(self, mode='r'):
+    def read(self, mode="r"):
         with self.open(mode=mode) as file:
             return file.read()
 
@@ -71,40 +69,38 @@ class TempFolderStorage(BaseStorage):
         os.remove(self.get_full_path())
 
     def get_full_path(self):
-        return os.path.join(
-            tempfile.gettempdir(),
-            self.name
-        )
+        return os.path.join(tempfile.gettempdir(), self.name)
 
 
 class CacheStorage(BaseStorage):
     """
     By default memcache maximum size per key is 1MB, be careful with large files.
     """
+
     CACHE_LIFETIME = 86400
-    CACHE_PREFIX = 'django-import-export-'
+    CACHE_PREFIX = "django-import-export-"
 
     def save(self, data, mode=None):
         if not self.name:
             self.name = uuid4().hex
         cache.set(self.CACHE_PREFIX + self.name, data, self.CACHE_LIFETIME)
 
-    def read(self, read_mode='r'):
+    def read(self, read_mode="r"):
         return cache.get(self.CACHE_PREFIX + self.name)
 
     def remove(self):
-        cache.delete(self.name)
+        cache.delete(self.CACHE_PREFIX + self.name)
 
 
 class MediaStorage(BaseStorage):
-    MEDIA_FOLDER = 'django-import-export'
+    MEDIA_FOLDER = "django-import-export"
 
     def save(self, data, mode=None):
         if not self.name:
             self.name = uuid4().hex
         default_storage.save(self.get_full_path(), ContentFile(data))
 
-    def read(self, read_mode='rb'):
+    def read(self, read_mode="rb"):
         with default_storage.open(self.get_full_path(), mode=read_mode) as f:
             return f.read()
 
@@ -112,7 +108,4 @@ class MediaStorage(BaseStorage):
         default_storage.delete(self.get_full_path())
 
     def get_full_path(self):
-        return os.path.join(
-            self.MEDIA_FOLDER,
-            self.name
-        )
+        return os.path.join(self.MEDIA_FOLDER, self.name)
