@@ -246,7 +246,7 @@ def start_import(request):
     if extension not in supported_extensions:
         messages.error(
             request,
-            _('File format of type "%(extension)s" is not supported')
+            _('File format of type "%(extension)s" is not supported.')
             % {"extension": extension},
         )
         return redirect("wagtailredirects:start_import")
@@ -263,14 +263,14 @@ def start_import(request):
     except UnicodeDecodeError as e:
         messages.error(
             request,
-            _("Imported file has a wrong encoding: %(error_message)s")
+            _("Imported file has a wrong encoding: %(error_message)s.")
             % {"error_message": e},
         )
         return redirect("wagtailredirects:start_import")
     except Exception as e:  # noqa: BLE001; pragma: no cover
         messages.error(
             request,
-            _("%(error)s encountered while trying to read file: %(filename)s")
+            _("%(error)s encountered while trying to read file: %(filename)s.")
             % {"error": type(e).__name__, "filename": import_file.name},
         )
         return redirect("wagtailredirects:start_import")
