@@ -1,8 +1,8 @@
-from django.http import HttpRequest
 from django.template import Context, RequestContext, Template, engines
 from django.test import TestCase
 from django.test.utils import override_settings
 
+from wagtail.coreutils import get_dummy_request
 from wagtail.models import Site
 from wagtail.test.utils import WagtailTestUtils
 
@@ -10,7 +10,7 @@ from .base import SiteSettingsTestMixin
 
 
 @override_settings(ALLOWED_HOSTS=["testserver", "localhost", "other"])
-class TemplateTestCase(SiteSettingsTestMixin, TestCase, WagtailTestUtils):
+class TemplateTestCase(SiteSettingsTestMixin, WagtailTestUtils, TestCase):
     def render(self, request, string, context=None, site=None):
         template = Template(string)
         context = RequestContext(request, context)
@@ -26,7 +26,7 @@ class TestContextProcessor(TemplateTestCase):
             self.default_settings.title,
         )
 
-    def test_multisite(self):
+    def test_multi_site(self):
         """Check that the correct setting for the current site is returned"""
         request = self.get_request(site=self.default_site)
         self.assertEqual(
@@ -174,7 +174,7 @@ class TestTemplateTag(TemplateTestCase):
 
         self.assertEqual(template.render(context), "")
 
-    def test_get_settings_variable_assigment_use_default(self):
+    def test_get_settings_variable_assignment_use_default(self):
         """
         Check that assigning the setting to a context variable with
         {% get_settings use_default_site=True as wagtail_settings %} works.
@@ -204,10 +204,7 @@ class TestSiteSettingsJinja(TemplateTestCase):
                 site = context["site"]
             else:
                 site = Site.objects.get(is_default_site=True)
-            request = HttpRequest()
-            request.META["HTTP_HOST"] = site.hostname
-            request.META["SERVER_PORT"] = site.port
-            context["request"] = request
+            context["request"] = get_dummy_request(site=site)
 
         template = self.engine.from_string(string)
         return template.render(context)
@@ -219,7 +216,7 @@ class TestSiteSettingsJinja(TemplateTestCase):
             self.default_settings.title,
         )
 
-    def test_multisite(self):
+    def test_multi_site(self):
         """Check that the correct setting for the current site is returned"""
         context = {"site": self.default_site}
         self.assertEqual(
