@@ -1,7 +1,7 @@
 from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
 
-from wagtail.tests.utils import TestCase
+from wagtail.test.utils import TestCase
 
 
 class TestMigrations(TestCase):
