@@ -1,10 +1,11 @@
 from django.utils.translation import gettext_lazy as _
 from django.utils.translation import ngettext
 
+from wagtail.admin.views.bulk_action.mixins import ReferenceIndexMixin
 from wagtail.documents.views.bulk_actions.document_bulk_action import DocumentBulkAction
 
 
-class DeleteBulkAction(DocumentBulkAction):
+class DeleteBulkAction(ReferenceIndexMixin, DocumentBulkAction):
     display_name = _("Delete")
     action_type = "delete"
     aria_label = _("Delete selected documents")
@@ -25,6 +26,9 @@ class DeleteBulkAction(DocumentBulkAction):
         ).delete()
         return num_parent_objects, 0
 
+    def get_usage_url(self, item):
+        return item.usage_url + "?describe_on_delete=1"
+
     def get_success_message(self, num_parent_objects, num_child_objects):
         return ngettext(
             "%(num_parent_objects)d document has been deleted",
