@@ -2,7 +2,7 @@ from django.core.exceptions import ValidationError
 from django.utils.functional import cached_property
 from django.utils.translation import gettext_lazy as _
 
-from wagtail.core import blocks
+from wagtail import blocks
 from wagtail.embeds.format import embed_to_frontend_html
 
 
@@ -14,12 +14,15 @@ class EmbedValue:
     we want to be able to do {% embed value.url 500 %} without
     doing a redundant fetch of the embed at the default width.
     """
-    def __init__(self, url):
+
+    def __init__(self, url, max_width=None, max_height=None):
         self.url = url
+        self.max_width = max_width
+        self.max_height = max_height
 
     @cached_property
     def html(self):
-        return embed_to_frontend_html(self.url)
+        return embed_to_frontend_html(self.url, self.max_width, self.max_height)
 
     def __str__(self):
         return self.html
@@ -27,14 +30,19 @@ class EmbedValue:
 
 class EmbedBlock(blocks.URLBlock):
     def get_default(self):
+        default = self._evaluate_callable(self.meta.default)
         # Allow specifying the default for an EmbedBlock as either an EmbedValue or a string (or None).
-        if not self.meta.default:
+        if not default:
             return None
-        elif isinstance(self.meta.default, EmbedValue):
-            return self.meta.default
+        elif isinstance(default, EmbedValue):
+            return default
         else:
             # assume default has been passed as a string
-            return EmbedValue(self.meta.default)
+            return EmbedValue(
+                default,
+                getattr(self.meta, "max_width", None),
+                getattr(self.meta, "max_height", None),
+            )
 
     def to_python(self, value):
         # The JSON representation of an EmbedBlock's value is a URL string;
@@ -42,19 +50,23 @@ class EmbedBlock(blocks.URLBlock):
         if not value:
             return None
         else:
-            return EmbedValue(value)
+            return EmbedValue(
+                value,
+                getattr(self.meta, "max_width", None),
+                getattr(self.meta, "max_height", None),
+            )
 
     def get_prep_value(self, value):
         # serialisable value should be a URL string
         if value is None:
-            return ''
+            return ""
         else:
             return value.url
 
     def value_for_form(self, value):
         # the value to be handled by the URLField is a plain URL string (or the empty string)
         if value is None:
-            return ''
+            return ""
         else:
             return value.url
 
@@ -63,12 +75,24 @@ class EmbedBlock(blocks.URLBlock):
         if not value:
             return None
         else:
-            return EmbedValue(value)
+            return EmbedValue(
+                value,
+                getattr(self.meta, "max_width", None),
+                getattr(self.meta, "max_height", None),
+            )
 
     def clean(self, value):
         if isinstance(value, EmbedValue) and not value.html:
             raise ValidationError(_("Cannot find an embed for this URL."))
         return super().clean(value)
 
+    def normalize(self, value):
+        if isinstance(value, EmbedValue):
+            return value if value.url else None
+        elif value:
+            return EmbedValue(value)
+        else:
+            return None
+
     class Meta:
         icon = "media"
