@@ -5,14 +5,6 @@ animoto = {
     ],
 }
 
-app_net = {
-    "endpoint": "https://alpha-api.app.net/oembed",
-    "urls": [
-        r"^https?://alpha\.app\.net/[^#?/]+/post/.+$",
-        r"^https?://photos\.app\.net/[^#?/]+/.+$",
-    ],
-}
-
 audioboom = {
     "endpoint": "https://audioboom.com/publishing/oembed.{format}",
     "urls": [
@@ -21,22 +13,6 @@ audioboom = {
     ],
 }
 
-bambuser = {
-    "endpoint": "http://api.bambuser.com/oembed.{format}",
-    "urls": [
-        r"^http://bambuser\.com/channel/[^#?/]+/broadcast/.+$",
-        r"^http://bambuser\.com/channel/.+$",
-        r"^http://bambuser\.com/v/.+$",
-    ],
-}
-
-blip_tv = {
-    "endpoint": "http://blip.tv/oembed/",
-    "urls": [
-        r"^http://[-\w]+\.blip\.tv/.+$",
-    ],
-}
-
 cacoo = {
     "endpoint": "http://cacoo.com/oembed.{format}",
     "urls": [
@@ -88,6 +64,14 @@ coub = {
     ],
 }
 
+crowdsignal = {
+    "endpoint": "https://api.crowdsignal.com/oembed",
+    "urls": [
+        r"^https?://(?:[-\w]+\.)?polldaddy\.com/.+$",
+        r"^https?://poll.fm/\d+$",
+    ],
+}
+
 crowd_ranking = {
     "endpoint": "http://crowdranking.com/api/oembed.{format}",
     "urls": [
@@ -271,6 +255,13 @@ ifttt = {
     ],
 }
 
+iheart = {
+    "endpoint": "https://www.iheart.com/oembed/",
+    "urls": [
+        r"^https?://(?:www\.)?iheart\.com/.+$",
+    ],
+}
+
 issuu = {
     "endpoint": "https://issuu.com/oembed",
     "urls": [
@@ -384,41 +375,6 @@ pinterest = {
 }
 
 
-polldaddy = {
-    "endpoint": "https://polldaddy.com/oembed/",
-    "urls": [
-        r"^https?://(?:[-\w]+\.)?polldaddy\.com/.+$",
-    ],
-}
-
-
-polleverywhere = {
-    "endpoint": "https://www.polleverywhere.com/services/oembed/",
-    "urls": [
-        r"^https?://www\.polleverywhere\.com/polls/.+$",
-        r"^https?://www\.polleverywhere\.com/multiple_choice_polls/.+$",
-        r"^https?://www\.polleverywhere\.com/free_text_polls/.+$",
-    ],
-}
-
-qik = {
-    "endpoint": "http://qik.com/api/oembed.{format}",
-    "urls": [
-        r"^http://qik\.com/.+$",
-        r"^http://qik\.ly/.+$",
-    ],
-}
-
-
-rdio = {
-    "endpoint": "http://www.rdio.com/api/oembed",
-    "urls": [
-        r"^http://(?:wwww\.)?rdio\.com/people/[^#?/]+/playlists/.+$",
-        r"^http://[-\w]+\.rdio\.com/artist/[^#?/]+/album/.+$",
-    ],
-}
-
-
 reddit = {
     "endpoint": "https://www.reddit.com/oembed",
     "urls": [
@@ -427,13 +383,6 @@ reddit = {
 }
 
 
-revision3 = {
-    "endpoint": "http://revision3.com/api/oembed/",
-    "urls": [
-        r"^http://[-\w]+\.revision3\.com/.+$",
-    ],
-}
-
 roomshare = {
     "endpoint": "https://roomshare.jp/oembed.{format}",
     "urls": [
@@ -520,7 +469,7 @@ speakerdeck = {
 
 
 spotify = {
-    "endpoint": "https://embed.spotify.com/oembed/",
+    "endpoint": "https://open.spotify.com/oembed",
     "urls": [
         r"^https?://open\.spotify\.com/.+$",
         r"^https?://spoti\.fi/.+$",
@@ -559,9 +508,10 @@ twenty_three_hq = {
 }
 
 twitter = {
-    "endpoint": "https://api.twitter.com/1/statuses/oembed.{format}",
+    "endpoint": "https://publish.twitter.com/oembed",
     "urls": [
         r"^https?://twitter\.com/(?:#!)?[^#?/]+/status/.+$",
+        r"^https?://x\.com/(?:#!)?[^#?/]+/status/.+$",
     ],
 }
 
@@ -583,14 +533,6 @@ ustream = {
 }
 
 
-vhx_tv = {
-    "endpoint": "http://vhx.tv/services/oembed.{format}",
-    "urls": [
-        r"^https?://(?:www\.)?vhx\.tv/.+$",
-    ],
-}
-
-
 videojug = {
     "endpoint": "http://www.videojug.com/oembed.{format}",
     "urls": [
@@ -681,16 +623,15 @@ youtube = {
         r"^https?://(?:[-\w]+\.)?youtube\.com/profile.+$",
         r"^https?://(?:[-\w]+\.)?youtube\.com/view_play_list.+$",
         r"^https?://(?:[-\w]+\.)?youtube\.com/playlist.+$",
+        r"^https?://(?:[-\w]+\.)?youtube\.com/shorts/.+$",
+        r"^https?://(?:[-\w]+\.)?youtube\.com/live/.+$",
     ],
 }
 
 
 all_providers = [
     animoto,
-    app_net,
     audioboom,
-    bambuser,
-    blip_tv,
     cacoo,
     chirb,
     circuitlab,
@@ -698,6 +639,7 @@ all_providers = [
     clikthrough,
     collegehumor,
     coub,
+    crowdsignal,
     crowd_ranking,
     dailymile,
     dailymotion,
@@ -720,6 +662,7 @@ all_providers = [
     hulu,
     ifixit,
     ifttt,
+    iheart,
     issuu,
     justin_tv,
     kickstarter,
@@ -734,12 +677,7 @@ all_providers = [
     opera,
     photobucket,
     pinterest,
-    polldaddy,
-    polleverywhere,
-    qik,
-    rdio,
     reddit,
-    revision3,
     roomshare,
     sapo,
     screenr,
@@ -759,7 +697,6 @@ all_providers = [
     twitter,
     urtak,
     ustream,
-    vhx_tv,
     viddler,
     videojug,
     vidyard,
