@@ -1,5 +1,5 @@
 from django.urls import include, path, reverse_lazy
-from django.utils.translation import gettext as _
+from django.utils.translation import gettext_lazy as _
 
 import wagtail.admin.rich_text.editors.draftail.features as draftail_features
 from wagtail import hooks
