@@ -1,10 +1,13 @@
+import datetime
 import json
 
+from django.core.exceptions import ValidationError
 from django.core.serializers.json import DjangoJSONEncoder
-from django.core.validators import MaxLengthValidator
+from django.core.validators import BaseValidator, MaxLengthValidator
 from django.db import models
 from django.utils.encoding import force_str
 from django.utils.functional import cached_property
+from django.utils.translation import gettext_lazy as _
 
 from wagtail.blocks import Block, BlockField, StreamBlock, StreamValue
 from wagtail.blocks.definition_lookup import (
@@ -18,6 +21,25 @@ from wagtail.rich_text import (
 )
 
 
+class NoFutureDateValidator(BaseValidator):
+    """
+    A validator that prevents future dates from being entered.
+
+    Useful for fields that should be in the past or present
+    but never in the future.
+    """
+
+    message = _("Date cannot be in the future.")
+    code = "future_date"
+
+    def __init__(self, message=None):
+        super().__init__(limit_value=None, message=message)
+
+    def __call__(self, value):
+        if value and value > datetime.date.today():
+            raise ValidationError(self.message, code=self.code)
+
+
 class RichTextField(models.TextField):
     def __init__(self, *args, **kwargs):
         # 'editor' and 'features' are popped before super().__init__ has chance to capture them
@@ -146,6 +168,13 @@ class StreamField(models.Field):
 
             block = StreamBlock(child_blocks)
 
+        if not isinstance(block, StreamBlock):
+            raise TypeError(
+                f"The top-level block must be a StreamBlock (got {type(block).__name__}). "
+                "Either pass a StreamBlock instance/class, or a list of block definitions "
+                "as (name, block) tuples."
+            )
+
         block.set_meta_options(self.block_opts)
         return block
 
