@@ -2,8 +2,7 @@ from django.apps import AppConfig
 from django.db.models import ForeignKey
 from django.utils.translation import gettext_lazy as _
 
-from . import checks, get_image_model  # NOQA
-from .signal_handlers import register_signal_handlers
+from . import checks, get_image_model  # NOQA: F401
 
 
 class WagtailImagesAppConfig(AppConfig):
@@ -11,8 +10,11 @@ class WagtailImagesAppConfig(AppConfig):
     label = "wagtailimages"
     verbose_name = _("Wagtail images")
     default_auto_field = "django.db.models.AutoField"
+    default_attrs = {}
 
     def ready(self):
+        from .signal_handlers import register_signal_handlers
+
         register_signal_handlers()
 
         # Set up model forms to use AdminImageChooser for any ForeignKey to the image model
@@ -34,3 +36,13 @@ class WagtailImagesAppConfig(AppConfig):
         register_comparison_class(
             ForeignKey, to=Image, comparison_class=ImageFieldComparison
         )
+
+        from wagtail.admin.ui.fields import register_display_class
+
+        from .components import ImageDisplay
+
+        register_display_class(ForeignKey, to=Image, display_class=ImageDisplay)
+
+        from wagtail.models.reference_index import ReferenceIndex
+
+        ReferenceIndex.register_model(Image)
