@@ -5,7 +5,7 @@ from django.core.checks import Warning, register
 from willow.image import Image
 
 
-@lru_cache()
+@lru_cache(maxsize=None)
 def has_jpeg_support():
     wagtail_jpg = os.path.join(os.path.dirname(__file__), "check_files", "wagtail.jpg")
     succeeded = True
@@ -13,13 +13,13 @@ def has_jpeg_support():
     with open(wagtail_jpg, "rb") as f:
         try:
             Image.open(f)
-        except IOError:
+        except OSError:
             succeeded = False
 
     return succeeded
 
 
-@lru_cache()
+@lru_cache(maxsize=None)
 def has_png_support():
     wagtail_png = os.path.join(os.path.dirname(__file__), "check_files", "wagtail.png")
     succeeded = True
@@ -27,7 +27,7 @@ def has_png_support():
     with open(wagtail_png, "rb") as f:
         try:
             Image.open(f)
-        except IOError:
+        except OSError:
             succeeded = False
 
     return succeeded
