@@ -1,7 +1,11 @@
+import hashlib
 import unittest
+from io import StringIO
+from unittest import mock
 
 from django.conf import settings
 from django.contrib.auth.models import Group, Permission
+from django.core import checks, management
 from django.core.cache import caches
 from django.core.files import File
 from django.core.files.storage import Storage, default_storage, storages
@@ -12,6 +16,7 @@ from django.test import SimpleTestCase, TestCase, TransactionTestCase, override_
 from django.urls import reverse
 from willow.image import Image as WillowImage
 
+from wagtail.images.exceptions import InvalidFilterSpecError
 from wagtail.images.models import (
     Filter,
     Picture,
@@ -22,7 +27,13 @@ from wagtail.images.models import (
 )
 from wagtail.images.rect import Rect
 from wagtail.models import Collection, GroupCollectionPermission, Page, ReferenceIndex
+from wagtail.search.backends import get_search_backend
+from wagtail.test.dummy_external_storage import (
+    DummyExternalStorage,
+    DummyExternalStorageFile,
+)
 from wagtail.test.testapp.models import (
+    CustomRendition,
     EventPage,
     EventPageCarouselItem,
     ReimportedImageModel,
@@ -41,6 +52,16 @@ class CustomStorage(Storage):
     pass
 
 
+class AnotherDummyExternalStorage(DummyExternalStorage):
+    def _open(self, name, mode="rb"):
+        # External storages has their own File type
+        return AnotherDummyExternalStorageFile(open(self.wrapped.path(name), mode))
+
+
+class AnotherDummyExternalStorageFile(DummyExternalStorageFile):
+    pass
+
+
 class TestImage(TestCase):
     def setUp(self):
         # Create an image for running tests on
@@ -118,6 +139,18 @@ class TestImage(TestCase):
     def test_is_stored_locally_with_external_storage(self):
         self.assertFalse(self.image.is_stored_locally())
 
+    @override_settings(
+        DEFAULT_FILE_STORAGE="wagtail.test.dummy_external_storage.DummyExternalStorage"
+    )
+    def test_reopen_based_on_storage_that_is_dynamically_set(self):
+        self.image.file.close()
+        # Simulate the case is which the storage is set dynamically
+        with mock.patch.object(
+            self.image.file, "storage", AnotherDummyExternalStorage()
+        ):
+            with self.image.open_file() as file:
+                self.assertIsInstance(file, AnotherDummyExternalStorageFile)
+
     def test_get_file_size(self):
         file_size = self.image.get_file_size()
         self.assertIsInstance(file_size, int)
@@ -129,9 +162,11 @@ class TestImage(TestCase):
             self.image.get_file_size()
 
     def test_file_hash(self):
-        self.assertEqual(
-            self.image.get_file_hash(), "4dd0211870e130b7e1690d2ec53c499a54a48fef"
-        )
+        with self.image.file.open() as f:
+            loaded_hash = hashlib.sha1(f.read()).hexdigest()
+        # Ensure get_file_hash() (which does the hash by chunks) returns the
+        # same hash as the one calculated by loading the file into memory.
+        self.assertEqual(self.image.get_file_hash(), loaded_hash)
 
     def test_get_suggested_focal_point_svg(self):
         """
@@ -516,10 +551,45 @@ class TestRenditions(TestCase):
             title="Test image",
             file=get_test_image_file(),
         )
+        self.svg_image = Image.objects.create(
+            title="Test SVG image",
+            file=get_test_image_file_svg(),
+        )
 
     def test_get_rendition_model(self):
         self.assertIs(Image.get_rendition_model(), Rendition)
 
+    def test_stock_rendition_contains_unique_together_constraint(self):
+        self.assertEqual([], Rendition.check())
+
+    def test_custom_rendition_may_have_unique_constraint(self):
+        self.assertEqual([], CustomRendition.check())
+
+    def test_custom_rendition_without_unique_constraint_raises_error(self):
+        custom_rendition_constraints = CustomRendition._meta.constraints
+        try:
+            CustomRendition._meta.constraints = []
+            errors = CustomRendition.check()
+        finally:
+            CustomRendition._meta.constraints = custom_rendition_constraints
+
+        self.assertEqual(CustomRendition._meta.unique_together, ())
+        self.assertEqual(len(errors), 1)
+        self.assertIsInstance(errors[0], checks.Error)
+        self.assertEqual(errors[0].id, "wagtailimages.E001")
+        self.assertEqual(errors[0].obj, CustomRendition)
+        self.assertEqual(
+            errors[0].msg,
+            "Custom rendition model 'tests.CustomRendition' must include a unique "
+            "constraint on the 'image', 'filter_spec', and 'focal_point_key' fields.",
+        )
+        self.assertEqual(
+            errors[0].hint,
+            "Add models.UniqueConstraint(fields=("
+            '"image", "filter_spec", "focal_point_key"), '
+            'name="unique_rendition") to CustomRendition.Meta.constraints.',
+        )
+
     def test_minification(self):
         rendition = self.image.get_rendition("width-400")
 
@@ -867,6 +937,10 @@ class TestRenditions(TestCase):
             rendition.background_position_style, "background-position: 15% 41%;"
         )
 
+        # Individual background position properties
+        self.assertEqual(rendition.background_position_x, "15%")
+        self.assertEqual(rendition.background_position_y, "41%")
+
     def test_background_position_style_default(self):
         # Generate a rendition that's half the size of the original
         rendition = self.image.get_rendition("width-320")
@@ -875,6 +949,10 @@ class TestRenditions(TestCase):
             rendition.background_position_style, "background-position: 50% 50%;"
         )
 
+        # Individual background position properties
+        self.assertEqual(rendition.background_position_x, "50%")
+        self.assertEqual(rendition.background_position_y, "50%")
+
     @override_settings()
     def test_rendition_storage_setting_absent(self):
         del settings.WAGTAILIMAGES_RENDITION_STORAGE
@@ -912,6 +990,107 @@ class TestRenditions(TestCase):
             get_rendition_storage(), storages[settings.WAGTAILIMAGES_RENDITION_STORAGE]
         )
 
+    def test_image_get_rendition_preserve_svg(self):
+        image_rendition_1 = self.image.get_rendition(
+            "width-400|bgcolor-000|format-jpeg|preserve-svg"
+        )
+        # no directives stripped except 'preserve-svg'
+        self.assertEqual(
+            image_rendition_1.filter_spec, "width-400|bgcolor-000|format-jpeg"
+        )
+
+        image_rendition_2 = self.image.get_rendition(
+            "width-400|bgcolor-000|format-jpeg"
+        )
+        # preserve-svg has no effect and thus the existing rendition will be returned
+        self.assertEqual(image_rendition_1, image_rendition_2)
+
+        # same behaviour when passing a Filter object rather than a string
+        image_rendition_3 = self.image.get_rendition(
+            Filter("width-400|bgcolor-000|format-jpeg|preserve-svg")
+        )
+        self.assertEqual(image_rendition_1, image_rendition_3)
+        image_rendition_4 = self.image.get_rendition(
+            Filter("width-400|bgcolor-000|format-jpeg")
+        )
+        self.assertEqual(image_rendition_1, image_rendition_4)
+
+        # preserve-svg with no other directive raises error
+        with self.assertRaises(InvalidFilterSpecError):
+            self.image.get_rendition("preserve-svg")
+
+    def test_svg_get_rendition_preserve_svg(self):
+        # rasterize directives stripped
+        svg_rendition_1 = self.svg_image.get_rendition(
+            "width-400|bgcolor-000|format-jpeg|preserve-svg"
+        )
+        self.assertEqual(svg_rendition_1.filter_spec, "width-400")
+
+        # str filter & Filter with preserve-svg produce same result
+        svg_rendition_2 = self.svg_image.get_rendition(
+            Filter("width-400|bgcolor-000|format-jpeg|preserve-svg")
+        )
+        self.assertEqual(svg_rendition_1, svg_rendition_2)
+
+        # fallback to 'original' if no SVG-safe directives remain
+        svg_rendition_3 = self.svg_image.get_rendition(
+            Filter("bgcolor-000|format-jpeg|preserve-svg")
+        )
+        self.assertEqual(svg_rendition_3.filter_spec, "original")
+
+        # no rasterize directives, no preserve-svg
+        svg_rendition_4 = self.svg_image.get_rendition(Filter("width-400"))
+        self.assertEqual(svg_rendition_4.filter_spec, "width-400")
+
+        # no rasterize directives, preserve-svg has no affect
+        svg_rendition_5 = self.svg_image.get_rendition(Filter("width-400|preserve-svg"))
+        self.assertEqual(svg_rendition_4, svg_rendition_5)
+
+        # has raterize directives but no preserve-svg raises error
+        with self.assertRaises(AttributeError):
+            self.svg_image.get_rendition(Filter("width-400|bgcolor-000|format-jpeg"))
+
+    def test_image_get_renditions_preserve_svg(self):
+        renditions = self.image.get_renditions(
+            "width-400|bgcolor-000|format-jpeg|preserve-svg",
+            "width-200|bgcolor-000|format-jpeg|preserve-svg",
+        )
+        filename1 = get_test_image_filename(
+            self.image, "width-400.bgcolor-000.format-jpeg"
+        )
+        filename2 = get_test_image_filename(
+            self.image, "width-200.bgcolor-000.format-jpeg"
+        )
+
+        # no directives stripped except 'preserve-svg'
+        # (which is stripped from both the dictionary key and the resulting rendition)
+        self.assertEqual(
+            renditions["width-400|bgcolor-000|format-jpeg"].filter_spec,
+            "width-400|bgcolor-000|format-jpeg",
+        )
+        self.assertEqual(renditions["width-400|bgcolor-000|format-jpeg"].url, filename1)
+
+        self.assertEqual(
+            renditions["width-200|bgcolor-000|format-jpeg"].filter_spec,
+            "width-200|bgcolor-000|format-jpeg",
+        )
+        self.assertEqual(renditions["width-200|bgcolor-000|format-jpeg"].url, filename2)
+
+    def test_svg_get_renditions_preserve_svg(self):
+        renditions = self.svg_image.get_renditions(
+            "width-400|bgcolor-000|format-jpeg|preserve-svg",
+            "width-200|bgcolor-000|format-jpeg|preserve-svg",
+        )
+        filename1 = get_test_image_filename(self.svg_image, "width-400")
+        filename2 = get_test_image_filename(self.svg_image, "width-200")
+
+        # all non-SVG-safe directives stripped (from both the dictionary key and the resulting rendition)
+        self.assertEqual(renditions["width-400"].filter_spec, "width-400")
+        self.assertEqual(renditions["width-400"].url, filename1)
+
+        self.assertEqual(renditions["width-200"].filter_spec, "width-200")
+        self.assertEqual(renditions["width-200"].url, filename2)
+
 
 @override_settings(
     CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
@@ -1096,22 +1275,20 @@ class TestIssue573(TestCase):
         image.get_rendition("fill-800x600")
 
 
-@override_settings(_WAGTAILSEARCH_FORCE_AUTO_UPDATE=["elasticsearch"])
 class TestIssue613(WagtailTestUtils, TestCase):
-    def get_elasticsearch_backend(self):
-        from django.conf import settings
-
-        from wagtail.search.backends import get_search_backend
-
+    def setUp(self):
         if "elasticsearch" not in settings.WAGTAILSEARCH_BACKENDS:
             raise unittest.SkipTest("No elasticsearch backend active")
 
-        return get_search_backend("elasticsearch")
-
-    def setUp(self):
-        self.search_backend = self.get_elasticsearch_backend()
         self.login()
 
+        management.call_command(
+            "update_index",
+            backend_name="elasticsearch",
+            stdout=StringIO(),
+            chunk_size=50,
+        )
+
     def add_image(self, **params):
         post_data = {
             "title": "Test image",
@@ -1161,36 +1338,62 @@ class TestIssue613(WagtailTestUtils, TestCase):
         return image
 
     def test_issue_613_on_add(self):
-        # Reset the search index
-        self.search_backend.reset_index()
-        self.search_backend.add_type(Image)
+        # Note to future developer troubleshooting this test...
+        # This test previously started by calling self.search_backend.reset_index(), but that was evidently redundant because
+        # this was broken on Elasticsearch prior to the fix in
+        # https://github.com/wagtail/wagtailsearch/commit/53a98169bccc3cef5b234944037f2b3f78efafd4 .
+        # If this turns out to be necessary after all, you might want to compare how wagtail.tests.test_page_search.PageSearchTests does it.
 
-        # Add an image with some tags
-        image = self.add_image(tags="hello")
-        self.search_backend.refresh_index()
+        backend_conf = settings.WAGTAILSEARCH_BACKENDS["elasticsearch"].copy()
+        backend_conf["AUTO_UPDATE"] = True
+        with self.settings(
+            WAGTAILSEARCH_BACKENDS={
+                "elasticsearch": backend_conf,
+            }
+        ):
+            search_backend = get_search_backend("elasticsearch")
 
-        # Search for it by tag
-        results = self.search_backend.search("hello", Image)
+            # Add an image with some tags
+            image = self.add_image(tags="hello")
 
-        # Check
-        self.assertEqual(len(results), 1)
-        self.assertEqual(results[0].id, image.id)
+            # TODO: remove this when https://github.com/kaedroho/django-modelsearch/pull/40 is merged and released
+            search_backend.refresh_indexes()
+
+            # Search for it by tag
+            results = search_backend.search("hello", Image)
+
+            # Check
+            self.assertEqual(len(results), 1)
+            self.assertEqual(results[0].id, image.id)
 
     def test_issue_613_on_edit(self):
-        # Reset the search index
-        self.search_backend.reset_index()
-        self.search_backend.add_type(Image)
+        # Note to future developer troubleshooting this test...
+        # This test previously started by calling self.search_backend.reset_index(), but that was evidently redundant because
+        # this was broken on Elasticsearch prior to the fix in
+        # https://github.com/wagtail/wagtailsearch/commit/53a98169bccc3cef5b234944037f2b3f78efafd4 .
+        # If this turns out to be necessary after all, you might want to compare how wagtail.tests.test_page_search.PageSearchTests does it.
 
-        # Add an image with some tags
-        image = self.edit_image(tags="hello")
-        self.search_backend.refresh_index()
+        backend_conf = settings.WAGTAILSEARCH_BACKENDS["elasticsearch"].copy()
+        backend_conf["AUTO_UPDATE"] = True
+        with self.settings(
+            WAGTAILSEARCH_BACKENDS={
+                "elasticsearch": backend_conf,
+            }
+        ):
+            search_backend = get_search_backend("elasticsearch")
 
-        # Search for it by tag
-        results = self.search_backend.search("hello", Image)
+            # Add an image with some tags
+            image = self.edit_image(tags="hello")
 
-        # Check
-        self.assertEqual(len(results), 1)
-        self.assertEqual(results[0].id, image.id)
+            # TODO: remove this when https://github.com/kaedroho/django-modelsearch/pull/40 is merged and released
+            search_backend.refresh_indexes()
+
+            # Search for it by tag
+            results = search_backend.search("hello", Image)
+
+            # Check
+            self.assertEqual(len(results), 1)
+            self.assertEqual(results[0].id, image.id)
 
 
 class TestIssue312(TestCase):
