@@ -15,6 +15,14 @@ from django.views import View
 from wagtail.admin import messages
 from wagtail.admin.auth import PermissionPolicyChecker
 from wagtail.admin.filters import BaseMediaFilterSet
+from wagtail.admin.ui.tables import (
+    BaseColumn,
+    BulkActionsCheckboxColumn,
+    Column,
+    DateColumn,
+    TitleColumn,
+    UsageCountColumn,
+)
 from wagtail.admin.utils import get_valid_next_url_from_request, set_query_params
 from wagtail.admin.views import generic
 from wagtail.images import get_image_model
@@ -23,7 +31,7 @@ from wagtail.images.forms import URLGeneratorForm, get_image_form
 from wagtail.images.models import Filter, SourceImageIOError
 from wagtail.images.permissions import permission_policy
 from wagtail.images.utils import generate_signature
-from wagtail.models import Site
+from wagtail.models import ReferenceIndex, Site
 
 permission_checker = PermissionPolicyChecker(permission_policy)
 
@@ -48,6 +56,8 @@ class IndexView(generic.IndexView):
         "-title": gettext_lazy("Title: (Z -> A)"),
         "file_size": gettext_lazy("File size: (low to high)"),
         "-file_size": gettext_lazy("File size: (high to low)"),
+        "usage_count": gettext_lazy("Usage count: (low to high)"),
+        "-usage_count": gettext_lazy("Usage count: (high to low)"),
     }
     default_ordering = "-created_at"
     context_object_name = "images"
@@ -65,13 +75,18 @@ class IndexView(generic.IndexView):
     edit_url_name = "wagtailimages:edit"
     template_name = "wagtailimages/images/index.html"
     results_template_name = "wagtailimages/images/index_results.html"
-    columns = []
 
     def get_paginate_by(self, queryset):
         return getattr(settings, "WAGTAILIMAGES_INDEX_PAGE_SIZE", 30)
 
     def get_valid_orderings(self):
-        return self.ORDERING_OPTIONS
+        orderings = self.ORDERING_OPTIONS.copy()
+        if self.is_searching:
+            # Ordering by usage count not currently available when searching,
+            # due to https://github.com/wagtail/django-modelsearch/issues/51
+            orderings.pop("usage_count", None)
+            orderings.pop("-usage_count", None)
+        return orderings.keys()
 
     def get_base_queryset(self):
         # Get images (filtered by user permission)
@@ -82,8 +97,22 @@ class IndexView(generic.IndexView):
             .select_related("collection")
             .prefetch_renditions("max-165x165")
         )
+
+        if self.needs_usage_count_subquery:
+            # Annotate usage_count on the whole queryset to allow ordering/filtering
+            # (On some databases, this can be slow when there are many objects)
+            images = images.annotate(
+                usage_count=ReferenceIndex.usage_count_subquery(self.model)
+            )
+
         return images
 
+    @cached_property
+    def needs_usage_count_subquery(self):
+        return self.ordering in ["usage_count", "-usage_count"] or (
+            self.is_filtering and "usage_count" in self.filters.form.cleaned_data
+        )
+
     @cached_property
     def current_collection(self):
         # Upon validation, the cleaned data is a Collection instance
@@ -108,6 +137,19 @@ class IndexView(generic.IndexView):
             next_url += "?" + request_query_string
         return next_url
 
+    def decorate_paginated_queryset(self, object_list):
+        if self.needs_usage_count_subquery:
+            # Already annotated in get_base_queryset
+            return object_list
+
+        # Use a separate, more efficient query that only gets usage counts for
+        # objects on the current page
+        # See https://github.com/wagtail/wagtail/issues/13561
+        counts = ReferenceIndex.get_count_references_to_in_bulk(list(object_list))
+        for obj in object_list:
+            obj.usage_count = counts.get(obj, 0)
+        return object_list
+
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
 
@@ -117,11 +159,76 @@ class IndexView(generic.IndexView):
                 "current_collection": self.current_collection,
                 "current_ordering": self.ordering,
                 "ORDERING_OPTIONS": self.ORDERING_OPTIONS,
+                "layout": self.layout,
+                "object_list": self.decorate_paginated_queryset(context["object_list"]),
             }
         )
 
         return context
 
+    @cached_property
+    def layout(self):
+        return self.request.GET.get("layout", "grid")
+
+    @cached_property
+    def columns(self):
+        if self.layout == "grid":
+            return []
+        else:
+            columns = [
+                BulkActionsColumn("bulk_actions"),
+                ImagePreviewColumn(
+                    "preview",
+                    label=_("Preview"),
+                    accessor="image",
+                    classname="image-preview",
+                ),
+                TitleColumnWithFilename(
+                    "title",
+                    label=_("Title"),
+                    sort_key="title",
+                    get_url=self.get_edit_url,
+                    width="35%",
+                    classname="title-with-filename",
+                ),
+                Column("collection", label=_("Collection"), accessor="collection.name"),
+                DateColumn(
+                    "created_at",
+                    label=_("Created"),
+                    sort_key="created_at",
+                ),
+                UsageCountColumn(
+                    "usage_count",
+                    label=_("Usage"),
+                    # Ordering by usage count not currently available when searching,
+                    # due to https://github.com/wagtail/django-modelsearch/issues/51
+                    sort_key="usage_count" if not self.is_searching else None,
+                    width="16%",
+                ),
+            ]
+
+            return columns
+
+
+class BulkActionsColumn(BulkActionsCheckboxColumn):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, obj_type="image", **kwargs)
+
+    def get_header_context_data(self, parent_context):
+        context = super().get_header_context_data(parent_context)
+        parent = parent_context.get("current_collection")
+        if parent:
+            context["parent"] = parent.id
+        return context
+
+
+class ImagePreviewColumn(BaseColumn):
+    cell_template_name = "wagtailimages/images/image_preview_column_cell.html"
+
+
+class TitleColumnWithFilename(TitleColumn):
+    cell_template_name = "wagtailimages/images/title_column_cell.html"
+
 
 class EditView(generic.EditView):
     permission_policy = permission_policy
