@@ -8,16 +8,16 @@ from django.db.models.signals import pre_save
 from django.dispatch import receiver
 from django.utils import translation
 from django.utils.encoding import force_str
+from django.utils.translation import gettext_lazy as _
 from modelcluster.fields import ParentalKey
 
+from wagtail.actions.copy_for_translation import CopyForTranslationAction
 from wagtail.coreutils import (
     get_content_languages,
     get_supported_content_language_variant,
 )
 from wagtail.signals import pre_validate_delete
 
-from .copying import _copy
-
 
 def pk(obj):
     if isinstance(obj, models.Model):
@@ -56,7 +56,7 @@ class Locale(models.Model):
     @classmethod
     def get_default(cls):
         """
-        Returns the default Locale based on the site's LANGUAGE_CODE setting
+        Returns the default Locale based on the site's ``LANGUAGE_CODE`` setting.
         """
         return cls.objects.get_for_language(settings.LANGUAGE_CODE)
 
@@ -82,18 +82,108 @@ class Locale(models.Model):
     def language_code_is_valid(self):
         return self.language_code in get_content_languages()
 
-    def get_display_name(self):
-        return get_content_languages().get(self.language_code)
+    def get_display_name(self) -> str:
+        try:
+            return get_content_languages()[self.language_code]
+        except KeyError:
+            pass
+        try:
+            return self.language_name
+        except KeyError:
+            pass
+
+        return self.language_code
 
     def __str__(self):
-        return force_str(self.get_display_name() or self.language_code)
+        return force_str(self.get_display_name())
+
+    def _get_language_info(self) -> dict[str, str]:
+        return translation.get_language_info(self.language_code)
+
+    @property
+    def language_info(self):
+        return translation.get_language_info(self.language_code)
+
+    @property
+    def language_name(self):
+        """
+        Uses data from ``django.conf.locale`` to return the language name in
+        English. For example, if the object's ``language_code`` were ``"fr"``,
+        the return value would be ``"French"``.
+
+        Raises ``KeyError`` if ``django.conf.locale`` has no information
+        for the object's ``language_code`` value.
+        """
+        return self.language_info["name"]
+
+    @property
+    def language_name_local(self):
+        """
+        Uses data from ``django.conf.locale`` to return the language name in
+        the language itself. For example, if the ``language_code`` were
+        ``"fr"`` (French), the return value would be ``"français"``.
+
+        Raises ``KeyError`` if ``django.conf.locale`` has no information
+        for the object's ``language_code`` value.
+        """
+        return self.language_info["name_local"]
+
+    @property
+    def language_name_localized(self):
+        """
+        Uses data from ``django.conf.locale`` to return the language name in
+        the currently active language. For example, if ``language_code`` were
+        ``"fr"`` (French), and the active language were ``"da"`` (Danish), the
+        return value would be ``"Fransk"``.
+
+        Raises ``KeyError`` if ``django.conf.locale`` has no information
+        for the object's ``language_code`` value.
+
+        """
+        return translation.gettext(self.language_name)
+
+    @property
+    def is_bidi(self) -> bool:
+        """
+        Returns a boolean indicating whether the language is bi-directional.
+        """
+        return self.language_code in settings.LANGUAGES_BIDI
+
+    @property
+    def is_default(self) -> bool:
+        """
+        Returns a boolean indicating whether this object is the default locale.
+        """
+        try:
+            return self.language_code == get_supported_content_language_variant(
+                settings.LANGUAGE_CODE
+            )
+        except LookupError:
+            return False
+
+    @property
+    def is_active(self) -> bool:
+        """
+        Returns a boolean indicating whether this object is the currently active locale.
+        """
+        try:
+            return self.language_code == get_supported_content_language_variant(
+                translation.get_language()
+            )
+        except LookupError:
+            return self.is_default
 
 
 class TranslatableMixin(models.Model):
     translation_key = models.UUIDField(default=uuid.uuid4, editable=False)
     locale = models.ForeignKey(
-        Locale, on_delete=models.PROTECT, related_name="+", editable=False
+        Locale,
+        on_delete=models.PROTECT,
+        related_name="+",
+        editable=False,
+        verbose_name=_("locale"),
     )
+    locale.wagtail_reference_index_ignore = True
 
     class Meta:
         abstract = True
@@ -101,29 +191,56 @@ class TranslatableMixin(models.Model):
 
     @classmethod
     def check(cls, **kwargs):
-        errors = super(TranslatableMixin, cls).check(**kwargs)
-        is_translation_model = cls.get_translation_model() is cls
-
-        # Raise error if subclass has removed the unique_together constraint
-        # No need to check this on multi-table-inheritance children though as it only needs to be applied to
+        errors = super().check(**kwargs)
+        # No need to check on multi-table-inheritance children as it only needs to be applied to
         # the table that has the translation_key/locale fields
-        if (
-            is_translation_model
-            and ("translation_key", "locale") not in cls._meta.unique_together
-        ):
+        is_translation_model = cls.get_translation_model() is cls
+        if not is_translation_model:
+            return errors
+
+        unique_constraint_fields = ("translation_key", "locale")
+
+        has_unique_constraint = any(
+            isinstance(constraint, models.UniqueConstraint)
+            and set(constraint.fields) == set(unique_constraint_fields)
+            for constraint in cls._meta.constraints
+        )
+
+        has_unique_together = unique_constraint_fields in cls._meta.unique_together
+
+        # Raise error if subclass has removed constraints
+        if not (has_unique_constraint or has_unique_together):
             errors.append(
                 checks.Error(
-                    "{0}.{1} is missing a unique_together constraint for the translation key and locale fields".format(
-                        cls._meta.app_label, cls.__name__
-                    ),
-                    hint="Add ('translation_key', 'locale') to {}.Meta.unique_together".format(
-                        cls.__name__
+                    "%s is missing a UniqueConstraint for the fields: %s."
+                    % (cls._meta.label, unique_constraint_fields),
+                    hint=(
+                        "Add models.UniqueConstraint(fields=%s, "
+                        "name='unique_translation_key_locale_%s_%s') to %s.Meta.constraints."
+                        % (
+                            unique_constraint_fields,
+                            cls._meta.app_label,
+                            cls._meta.model_name,
+                            cls.__name__,
+                        )
                     ),
                     obj=cls,
                     id="wagtailcore.E003",
                 )
             )
 
+        # Raise error if subclass has both UniqueConstraint and unique_together
+        if has_unique_constraint and has_unique_together:
+            errors.append(
+                checks.Error(
+                    "%s should not have both UniqueConstraint and unique_together for: %s."
+                    % (cls._meta.label, unique_constraint_fields),
+                    hint="Remove unique_together in favor of UniqueConstraint.",
+                    obj=cls,
+                    id="wagtailcore.E003",
+                )
+            )
+
         return errors
 
     @property
@@ -154,6 +271,9 @@ class TranslatableMixin(models.Model):
         Note: This will return translations that are in draft. If you want to exclude
         these, use the ``.localized`` attribute.
         """
+        if not getattr(settings, "WAGTAIL_I18N_ENABLED", False):
+            return self
+
         try:
             locale = Locale.get_active()
         except (LookupError, Locale.DoesNotExist):
@@ -189,7 +309,7 @@ class TranslatableMixin(models.Model):
         """
         Finds the translation in the specified locale.
 
-        If there is no translation in that locale, this returns None.
+        If there is no translation in that locale, this returns ``None``.
         """
         try:
             return self.get_translation(locale)
@@ -204,22 +324,17 @@ class TranslatableMixin(models.Model):
             self.get_translations(inclusive=True).filter(locale_id=pk(locale)).exists()
         )
 
-    def copy_for_translation(self, locale):
+    def copy_for_translation(self, locale, exclude_fields=None):
         """
         Creates a copy of this instance with the specified locale.
 
         Note that the copy is initially unsaved.
         """
-        translated, child_object_map = _copy(self)
-        translated.locale = locale
-
-        # Update locale on any translatable child objects as well
-        # Note: If this is not a subclass of ClusterableModel, child_object_map will always be '{}'
-        for (_child_relation, _old_pk), child_object in child_object_map.items():
-            if isinstance(child_object, TranslatableMixin):
-                child_object.locale = locale
-
-        return translated
+        return CopyForTranslationAction(
+            self,
+            locale,
+            exclude_fields=exclude_fields,
+        ).execute()
 
     def get_default_locale(self):
         """
@@ -267,7 +382,7 @@ def bootstrap_translatable_model(model, locale):
     This function populates the "translation_key", and "locale" fields on model instances that were created
     before wagtail-localize was added to the site.
 
-    This can be called from a data migration, or instead you could use the "boostrap_translatable_models"
+    This can be called from a data migration, or instead you could use the "bootstrap_translatable_models"
     management command.
     """
     for instance in (
@@ -363,7 +478,7 @@ def set_locale_on_new_instance(sender, instance, **kwargs):
         return
 
     # If this is a fixture load, use the global default Locale
-    # as the page tree is probably in an flux
+    # as the page tree is probably in flux
     if kwargs["raw"]:
         instance.locale = Locale.get_default()
         return
