@@ -1,6 +1,6 @@
 from django.db import models
 
-from wagtail.core.models import Page
+from wagtail.models import Page
 
 
 class HomePage(Page):
