@@ -4,7 +4,7 @@ from wagtail.snippets.models import get_snippet_models
 
 
 def get_permission_name(action, model):
-    return "%s.%s" % (
+    return "{}.{}".format(
         model._meta.app_label,
         get_permission_codename(action, model._meta),
     )
@@ -19,15 +19,19 @@ def user_can_edit_snippet_type(user, model):
     return False
 
 
-def user_can_edit_snippets(user):
+def user_can_access_snippets(user, models=None):
     """
-    true if user has 'add', 'change' or 'delete' permission
-    on any model registered as a snippet type
+    true if user has 'add', 'change', 'delete', or 'view' permission
+    on any model registered as a snippet type - or if a `models` list
+    is passed, any of those models
     """
-    snippet_models = get_snippet_models()
+    if models is None:
+        models = get_snippet_models()
 
-    for model in snippet_models:
-        if user_can_edit_snippet_type(user, model):
+    for model in models:
+        if model.snippet_viewset.permission_policy.user_has_any_permission(
+            user, {"add", "change", "delete", "view"}
+        ):
             return True
 
     return False
