@@ -13,6 +13,7 @@ from django.db import models
 from wagtail.admin.auth import permission_denied  # noqa: F401
 from wagtail.admin.panels import FieldPanel
 from wagtail.admin.views.generic import chooser as chooser_views  # noqa: F401
+from wagtail.search import index
 
 from .fields import ConvertedValueField
 
@@ -55,7 +56,7 @@ class CustomUserManager(BaseUserManager):
         return self._create_user(username, email, password, True, True, **extra_fields)
 
 
-class CustomUser(AbstractBaseUser, PermissionsMixin):
+class CustomUser(index.Indexed, AbstractBaseUser, PermissionsMixin):
     identifier = ConvertedValueField(primary_key=True)
     username = models.CharField(max_length=100, unique=True)
     email = models.EmailField(max_length=255, blank=True)
@@ -81,3 +82,15 @@ class CustomUser(AbstractBaseUser, PermissionsMixin):
         FieldPanel("first_name"),
         FieldPanel("last_name"),
     ]
+
+    search_fields = [
+        index.SearchField("country"),
+        index.SearchField("first_name"),
+        index.SearchField("last_name"),
+        index.AutocompleteField("country"),
+        index.AutocompleteField("first_name"),
+        index.AutocompleteField("last_name"),
+        # The PK must be added as FilterField to allow searching
+        # and filtering by group
+        index.FilterField("identifier"),
+    ]
