@@ -3,7 +3,6 @@ import os
 import pickle
 import tempfile
 import unittest
-import warnings
 from io import BytesIO
 from pathlib import Path
 
@@ -31,7 +30,6 @@ from wagtail.coreutils import (
     string_to_ascii,
 )
 from wagtail.models import Page, Site
-from wagtail.utils.deprecation import RemovedInWagtail70Warning
 from wagtail.utils.file import hash_filelike
 from wagtail.utils.templates import template_is_overridden
 from wagtail.utils.utils import deep_update, flatten_choices
@@ -53,7 +51,7 @@ class TestStringToAscii(TestCase):
     def test_string_to_ascii(self):
         test_cases = [
             ("30 \U0001d5c4\U0001d5c6/\U0001d5c1", "30 km/h"),
-            ("\u5317\u4EB0", "BeiJing"),
+            ("\u5317\u4eb0", "BeiJing"),
             ("ぁ あ ぃ い ぅ う ぇ", "a a i i u u e"),
             (
                 "Ա Բ Գ Դ Ե Զ Է Ը Թ Ժ Ի Լ Խ Ծ Կ Հ Ձ Ղ Ճ Մ Յ Ն",
@@ -657,23 +655,3 @@ class TestFlattenChoices(SimpleTestCase):
                 "unknown": "Unknown",
             },
         )
-
-
-class TestWidgetWithScript(TestCase):
-    def test_deprecation(self):
-        message = "The usage of `WidgetWithScript` hook is deprecated. Use external scripts instead."
-
-        with unittest.mock.patch("warnings.warn", wraps=warnings.warn) as warn_mock:
-            with self.assertWarnsMessage(RemovedInWagtail70Warning, message):
-                from wagtail.utils.widgets import WidgetWithScript
-
-                class MyWidget(WidgetWithScript):
-                    pass
-
-        # Make sure warn was called with stacklevel=3, so the actual caller
-        # that imports WidgetWithScript is shown in the warning message
-        warn_mock.assert_called_with(
-            message,
-            category=RemovedInWagtail70Warning,
-            stacklevel=3,
-        )
