@@ -1,5 +1,4 @@
 from itertools import groupby
-from warnings import warn
 
 from django import forms
 from django.conf import settings
@@ -23,22 +22,11 @@ from wagtail.models import (
     GroupPagePermission,
     Page,
 )
-from wagtail.utils.deprecation import RemovedInWagtail70Warning
 
 User = get_user_model()
 
 # The standard fields each user model is expected to have, as a minimum.
 standard_fields = {"email", "first_name", "last_name", "is_superuser", "groups"}
-# Custom fields
-if hasattr(settings, "WAGTAIL_USER_CUSTOM_FIELDS"):
-    custom_fields = set(settings.WAGTAIL_USER_CUSTOM_FIELDS)
-    warn(
-        "The `WAGTAIL_USER_CUSTOM_FIELDS` setting is deprecated. Use a custom "
-        "`UserViewSet` subclass and override `get_form_class()` instead.",
-        RemovedInWagtail70Warning,
-    )
-else:
-    custom_fields = set()
 
 
 class UsernameForm(forms.ModelForm):
@@ -110,9 +98,7 @@ class UserForm(UsernameForm):
     is_superuser = forms.BooleanField(
         label=_("Administrator"),
         required=False,
-        help_text=_(
-            "Administrators have full access to manage any object " "or setting."
-        ),
+        help_text=_("Administrators have full access to manage any object or setting."),
     )
 
     def __init__(self, *args, **kwargs):
@@ -207,7 +193,7 @@ class UserForm(UsernameForm):
 class UserCreationForm(UserForm):
     class Meta:
         model = User
-        fields = {User.USERNAME_FIELD} | standard_fields | custom_fields
+        fields = {User.USERNAME_FIELD} | standard_fields
         widgets = {"groups": forms.CheckboxSelectMultiple}
 
 
@@ -224,7 +210,7 @@ class UserEditForm(UserForm):
 
     class Meta:
         model = User
-        fields = {User.USERNAME_FIELD, "is_active"} | standard_fields | custom_fields
+        fields = {User.USERNAME_FIELD, "is_active"} | standard_fields
         widgets = {"groups": forms.CheckboxSelectMultiple}
 
 
