@@ -1,11 +1,13 @@
+from django.urls import reverse
 from django.utils.translation import gettext_lazy as _
 from django.utils.translation import ngettext
 
+from wagtail.admin.views.bulk_action.mixins import ReferenceIndexMixin
 from wagtail.users.utils import user_can_delete_user
 from wagtail.users.views.bulk_actions.user_bulk_action import UserBulkAction
 
 
-class DeleteBulkAction(UserBulkAction):
+class DeleteBulkAction(ReferenceIndexMixin, UserBulkAction):
     display_name = _("Delete")
     action_type = "delete"
     aria_label = _("Delete selected users")
@@ -26,6 +28,12 @@ class DeleteBulkAction(UserBulkAction):
         model.objects.filter(pk__in=[obj.pk for obj in objects]).delete()
         return len(objects), 0
 
+    def get_usage_url(self, item):
+        return (
+            reverse("wagtailusers_users:usage", args=(item.pk,))
+            + "?describe_on_delete=1"
+        )
+
     def get_success_message(self, num_parent_objects, num_child_objects):
         return ngettext(
             "%(num_parent_objects)d user has been deleted",
