@@ -1,13 +1,16 @@
 from django.contrib.auth.models import Group
-from django.urls import re_path
+from django.urls import reverse
+from django.utils.functional import cached_property
+from django.utils.translation import gettext
 from django.utils.translation import gettext_lazy as _
 
 from wagtail import hooks
 from wagtail.admin.ui.tables import TitleColumn
+from wagtail.admin.utils import set_query_params
 from wagtail.admin.views import generic
 from wagtail.admin.viewsets.model import ModelViewSet
+from wagtail.admin.widgets.button import HeaderButton
 from wagtail.users.forms import GroupForm, GroupPagePermissionFormSet
-from wagtail.users.views.users import index
 
 _permission_panel_classes = None
 
@@ -45,11 +48,30 @@ class PermissionPanelFormsMixin:
             for cls in get_permission_panel_classes()
         ]
 
+    def process_form(self):
+        form = self.get_form()
+        permission_panels = self.get_permission_panel_forms()
+        if form.is_valid() and all(panel.is_valid() for panel in permission_panels):
+            response = self.form_valid(form)
+
+            for panel in permission_panels:
+                panel.save()
+
+            return response
+        else:
+            return self.form_invalid(form)
+
     def get_context_data(self, **kwargs):
         if "permission_panels" not in kwargs:
             kwargs["permission_panels"] = self.get_permission_panel_forms()
 
-        return super().get_context_data(**kwargs)
+        context = super().get_context_data(**kwargs)
+
+        # Add js/css media from the formsets to the existing media
+        for panel in context["permission_panels"]:
+            context["media"] += panel.media
+
+        return context
 
 
 class IndexView(generic.IndexView):
@@ -59,8 +81,6 @@ class IndexView(generic.IndexView):
     search_fields = ["name"]
     context_object_name = "groups"
     paginate_by = 20
-    ordering = ["name"]
-    default_ordering = "name"
 
     columns = [
         TitleColumn(
@@ -71,17 +91,10 @@ class IndexView(generic.IndexView):
         ),
     ]
 
-    def get_template_names(self):
-        if self.request.headers.get("x-requested-with") == "XMLHttpRequest":
-            return ["wagtailusers/groups/results.html"]
-        else:
-            return ["wagtailusers/groups/index.html"]
-
 
 class CreateView(PermissionPanelFormsMixin, generic.CreateView):
     page_title = _("Add group")
-    success_message = _("Group '{0}' created.")
-    template_name = "wagtailusers/groups/create.html"
+    success_message = _("Group '%(object)s' created.")
 
     def post(self, request, *args, **kwargs):
         """
@@ -90,37 +103,26 @@ class CreateView(PermissionPanelFormsMixin, generic.CreateView):
         """
         # Create an object now so that the permission panel forms have something to link them against
         self.object = Group()
-
-        form = self.get_form()
-        permission_panels = self.get_permission_panel_forms()
-        if form.is_valid() and all(panel.is_valid() for panel in permission_panels):
-            response = self.form_valid(form)
-
-            for panel in permission_panels:
-                panel.save()
-
-            return response
-        else:
-            return self.form_invalid(form)
-
-    def get_context_data(self, **kwargs):
-        context = super().get_context_data(**kwargs)
-
-        # add a 'form_media' variable for the collected js/css media from the form and all formsets
-        form_media = context["form"].media
-        for panel in context["permission_panels"]:
-            form_media += panel.media
-        context["form_media"] = form_media
-
-        return context
+        return self.process_form()
 
 
 class EditView(PermissionPanelFormsMixin, generic.EditView):
-    success_message = _("Group '{0}' updated.")
+    success_message = _("Group '%(object)s' updated.")
     error_message = _("The group could not be saved due to errors.")
-    delete_item_label = _("Delete group")
     context_object_name = "group"
-    template_name = "wagtailusers/groups/edit.html"
+
+    @cached_property
+    def header_buttons(self):
+        return [
+            HeaderButton(
+                gettext("View users in this group"),
+                url=set_query_params(
+                    reverse("wagtailusers_users:index"),
+                    {"group": self.object.pk},
+                ),
+                icon_name="user",
+            )
+        ]
 
     def post(self, request, *args, **kwargs):
         """
@@ -128,55 +130,43 @@ class EditView(PermissionPanelFormsMixin, generic.EditView):
         POST variables and then check if it's valid.
         """
         self.object = self.get_object()
-
-        form = self.get_form()
-        permission_panels = self.get_permission_panel_forms()
-        if form.is_valid() and all(panel.is_valid() for panel in permission_panels):
-            response = self.form_valid(form)
-
-            for panel in permission_panels:
-                panel.save()
-
-            return response
-        else:
-            return self.form_invalid(form)
-
-    def get_context_data(self, **kwargs):
-        context = super().get_context_data(**kwargs)
-
-        # add a 'form_media' variable for the collected js/css media from the form and all formsets
-        form_media = context["form"].media
-        for panel in context["permission_panels"]:
-            form_media += panel.media
-        context["form_media"] = form_media
-
-        return context
+        return self.process_form()
 
 
 class DeleteView(generic.DeleteView):
-    success_message = _("Group '{0}' deleted.")
+    success_message = _("Group '%(object)s' deleted.")
     page_title = _("Delete group")
     confirmation_message = _("Are you sure you want to delete this group?")
-    template_name = "wagtailusers/groups/confirm_delete.html"
 
 
 class GroupViewSet(ModelViewSet):
     icon = "group"
     model = Group
+    ordering = ["name"]
+    add_to_reference_index = False
+    menu_name = "groups"
+    menu_label = _("Groups")
+    menu_order = 601
+    add_to_settings_menu = True
 
     index_view_class = IndexView
     add_view_class = CreateView
     edit_view_class = EditView
     delete_view_class = DeleteView
 
-    @property
-    def users_view(self):
-        return index
+    # Copying requires prepopulating the formsets with copies of the existing
+    # permissions, which is not currently supported.
+    copy_view_enabled = False
+
+    template_prefix = "wagtailusers/groups/"
+
+    def get_common_view_kwargs(self, **kwargs):
+        return super().get_common_view_kwargs(
+            **{
+                "usage_url_name": None,
+                **kwargs,
+            }
+        )
 
     def get_form_class(self, for_update=False):
         return GroupForm
-
-    def get_urlpatterns(self):
-        return super().get_urlpatterns() + [
-            re_path(r"(\d+)/users/$", self.users_view, name="users"),
-        ]
